@@ -18,8 +18,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: dataeditdialog.py 1427 2010-10-23 21:03:33Z jeremysanders $
-
 """Module for implementing dialog box for viewing/editing data."""
 
 import bisect
@@ -28,6 +26,7 @@ import veusz.qtall as qt4
 
 import veusz.document as document
 import veusz.utils as utils
+from veusz.qtwidgets.datasetbrowser import DatasetBrowser
 from veuszdialog import VeuszDialog
 
 # register function to dataset class to edit dataset
@@ -75,13 +74,11 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
             if index.row() == len(data):
                 return qt4.QVariant()
 
+            # convert data to QVariant
             d = data[index.row()]
-            if isinstance(d, basestring):
-                return qt4.QVariant(d)
-            else:
-                # value needs converting to float as QVariant doesn't
-                # support numpy numeric types
-                return qt4.QVariant(float(d))
+            return ds.uiDataItemToQVariant(d)
+
+        # empty entry
         return qt4.QVariant()
 
     def headerData(self, section, orientation, role):
@@ -147,7 +144,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
 
         # update if conversion okay
         try:
-            val = ds.convertToDataItem( value.toString() )
+            val = ds.uiConvertToDataItem( value.toString() )
         except ValueError:
             return False
 
@@ -166,6 +163,8 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsname = datasetname
+        self.connect(document, qt4.SIGNAL('sigModified'),
+                     self.slotDocumentModified)
 
     def rowCount(self, parent):
         ds = self.document.data[self.dsname].data
@@ -210,65 +209,39 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
                 return qt4.QVariant( '%g' % val )
 
         return qt4.QVariant()
-
-class DatasetListModel(qt4.QStringListModel):
-    def __init__(self, parent, document):
-        dsnames = document.data.keys()
-        dsnames.sort()
-        qt4.QStringListModel.__init__(self, dsnames, parent)
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotDocumentModified)
-        self.document = document
-
-    def datasetName(self, index):
-        """Get name at index."""
-        return unicode(self.stringList()[index.row()])
-        
-    @property
-    def datasets(self):
-        """Get list of datasets."""
-        return [unicode(x) for x in self.stringList()]
+    
+    def flags(self, index):
+        """Update flags to say that items are editable."""
+        if not index.isValid():
+            return qt4.Qt.ItemIsEnabled
+        else:
+            return qt4.QAbstractTableModel.flags(self, index) | qt4.Qt.ItemIsEditable
 
     def slotDocumentModified(self):
-        """Update list when document modified."""
-        dslist = self.datasets
-        old = set(dslist)
-        new = set(self.document.data.keys())
+        """Called when document modified."""
+        self.emit( qt4.SIGNAL('layoutChanged()') )
 
-        # dslist used to keep track of changes
-        # add new entries in appropriate (sorted) place
-        for a in new-old:
-            i = bisect.bisect_left(dslist, a)
-            dslist.insert(i, a)
-            self.insertRows(i, 1)
-            qt4.QStringListModel.setData(
-                self, self.index(i, 0), qt4.QVariant(a) )
+    def setData(self, index, value, role):
+        """Called to set the data."""
 
-        # remove entries no longer there
-        for d in old-new:
-            i = dslist.index(d)
-            del dslist[i]
-            self.removeRows(i, 1)
+        if not index.isValid() or role != qt4.Qt.EditRole:
+            return False
 
-    def setData(self, index, value, role=qt4.Qt.EditRole):
-        """Called to rename a dataset."""
+        ds = self.document.data[self.dsname]
+        row = ds.data.shape[0]-index.row()-1
+        col = index.column()
 
-        if index.isValid() and role == qt4.Qt.EditRole:
-            name = self.datasetName(index)
-            newname = unicode( value.toString() )
+        # update if conversion okay
+        try:
+            val = ds.uiConvertToDataItem( value.toString() )
+        except ValueError:
+            return False
 
-            if not utils.validateDatasetName(newname):
-                return False
+        op = document.OperationDatasetSetVal2D(
+            self.dsname, row, col, val)
+        self.document.applyOperation(op)
+        return True
 
-            self.document.applyOperation(
-                document.OperationDatasetRename(name, newname))
-            self.emit(qt4.SIGNAL(
-                    'dataChanged(const QModelIndex &, const QModelIndex &'),
-                      index, index)
-            return True
-
-        return False
-    
 class DataEditDialog(VeuszDialog):
     """Dialog for editing and rearranging data sets."""
     
@@ -277,13 +250,8 @@ class DataEditDialog(VeuszDialog):
         self.document = document
 
         # set up dataset list
-        self.dslistmodel = DatasetListModel(self, document)
-
-        #self.modelproxy = qt4.QSortFilterProxyModel(self)
-        #self.modelproxy.setSourceModel(self.dslistmodel)
-        #self.modelproxy.setDynamicSortFilter(True)
-
-        self.datasetlistview.setModel(self.dslistmodel)
+        self.dsbrowser = DatasetBrowser(document, parent, parent)
+        self.splitter.insertWidget(0, self.dsbrowser)
 
         # actions for data table
         for text, slot in (
@@ -297,8 +265,8 @@ class DataEditDialog(VeuszDialog):
         self.datatableview.setContextMenuPolicy( qt4.Qt.ActionsContextMenu )
 
         # layout edit dialog improvement
-        self.splitter.setStretchFactor(0, 1)
-        self.splitter.setStretchFactor(1, 3)
+        self.splitter.setStretchFactor(0, 3)
+        self.splitter.setStretchFactor(1, 4)
 
         # don't want text to look editable or special
         self.linkedlabel.setFrameShape(qt4.QFrame.NoFrame)
@@ -308,16 +276,19 @@ class DataEditDialog(VeuszDialog):
         self.connect(document, qt4.SIGNAL('sigModified'),
                      self.slotDocumentModified)
 
-        # receive change in selection
-        self.connect(self.datasetlistview.selectionModel(),
-                     qt4.SIGNAL('selectionChanged(const QItemSelection &, const QItemSelection &)'),
-                     self.slotDatasetSelected)
+        # select first item, if any or initialise if none
+        if len(self.document.data) > 0:
+            self.selectDataset( sorted(self.document.data.keys())[0] )
 
-        # select first item (phew)
-        if self.dslistmodel.rowCount() > 0:
-            self.datasetlistview.selectionModel().select(
-                self.dslistmodel.createIndex(0, 0),
-                qt4.QItemSelectionModel.Select)
+        #if self.dslistmodel.rowCount() > 0:
+        #    self.datasetlistview.selectionModel().select(
+        #        self.dslistmodel.createIndex(0, 0),
+        #        qt4.QItemSelectionModel.Select)
+        #else:
+        #    self.slotDatasetSelected(None, None)
+
+        self.connect(self.dsbrowser.navtree, qt4.SIGNAL("selecteditem"),
+                     self.slotDatasetSelected)
 
         # connect buttons
         for btn, slot in ( (self.deletebutton, self.slotDatasetDelete),
@@ -332,49 +303,52 @@ class DataEditDialog(VeuszDialog):
         # menu for new button
         self.newmenu = qt4.QMenu()
         for text, slot in ( ('Numerical dataset', self.slotNewNumericalDataset),
-                            ('Text dataset', self.slotNewTextDataset) ):
+                            ('Text dataset', self.slotNewTextDataset),
+                            ('Date/time dataset', self.slotNewDateDataset) ):
             a = self.newmenu.addAction(text)
-            self.connect(a, qt4.SIGNAL('activated()'), slot)
+            self.connect(a, qt4.SIGNAL('triggered()'), slot)
         self.newbutton.setMenu(self.newmenu)
 
-    def slotDatasetSelected(self, current, deselected):
+    def slotDatasetSelected(self, name):
         """Called when a new dataset is selected."""
 
         # FIXME: Make readonly models readonly!!
-        indexes = current.indexes()
-        if len(indexes) == 0:
-            self.datatableview.setModel(None)
-            return
+        model = None
+        if name:
+            # get selected dataset
+            ds = self.document.data[name]
 
-        name = self.dslistmodel.datasetName(indexes[0])
-        ds = self.document.data[name]
+            # make model for dataset
+            if ds.dimensions == 1:
+                model = DatasetTableModel1D(self, self.document, name)
+            elif ds.dimensions == 2:
+                model = DatasetTableModel2D(self, self.document, name)
 
-        if ds.dimensions == 1:
-            model = DatasetTableModel1D(self, self.document, name)
-        elif ds.dimensions == 2:
-            model = DatasetTableModel2D(self, self.document, name)
+        # disable context menu if no menu
+        for a in self.datatableview.actions():
+            a.setEnabled(model is not None)
 
-        self.datatableview.setModel(model)
-            
+        self.datatableview.setModel(model)    
         self.setUnlinkState()
 
     def setUnlinkState(self):
         """Enable the unlink button correctly."""
         # get dataset
         dsname = self.getSelectedDataset()
-
         try:
             ds = self.document.data[dsname]
+            unlink = ds.canUnlink()
+            linkinfo = ds.linkedInformation()
         except KeyError:
-            return
-
-        # linked dataset
-        unlink = ds.canUnlink()
-        readonly = not unlink
+            ds = None
+            unlink = False
+            linkinfo = ""
 
         self.editbutton.setVisible(type(ds) in recreate_register)
         self.unlinkbutton.setEnabled(unlink)
-        self.linkedlabel.setText( ds.linkedInformation() )
+        self.linkedlabel.setText(linkinfo)
+        self.deletebutton.setEnabled(ds is not None)
+        self.duplicatebutton.setEnabled(ds is not None)
 
     def slotDocumentModified(self):
         """Set unlink status when document modified."""
@@ -382,34 +356,27 @@ class DataEditDialog(VeuszDialog):
 
     def getSelectedDataset(self):
         """Return the selected dataset."""
-        selitems = self.datasetlistview.selectionModel().selection().indexes()
-        if len(selitems) != 0:
-            return self.dslistmodel.datasetName(selitems[0])
-        else:
-            return None
-        
+        return self.dsbrowser.navtree.getSelectedDataset()
+
+    def selectDataset(self, dsname):
+        """Select dataset with name given."""
+        self.dsbrowser.navtree.selectDataset(dsname)
+        self.slotDatasetSelected(dsname)
+
     def slotDatasetDelete(self):
         """Delete selected dataset."""
-
-        datasetname = self.getSelectedDataset()
-        if datasetname is not None:
-            row = self.datasetlistview.selectionModel(
-                ).selection().indexes()[0].row()
-
-            self.document.applyOperation(
-                document.OperationDatasetDelete(datasetname))
+        self.document.applyOperation(
+            document.OperationDatasetDelete(self.getSelectedDataset()) )
 
     def slotDatasetUnlink(self):
         """Allow user to remove link to file or other datasets."""
-
         datasetname = self.getSelectedDataset()
-        if datasetname is not None:
-            d = self.document.data[datasetname]
-            if d.linked is not None:
-                op = document.OperationDatasetUnlinkFile(datasetname)
-            else:
-                op = document.OperationDatasetUnlinkRelation(datasetname)
-            self.document.applyOperation(op)
+        d = self.document.data[datasetname]
+        if d.linked is not None:
+            op = document.OperationDatasetUnlinkFile(datasetname)
+        else:
+            op = document.OperationDatasetUnlinkRelation(datasetname)
+        self.document.applyOperation(op)
 
     def slotDatasetDuplicate(self):
         """Duplicate selected dataset."""
@@ -488,9 +455,13 @@ class DataEditDialog(VeuszDialog):
         self.newDataset( document.Dataset(data=[0.]) )
 
     def slotNewTextDataset(self):
-        """Add new value dataset."""
+        """Add new text dataset."""
         self.newDataset( document.DatasetText(data=['']) )
 
+    def slotNewDateDataset(self):
+        """Add new date dataset."""
+        self.newDataset( document.DatasetDateTime(data=[]) )
+
     def newDataset(self, ds):
         """Add new dataset to document."""
         # get a name for dataset
@@ -504,3 +475,5 @@ class DataEditDialog(VeuszDialog):
         # add new dataset
         self.document.applyOperation(
             document.OperationDatasetSet(name, ds))
+
+        self.dsbrowser.selectDataset(name)
