@@ -18,8 +18,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: importdialog.py 1471 2010-12-10 22:18:24Z jeremysanders $
-
 """Module for implementing dialog boxes for importing data in Veusz."""
 
 import os.path
@@ -69,6 +67,15 @@ class ImportTab(qt4.QWidget):
         """Secondary check (after preview) for enabling import button."""
         return True
 
+    def isFiletypeSupported(self, ftype):
+        """Is the filetype supported by this tab?"""
+        return False
+
+    def useFiletype(self, ftype):
+        """If the tab can do something with the selected filetype,
+        update itself."""
+        pass
+
 class ImportTabStandard(ImportTab):
     """Standard import format tab."""
 
@@ -144,10 +151,14 @@ class ImportTabStandard(ImportTab):
         if len(lines) != 0:
             lines.append('')
             
-        lines += self.dialog.retnDatasetInfo(dsnames)
+        lines += self.dialog.retnDatasetInfo(dsnames, linked, filename)
 
         self.previewedit.setPlainText( '\n'.join(lines) )
 
+    def isFiletypeSupported(self, ftype):
+        """Is the filetype supported by this tab?"""
+        return ftype in ('.dat', '.txt')
+
 class ImportTabCSV(ImportTab):
     """For importing data from CSV files."""
 
@@ -167,6 +178,13 @@ class ImportTabCSV(ImportTab):
         self.csvdelimitercombo.default = [
             ',', '{tab}', '{space}', '|', ':', ';']
         self.csvtextdelimitercombo.default = ['"', "'"]
+        self.csvdatefmtcombo.default = [
+            'YYYY-MM-DD|T|hh:mm:ss',
+            'DD/MM/YY| |hh:mm:ss',
+            'M/D/YY| |hh:mm:ss'
+            ]
+        self.csvnumfmtcombo.defaultlist = ['System', 'English', 'European']
+        self.csvheadermodecombo.defaultlist = ['Multiple', '1st row', 'None']
 
     def reset(self):
         """Reset controls."""
@@ -174,6 +192,12 @@ class ImportTabCSV(ImportTab):
         self.csvtextdelimitercombo.setEditText('"')
         self.csvdirectioncombo.setCurrentIndex(0)
         self.csvignorehdrspin.setValue(0)
+        self.csvignoretopspin.setValue(0)
+        self.csvblanksdatacheck.setChecked(False)
+        self.csvnumfmtcombo.setCurrentIndex(0)
+        self.csvdatefmtcombo.setEditText(
+            document.ParamsCSV.defaults['dateformat'])
+        self.csvheadermodecombo.setCurrentIndex(0)
 
     def slotHelp(self):
         """Asked for help."""
@@ -212,7 +236,7 @@ class ImportTabCSV(ImportTab):
             return False
 
         try:
-            reader = utils.UnicodeCSVReader( open(filename),
+            reader = utils.UnicodeCSVReader( filename,
                                              delimiter=delimiter,
                                              quotechar=textdelimiter,
                                              encoding=encoding )
@@ -256,20 +280,38 @@ class ImportTabCSV(ImportTab):
         except UnicodeEncodeError:
             return
 
+        numericlocale = ( str(qt4.QLocale().name()),
+                          'en_US',
+                          'de_DE' )[self.csvnumfmtcombo.currentIndex()]
         headerignore = self.csvignorehdrspin.value()
-        op = document.OperationDataImportCSV(filename, readrows=inrows,
-                                             prefix=prefix, suffix=suffix,
-                                             linked=linked,
-                                             delimiter=delimiter,
-                                             textdelimiter=textdelimiter,
-                                             headerignore=headerignore,
-                                             encoding=encoding)
-        
+        rowsignore = self.csvignoretopspin.value()
+        blanksaredata = self.csvblanksdatacheck.isChecked()
+        dateformat = unicode(self.csvdatefmtcombo.currentText())
+        headermode = ('multi', '1st', 'none')[
+            self.csvheadermodecombo.currentIndex()]
+
+        # create import parameters and operation objects
+        params = document.ParamsCSV(
+            filename,
+            readrows=inrows,
+            encoding=encoding,
+            delimiter=delimiter,
+            textdelimiter=textdelimiter,
+            headerignore=headerignore,
+            rowsignore=rowsignore,
+            blanksaredata=blanksaredata,
+            numericlocale=numericlocale,
+            dateformat=dateformat,
+            headermode=headermode,
+            dsprefix=prefix, dssuffix=suffix
+            )
+        op = document.OperationDataImportCSV(params, linked=linked)
+
         # actually import the data
         dsnames = doc.applyOperation(op)
-        
-        # what datasets were imported
-        lines = self.dialog.retnDatasetInfo(dsnames)
+
+        # update output, showing what datasets were imported
+        lines = self.dialog.retnDatasetInfo(dsnames, linked, filename)
 
         t = self.previewtablecsv
         t.verticalHeader().hide()
@@ -283,6 +325,10 @@ class ImportTabCSV(ImportTab):
             item = qt4.QTableWidgetItem(l)
             t.setItem(i, 0, item)
 
+    def isFiletypeSupported(self, ftype):
+        """Is the filetype supported by this tab?"""
+        return ftype in ('.tsv', '.csv')
+
 class ImportTab2D(ImportTab):
     """Tab for importing from a 2D data file."""
 
@@ -401,7 +447,7 @@ class ImportTabFITS(ImportTab):
         self.connect( self.fitshdulist, qt4.SIGNAL('itemSelectionChanged()'),
                       self.slotFitsUpdateCombos )
         self.connect( self.fitsdatasetname,
-                      qt4.SIGNAL('textChanged(const QString&)'),
+                      qt4.SIGNAL('editTextChanged(const QString&)'),
                       self.dialog.enableDisableImport )
         self.connect( self.fitsdatacolumn,
                       qt4.SIGNAL('currentIndexChanged(int)'),
@@ -482,7 +528,7 @@ class ImportTabFITS(ImportTab):
             except AttributeError:
                 # this is an image
                 naxis = header['NAXIS']
-                if naxis ==2:
+                if naxis == 1 or naxis == 2:
                     data = ['image']
                 else:
                     data = ['invalidimage']
@@ -592,11 +638,24 @@ class ImportTabFITS(ImportTab):
         self.fitsimportstatus.setText("Imported dataset '%s'" % name)
         qt4.QTimer.singleShot(2000, self.fitsimportstatus.clear)
 
+    def isFiletypeSupported(self, ftype):
+        """Is the filetype supported by this tab?"""
+        return ftype in ('.fit', '.fits')
+
 class ImportTabPlugins(ImportTab):
     """Tab for importing using a plugin."""
 
     resource = 'import_plugins.ui'
 
+    def __init__(self, importdialog, promote=None):
+        """Initialise dialog. importdialog is the import dialog itself.
+
+        If promote is set to a name of a plugin, it is promoted to its own tab
+        """
+        ImportTab.__init__(self, importdialog)
+        self.promote = promote
+        self.plugininstance = None
+
     def loadUi(self):
         """Load the user interface."""
         ImportTab.loadUi(self)
@@ -611,13 +670,22 @@ class ImportTabPlugins(ImportTab):
         self.fields = []
 
         # load previous plugin
-        if 'import_plugin' in setting.settingdb:
-            try:
-                idx = names.index(setting.settingdb['import_plugin'])
-                self.pluginType.setCurrentIndex(idx)
-            except ValueError:
-                pass
+        idx = -1
+        if self.promote is None:
+            if 'import_plugin' in setting.settingdb:
+                try:
+                    idx = names.index(setting.settingdb['import_plugin'])
+                except ValueError:
+                    pass
+        else:
+            # set the correct entry for the plugin
+            idx = names.index(self.promote)
+            # then hide the widget so it can't be changed
+            self.pluginchoicewidget.hide()
 
+        if idx >= 0:
+            self.pluginType.setCurrentIndex(idx)
+        
         self.pluginChanged(-1)
 
     def getPluginFields(self):
@@ -636,12 +704,23 @@ class ImportTabPlugins(ImportTab):
             idx = names.index(selname)
         except ValueError:
             return None
-        return plugins.importpluginregistry[idx]
+
+        p = plugins.importpluginregistry[idx]
+        if isinstance(p, type):
+            # this is a class, rather than an object
+            if not isinstance(self.plugininstance, p):
+                # create new instance, if required
+                self.plugininstance = p()
+            return self.plugininstance
+        else:
+            # backward compatibility with old API
+            return p
 
     def pluginChanged(self, index):
         """Update controls based on index."""
         plugin = self.getSelectedPlugin()
-        setting.settingdb['import_plugin'] = plugin.name
+        if self.promote is None:
+            setting.settingdb['import_plugin'] = plugin.name
 
         # delete old controls
         layout = self.pluginParams.layout()
@@ -669,12 +748,13 @@ class ImportTabPlugins(ImportTab):
         """Preview using plugin."""
 
         # check file exists
-        try:
-            f = open(filename, 'r')
-            f.close()
-        except IOError:
-            self.pluginPreview.setPlainText('')
-            return False
+        if filename != '{clipboard}':
+            try:
+                f = open(filename, 'r')
+                f.close()
+            except IOError:
+                self.pluginPreview.setPlainText('')
+                return False
 
         # get the plugin selected
         plugin = self.getSelectedPlugin()
@@ -702,17 +782,49 @@ class ImportTabPlugins(ImportTab):
             plugin, filename, linked=linked, encoding=encoding,
             prefix=prefix, suffix=suffix, **params)
         try:
-            results = doc.applyOperation(op)
+            datasets, customs = doc.applyOperation(op)
         except plugins.ImportPluginException, ex:
             self.pluginPreview.setPlainText( unicode(ex) )
             return
 
         out = ['Imported data for datasets:']
-        for ds in results:
+        for ds in datasets:
             out.append( doc.data[ds].description(showlinked=False) )
+        if customs:
+            out.append('')
+            out.append('Set custom definitions:')
+            out += customs
 
         self.pluginPreview.setPlainText('\n'.join(out))
 
+    def isFiletypeSupported(self, ftype):
+        """Is the filetype supported by this tab?"""
+
+        if self.promote is None:
+            # look through list of supported plugins to check filetypes
+            inany = False
+            for p in plugins.importpluginregistry:
+                if ftype in p.file_extensions:
+                    inany = True
+            return inany
+        else:
+            # find plugin class and check filetype
+            for p in plugins.importpluginregistry:
+                if p.name == self.promote:
+                    return ftype in p.file_extensions
+
+    def useFiletype(self, ftype):
+        """Select the plugin corresponding to the filetype."""
+
+        if self.promote is None:
+            plugin = None
+            for p in plugins.importpluginregistry:
+                if ftype in p.file_extensions:
+                    plugin = p.name
+            idx = self.pluginType.findText(plugin, qt4.Qt.MatchExactly)
+            self.pluginType.setCurrentIndex(idx)
+            self.pluginChanged(-1)
+
 class ImportDialog(VeuszDialog):
     """Dialog box for importing data.
     See ImportTab classes above which actually do the work of importing
@@ -739,6 +851,13 @@ class ImportDialog(VeuszDialog):
             ):
             w = tabclass(self)
             self.methodtab.addTab(w, tabname)
+
+        # add promoted plugins
+        for p in plugins.importpluginregistry:
+            if p.promote_tab is not None:
+                w = ImportTabPlugins(self, promote=p.name)
+                self.methodtab.addTab(w, p.promote_tab)
+
         self.connect( self.methodtab, qt4.SIGNAL('currentChanged(int)'),
                       self.slotUpdatePreview )
 
@@ -783,6 +902,14 @@ class ImportDialog(VeuszDialog):
         self.encodingcombo.defaultlist = utils.encodings
         self.encodingcombo.defaultval = 'utf_8'
 
+        # load icon for clipboard
+        self.clipbutton.setIcon( utils.getIcon('kde-clipboard') )
+        self.connect(qt4.QApplication.clipboard(), qt4.SIGNAL('dataChanged()'),
+                     self.updateClipPreview)
+        self.connect(
+            self.clipbutton, qt4.SIGNAL("clicked()"), self.slotClipButtonClicked)
+        self.updateClipPreview()
+
     def slotBrowseClicked(self):
         """Browse for a data file."""
 
@@ -802,6 +929,29 @@ class ImportDialog(VeuszDialog):
         if fd.exec_() == qt4.QDialog.Accepted:
             ImportDialog.dirname = fd.directory().absolutePath()
             self.filenameedit.replaceAndAddHistory( fd.selectedFiles()[0] )
+            self.guessImportTab()
+
+    def guessImportTab(self):
+        """Guess import tab based on filename."""
+        filename = unicode( self.filenameedit.text() )
+
+        fname, ftype = os.path.splitext(filename)
+        # strip off any gz, bz2 extensions to get real extension
+        while ftype.lower() in ('gz', 'bz2'):
+            fname, ftype = os.path.splitext(fname)
+        ftype = ftype.lower()
+
+        # examine from left to right
+        # promoted plugins come after plugins
+        idx = -1
+        for i in xrange(self.methodtab.count()):
+            w = self.methodtab.widget(i)
+            if w.isFiletypeSupported(ftype):
+                idx = i
+
+        if idx >= 0:
+            self.methodtab.setCurrentIndex(idx)
+            self.methodtab.widget(idx).useFiletype(ftype)
 
     def slotUpdatePreview(self, *args):
         """Update preview window when filename or tab changed."""
@@ -838,15 +988,20 @@ class ImportDialog(VeuszDialog):
         """Do the importing"""
 
         filename = unicode( self.filenameedit.text() )
-        filename = os.path.abspath(filename)
         linked = self.linkcheckbox.isChecked()
         encoding = str(self.encodingcombo.currentText())
+        if filename == '{clipboard}':
+            linked = False
+        else:
+            # normalise filename
+            filename = os.path.abspath(filename)
 
         # import according to tab selected
         importtab = self.methodtab.currentWidget()
         prefix, suffix = self.getPrefixSuffix(filename)
         try:
             qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
+            self.document.suspendUpdates()
             importtab.doImport(self.document, filename, linked, encoding,
                                prefix, suffix)
             qt4.QApplication.restoreOverrideCursor()
@@ -856,8 +1011,9 @@ class ImportDialog(VeuszDialog):
             # show exception dialog
             d = exceptiondialog.ExceptionDialog(sys.exc_info(), self)
             d.exec_()
+        self.document.enableUpdates()
 
-    def retnDatasetInfo(self, dsnames):
+    def retnDatasetInfo(self, dsnames, linked, filename):
         """Return a list of information for the dataset names given."""
         
         lines = ['Imported data for datasets:']
@@ -867,10 +1023,6 @@ class ImportDialog(VeuszDialog):
             # build up description
             lines.append( ' %s' % ds.description(showlinked=False) )
 
-        linked = self.linkcheckbox.isChecked()
-        filename = unicode( self.filenameedit.text() )
-        filename = os.path.abspath(filename)
-
         # whether the data were linked
         if linked:
             lines.append('')
@@ -899,3 +1051,14 @@ class ImportDialog(VeuszDialog):
 
         importtab = self.methodtab.currentWidget()
         importtab.reset()
+
+    def slotClipButtonClicked(self):
+        """Clicked clipboard button."""
+        self.filenameedit.setText("{clipboard}")
+
+    def updateClipPreview(self):
+        """Clipboard contents changed, so update preview if showing clipboard."""
+
+        filename = unicode(self.filenameedit.text())
+        if filename == '{clipboard}':
+            self.slotUpdatePreview()
