@@ -18,8 +18,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: __init__.py 1224 2010-05-13 21:00:22Z jeremysanders $
-
 from widgetfactory import *
 from doc import *
 from datasets import *
@@ -30,3 +28,6 @@ from operations import *
 from capture import *
 from mime import *
 from dataset_histo import *
+from painthelper import *
+from export import Export
+from readcsv import ParamsCSV
