@@ -19,14 +19,13 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: commandinterface.py 1471 2010-12-10 22:18:24Z jeremysanders $
-
 """
 Module supplies the command interface used in the program, and for
 external programs.
 """
 
 import os.path
+import traceback
 
 import veusz.qtall as qt4
 import veusz.setting as setting
@@ -37,6 +36,8 @@ import datasets
 import operations
 import dataset_histo
 import mime
+import export
+import readcsv
 
 class CommandInterface(qt4.QObject):
     """Class provides command interface."""
@@ -66,6 +67,7 @@ class CommandInterface(qt4.QObject):
         'NodeType',
         'ReloadData',
         'Remove',
+        'RemoveCustom',
         'Rename',
         'ResolveReference',
         'Set',
@@ -128,12 +130,16 @@ class CommandInterface(qt4.QObject):
         self.verbose = v
 
     def Add(self, widgettype, *args, **args_opt):
-        """Add a graph to the graph with the type given.
+        """Add a widget to the widget with the type given.
 
         optional argument:
           widget: widget path to place widget in
 
         The optional arguments are sent to construct the widget.
+
+        If autoadd is True (the default), then any sub widgets
+        associated with the widgets are added automatically (e.g. axes
+        for graph widgets).
         """
 
         at = self.currentwidget
@@ -145,19 +151,58 @@ class CommandInterface(qt4.QObject):
         w = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Added a graph of type '%s' (%s)" % (type, w.userdescription)
+            print "Added a widget of type '%s' (%s)" % (type, w.userdescription)
 
         return w.name
 
-    def AddCustom(self, name, ctype, val):
+    def AddCustom(self, ctype, name, val, mode='appendalways'):
         """Add a custom definition for evaluation of expressions.
+	This can define a constant (can be in terms of other
+	constants), a function of 1 or more variables, or a function
+	imported from an external python module.
 
-        name is name of constant, or function(params)
-        ctype is constant or function
-        val is definition."""
+        ctype is "constant", "function" or "import".
+
+        name is name of constant, or "function(x, y, ...)" or module
+        name.
+
+        val is definition for constant or function (both are
+        _strings_), or is a list of symbols for a module (comma
+        separated items in a string).
+
+        if mode is 'appendalways', the custom value is appended to the
+        end of the list even if there is one with the same name. If
+        mode is 'replace', it replaces any existing definition in the
+        same place in the list or is appended otherwise. If mode is
+        'append', then an existing definition is deleted, and the new
+        one appended to the end.
+        """
+
+        if not isinstance(val, basestring):
+            raise RuntimeError, 'Value should be string'
+        if mode not in ('appendalways', 'append', 'replace'):
+            raise RuntimeError, 'Invalid mode'
+        if ctype not in ('constant', 'import', 'function'):
+            raise RuntimeError, 'Invalid type'
 
         vals = list( self.document.customs )
-        vals.append( [name, ctype, val] )
+        item = [ctype, name, val]
+        if mode == 'appendalways':
+            vals.append(item)
+        else:
+            # find any existing item
+            for i, (t, n, v) in enumerate(vals):
+                if n == name:
+                    if mode == 'append':
+                        del vals[i]
+                        vals.append(item)
+                    else: # replace
+                        vals[i] = item
+                    break
+            else:
+                # no existing item, so append
+                vals.append(item)
+
         op = operations.OperationSetCustom(vals)
         self.document.applyOperation(op)
 
@@ -240,17 +285,32 @@ class CommandInterface(qt4.QObject):
         if self.verbose:
             print "Removed widget '%s'" % name
 
+    def RemoveCustom(self, name):
+        """Removes a custom-defined constant, function or import."""
+        vals = list( self.document.customs )
+        for i, (t, n, v) in enumerate(vals):
+            if n == name:
+                del vals[i]
+                break
+        else:
+            raise ValueError, 'Custom variable not defined'
+        op = operations.OperationSetCustom(vals)
+        self.document.applyOperation(op)
+
     def To(self, where):
-        """Change to a graph within the current graph."""
+        """Change to a widget within the current widget.
+
+        where is a path to the widget relative to the current widget
+        """
 
         self.currentwidget = self.document.resolve(self.currentwidget,
                                                    where)
 
         if self.verbose:
-            print "Changed to graph '%s'" % self.currentwidget.path
+            print "Changed to widget '%s'" % self.currentwidget.path
 
     def List(self, where='.'):
-        """List the contents of a graph."""
+        """List the contents of a widget, by default the current widget."""
 
         widget = self.document.resolve(self.currentwidget, where)
         children = widget.childnames
@@ -630,27 +690,39 @@ class CommandInterface(qt4.QObject):
 
         return (dsnames, errors)
 
-    def ImportFileCSV(self, filename, readrows=False, prefix=None,
+    def ImportFileCSV(self, filename,
+                      readrows=False,
                       delimiter=',', textdelimiter='"',
                       encoding='utf_8',
-                      headerignore=0,
-                      dsprefix='', dssuffix='',
+                      headerignore=0, rowsignore=0,
+                      blanksaredata=False,
+                      numericlocale='en_US',
+                      dateformat='YYYY-MM-DD|T|hh:mm:ss',
+                      headermode='multi',
+                      dsprefix='', dssuffix='', prefix=None,
                       linked=False):
         """Read data from a comma separated file (CSV).
 
         Data are read from filename
-        If readrows is True, then data are read across rather than down
         
+        readrows: if true, data are read across rather than down
+        delimiter: character for delimiting data (usually ',')
+        textdelimiter: character surrounding text (usually '"')
+        encoding: encoding used in file
+        headerignore: number of lines to ignore after header text
+        rowsignore: number of rows to ignore at top of file
+        blanksaredata: treats blank lines in csv files as blank data values
+        numericlocale: format to use for reading numbers
+        dateformat: format for interpreting dates
+        headermode: 'multi': multiple headers allowed in file
+                    '1st': first text found are headers
+                    'none': no headers, guess data and use default names
+
         Dataset names are prepended and appended, by dsprefix and dssuffix,
         respectively
          (prefix is backware compatibility only, it adds an underscore
           relative to dsprefix)
 
-        delimiter is the character for delimiting data (usually ',')
-        textdelimiter is the character surrounding text (usually '"')
-        encoding is the encoding used in the file
-        headerignore is number of lines to ignore after header text
-
         If linked is True the data are linked with the file."""
 
         # backward compatibility
@@ -660,13 +732,17 @@ class CommandInterface(qt4.QObject):
         # lookup filename
         realfilename = self.findFileOnImportPath(filename)
 
-        op = operations.OperationDataImportCSV(
+        params = readcsv.ParamsCSV(
             realfilename, readrows=readrows,
             delimiter=delimiter, textdelimiter=textdelimiter,
             encoding=encoding,
-            headerignore=headerignore,
-            prefix=dsprefix, suffix=dssuffix,
-            linked=linked)
+            headerignore=headerignore, rowsignore=rowsignore,
+            blanksaredata=blanksaredata,
+            numericlocale=numericlocale, dateformat=dateformat,
+            headermode=headermode,
+            dsprefix=dsprefix, dssuffix=dssuffix,
+            )
+        op = operations.OperationDataImportCSV(params, linked=linked)
         dsnames = self.document.applyOperation(op)
             
         if self.verbose:
@@ -717,8 +793,10 @@ class CommandInterface(qt4.QObject):
                                                   **args)
         try:
             self.document.applyOperation(op)
-        except Exception, ex:
-            self.document.log("Error in plugin %s: %s" % (plugin, unicode(ex)))
+        except:
+            self.document.log("Error in plugin %s" % plugin)
+            exc =  ''.join(traceback.format_exc())
+            self.document.log(exc)
 
     def ReloadData(self):
         """Reload any linked datasets.
@@ -749,7 +827,8 @@ class CommandInterface(qt4.QObject):
                                    range(self.document.getNumberPages()) )
             
     def Export(self, filename, color=True, page=0, dpi=100,
-               antialias=True, quality=85, backcolor='#ffffff00'):
+               antialias=True, quality=85, backcolor='#ffffff00',
+               pdfdpi=150):
         """Export plot to filename.
 
         color is True or False if color is requested in output file
@@ -759,12 +838,15 @@ class CommandInterface(qt4.QObject):
         quality is a quality parameter for jpeg output
         backcolor is the background color for bitmap files, which is a name or
          a #RRGGBBAA value (red, green, blue, alpha)
+        pdfdpi is the dpi to use when exporting eps or pdf files
         """
         
-        self.document.export(filename, page, color=color,
-                             dpi=dpi, antialias=antialias,
-                             quality=quality, backcolor=backcolor)
-            
+        e = export.Export(self.document, filename, page, color=color,
+                          bitmapdpi=dpi, antialias=antialias,
+                          quality=quality, backcolor=backcolor,
+                          pdfdpi=pdfdpi)
+        e.export()
+
     def Rename(self, widget, newname):
         """Rename the widget with the path given to the new name.
 
