@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #    Copyright (C) 2006 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -16,8 +17,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: datasets.py 1471 2010-12-10 22:18:24Z jeremysanders $
-
 """Classes to represent datasets."""
 
 import re
@@ -29,6 +28,7 @@ import simpleread
 import operations
 import readcsv
 
+import veusz.qtall as qt4
 import veusz.utils as utils
 import veusz.setting as setting
 import veusz.plugins as plugins
@@ -95,7 +95,7 @@ def generateValidDatasetParts(*datasets):
             # if not a dataset
             pass
 
-    # get indexes of invalid pounts
+    # get indexes of invalid points
     indexes = invalid.nonzero()[0].tolist()
 
     # no bad points: optimisation
@@ -152,7 +152,7 @@ class LinkedFileBase(object):
 
         for name, ds in document.data.items():
             if ds.linked == self:
-                del document.data[name]
+                document.deleteData(name)
 
     def _moveReadDatasets(self, tempdoc, document):
         """Move datasets from tempdoc to document if they do not exist
@@ -162,10 +162,9 @@ class LinkedFileBase(object):
         for name, ds in tempdoc.data.items():
             if name not in document.data:
                 read.append(name)
-                document.data[name] = ds
+                document.setData(name, ds)
                 ds.document = document
                 ds.linked = self
-        document.setModified(True)
         return read
 
     def _reloadViaOperation(self, document, op):
@@ -356,61 +355,34 @@ class LinkedFITSFile(LinkedFileBase):
 class LinkedCSVFile(LinkedFileBase):
     """A CSV file linked to datasets."""
 
-    def __init__(self, filename, readrows=False,
-                 delimiter=',', textdelimiter='"',
-                 encoding='utf_8',
-                 headerignore=0,
-                 prefix='', suffix=''):
+    def __init__(self, params):
         """Read CSV data from filename
-
-        Read across rather than down if readrows
-        Prepend prefix to dataset names if set.
+        params is a ParamsCSV object
         """
-
-        self.filename = filename
-        self.readrows = readrows
-        self.delimiter = delimiter
-        self.textdelimiter = textdelimiter
-        self.encoding = encoding
-        self.headerignore = headerignore
-        self.prefix = prefix
-        self.suffix = suffix
+        self.filename = params.filename
+        self.params = params
 
     def saveToFile(self, fileobj, relpath=None):
         """Save the link to the document file."""
 
-        params = [repr(self._getSaveFilename(relpath)),
-                  'linked=True']
-        if self.prefix:
-            params.append('dsprefix=' + repr(self.prefix))
-        if self.suffix:
-            params.append('dssuffix=' + repr(self.suffix))
-        if self.readrows:
-            params.append('readrows=True')
-        if self.encoding != 'utf_8':
-            params.append('encoding=' + repr(self.encoding))
-        if self.delimiter != ',':
-            params.append('delimiter=' + repr(self.delimiter))
-        if self.textdelimiter != '"':
-            params.append('textdelimiter=' + repr(self.textdelimiter))
-        if self.headerignore > 0:
-            params.append('headerignore=' + repr(self.headerignore))
+        paramsout = [repr(self._getSaveFilename(relpath)),
+                     'linked=True']
+
+        # add parameters which aren't defaults
+        for param, default in sorted(self.params.defaults.items()):
+            v = getattr(self.params, param)
+            if v != default:
+                paramsout.append('%s=%s' % (param, repr(v)))
+
+        fileobj.write('ImportFileCSV(%s)\n' % (', '.join(paramsout)))
 
-        fileobj.write('ImportFileCSV(%s)\n' % (', '.join(params)))
-        
     def reloadLinks(self, document):
         """Reload any linked data from the CSV file."""
 
         # again, this is messy as we have to make sure we don't
         # overwrite any non-linked data
 
-        op = operations.OperationDataImportCSV(
-            self.filename, readrows=self.readrows,
-            delimiter=self.delimiter,
-            textdelimiter=self.textdelimiter,
-            encoding=self.encoding,
-            headerignore=self.headerignore,
-            prefix=self.prefix, suffix=self.suffix )
+        op = operations.OperationDataImportCSV(self.params)
         return self._reloadViaOperation(document, op)
 
 class LinkedFilePlugin(LinkedFileBase):
@@ -461,13 +433,34 @@ class DatasetBase(object):
 
     # number of dimensions the dataset holds
     dimensions = 0
-    datatype = 'numeric'
+
+    # datatype is fundamental type of data
+    # displaytype is formatting suggestion for data
+    datatype = displaytype = 'numeric'
+
+    # dataset type to show to user
+    dstype = 'Dataset'
+
+    # list of columns in dataset (if any)
     columns = ()
+    # use descriptions for columns
     column_descriptions = ()
 
     # class for representing part of this dataset
     subsetclass = None
 
+    # whether this dataset's columns will change without updating the document's
+    # changeset
+    isstable = False
+
+    def __init__(self, linked=None):
+        """Initialise common members."""
+        # document member set when this dataset is set in document
+        self.document = None
+
+        # file this dataset is linked to
+        self.linked = linked
+
     def saveLinksToSavedDoc(self, fileobj, savedlinks, relpath=None):
         '''Save the link to the saved document, if this dataset is linked.
 
@@ -489,13 +482,32 @@ class DatasetBase(object):
                 return name
         raise ValueError('Could not find self in document.data')
 
+    def userSize(self):
+        """Return dimensions of dataset for user."""
+        return ""
+
+    def userPreview(self):
+        """Return a small preview of the dataset for the user, e.g.
+        1, 2, 3, ..., 4, 5, 6."""
+        return None
+
     def description(self, showlinked=True):
         """Get description of database."""
         return ""
 
-    def convertToDataItem(self, val):
-        """Return a value cast to this dataset data type."""
-        return None
+    def uiConvertToDataItem(self, val):
+        """Return a value cast to this dataset data type.
+        We assume here it is a float, so override if not
+        """
+        if isinstance(val, basestring) or isinstance(val, qt4.QString):
+            val, ok = setting.uilocale.toDouble(val)
+            if ok: return val
+            raise ValueError, "Invalid floating point number"
+        return float(val)
+
+    def uiDataItemToQVariant(self, val):
+        """Return val converted to QVariant."""
+        return qt4.QVariant(float(val))
     
     def _getItemHelper(self, key):
         """Help get arguments to constructor."""
@@ -549,12 +561,22 @@ class DatasetBase(object):
         """Is it possible to rename this dataset?"""
         return self.linked is None
 
+    def datasetAsText(self, fmt='%g', join='\t'):
+        """Return dataset as text (for use by user)."""
+        return ''
+
 class Dataset2D(DatasetBase):
     '''Represents a two-dimensional dataset.'''
 
     # number of dimensions the dataset holds
     dimensions = 2
 
+    # dataset type
+    dstype = '2D'
+    
+    # the dataset is recreated if its data changes
+    isstable = True
+
     def __init__(self, data, xrange=None, yrange=None):
         '''Create a two dimensional dataset based on data.
 
@@ -563,17 +585,22 @@ class Dataset2D(DatasetBase):
         yrange: a tuple of (start, end) coordinates for y
         '''
 
-        self.document = None
-        self.linked = None
-        self.data = _convertNumpy(data)
+        DatasetBase.__init__(self)
 
-        self.xrange = xrange
-        self.yrange = yrange
+        # we don't want these set if a inheriting class uses properties instead
+        if not hasattr(self, 'data'):
+            try:
+                self.data = _convertNumpy(data)
+                self.xrange = (0, self.data.shape[1])
+                self.yrange = (0, self.data.shape[0])
 
-        if not xrange:
-            self.xrange = (0, data.shape[1])
-        if not yrange:
-            self.yrange = (0, data.shape[0])
+                if xrange:
+                    self.xrange = xrange
+                if yrange:
+                    self.yrange = yrange
+            except AttributeError:
+                # for some reason hasattr doesn't always work
+                pass
 
     def indexToPoint(self, xidx, yidx):
         """Convert a set of indices to pixels in integers to
@@ -598,31 +625,63 @@ class Dataset2D(DatasetBase):
         fileobj.write("ImportString2D(%s, '''\n" % repr(name))
         fileobj.write("xrange %e %e\n" % self.xrange)
         fileobj.write("yrange %e %e\n" % self.yrange)
+        fileobj.write(self.datasetAsText(fmt='%e', join=' '))
+        fileobj.write("''')\n")
+
+    def datasetAsText(self, fmt='%g', join='\t'):
+        """Return dataset as text.
+        fmt is the format specifier to use
+        join is the string to separate the items
+        """
+        format = ((fmt+join) * (self.data.shape[1]-1)) + fmt + '\n'
 
         # write rows backwards, so lowest y comes first
+        lines = []
         for row in self.data[::-1]:
-            s = ('%e ' * len(row)) % tuple(row)
-            fileobj.write("%s\n" % (s[:-1],))
+            line = format % tuple(row)
+            lines.append(line)
+        return ''.join(lines)
 
-        fileobj.write("''')\n")
+    def userSize(self):
+        """Return dimensions of dataset for user."""
+        return u'%i×%i' % self.data.shape
+
+    def userPreview(self):
+        """Return preview of data."""
+        return dsPreviewHelper(self.data.flatten())
 
     def description(self, showlinked=True):
         """Get description of dataset."""
         text = self.name()
-        text += ' (%ix%i)' % self.data.shape
+        text += u' (%i×%i)' % self.data.shape
         text += ', x=%g->%g' % tuple(self.xrange)
         text += ', y=%g->%g' % tuple(self.yrange)
         if self.linked and showlinked:
             text += ', linked to %s' % self.linked.filename
         return text
 
-    def convertToDataItem(self, val):
-        """Return a value cast to this dataset data type."""
-        return float(val)
-
     def returnCopy(self):
         return Dataset2D( N.array(self.data), self.xrange, self.yrange)
 
+def dsPreviewHelper(d):
+    """Get preview of numpy data d."""
+    if d.shape[0] <= 6:
+        line1 = ', '.join( ['%.3g' % x for x in d] )
+    else:
+        line1 = ', '.join( ['%.3g' % x for x in d[:3]] +
+                           [ '...' ] +
+                           ['%.3g' % x for x in d[-3:]] )
+
+    try:
+        line2 = 'mean: %.3g, min: %.3g, max: %.3g' % (
+            N.nansum(d) / N.isfinite(d).sum(),
+            N.nanmin(d),
+            N.nanmax(d))
+    except (ValueError, ZeroDivisionError):
+        # nanXXX returns error if no valid data points
+        return line1
+    return line1 + '\n' + line2
+
 class Dataset(DatasetBase):
     '''Represents a dataset.'''
 
@@ -630,6 +689,10 @@ class Dataset(DatasetBase):
     dimensions = 1
     columns = ('data', 'serr', 'nerr', 'perr')
     column_descriptions = ('Data', 'Sym. errors', 'Neg. errors', 'Pos. errors')
+    dstype = '1D'
+
+    # the dataset is recreated if its data changes
+    isstable = True
 
     def __init__(self, data = None, serr = None, nerr = None, perr = None,
                  linked = None):
@@ -639,6 +702,8 @@ class Dataset(DatasetBase):
         linked optionally specifies a LinkedFile to link the dataset to
         '''
         
+        DatasetBase.__init__(self, linked=linked)
+
         # convert data to numpy arrays
         data = _convertNumpy(data)
         serr = _convertNumpyAbs(serr)
@@ -652,13 +717,25 @@ class Dataset(DatasetBase):
                 raise DatasetException('Lengths of error data do not match data')
 
         # finally assign data
-        self.document = None
         self._invalidpoints = None
-        self.linked = linked
-        self.data = data
-        self.serr = serr
-        self.perr = perr
-        self.nerr = nerr
+
+        try:
+            if not hasattr(self, 'data'):
+                self.data = data
+                self.serr = serr
+                self.perr = perr
+                self.nerr = nerr
+        except AttributeError:
+            # we don't want these set if a inheriting class uses properties instead
+            pass
+
+    def userSize(self):
+        """Size of dataset."""
+        return str( self.data.shape[0] )
+
+    def userPreview(self):
+        """Preview of data."""
+        return dsPreviewHelper(self.data)
 
     def description(self, showlinked=True):
         """Get description of dataset."""
@@ -741,7 +818,8 @@ class Dataset(DatasetBase):
         for x in (self.serr, self.nerr, self.perr):
             assert x is None or x.shape == s
 
-        self.document.setModified(True)
+        # tell the document that we've changed
+        self.document.modifiedData(self)
 
     def saveToFile(self, fileobj, name):
         '''Save data to file.
@@ -752,32 +830,35 @@ class Dataset(DatasetBase):
             return
 
         # build up descriptor
-        datasets = [self.data]
-
         descriptor = datasetNameToDescriptorName(name) + '(numeric)'
         if self.serr is not None:
             descriptor += ',+-'
-            datasets.append(self.serr)
         if self.perr is not None:
             descriptor += ',+'
-            datasets.append(self.perr)
         if self.nerr is not None:
             descriptor += ',-'
-            datasets.append(self.nerr)
 
         fileobj.write( "ImportString(%s,'''\n" % repr(descriptor) )
-
-        # write line line-by-line
-        format = '%e ' * len(datasets)
-        format = format[:-1] + '\n'
-        for line in izip( *datasets ):
-            fileobj.write( format % line )
-
+        fileobj.write( self.datasetAsText(fmt='%e', join=' ') )
         fileobj.write( "''')\n" )
 
-    def convertToDataItem(self, val):
-        """Return a value cast to this dataset data type."""
-        return float(val)
+    def datasetAsText(self, fmt='%g', join='\t'):
+        """Return data as text."""
+
+        # work out which columns to write
+        cols = []
+        for c in (self.data, self.serr, self.perr, self.nerr):
+            if c is not None:
+                cols.append(c)
+
+        # format statement
+        format = (fmt + join) * (len(cols)-1) + fmt + '\n'
+
+        # do the conversion
+        lines = []
+        for line in izip(*cols):
+            lines.append( format % line )
+        return ''.join(lines)
 
     def deleteRows(self, row, numrows):
         """Delete numrows rows starting from row.
@@ -790,6 +871,7 @@ class Dataset(DatasetBase):
                 retn[col] = coldata[row:row+numrows]
                 setattr(self, col, N.delete( coldata, N.s_[row:row+numrows] ))
         
+        self.document.modifiedData(self)
         return retn
 
     def insertRows(self, row, numrows, rowdata):
@@ -804,6 +886,8 @@ class Dataset(DatasetBase):
             if coldata is not None:
                 setattr(self, col, N.insert(coldata, [row]*numrows, data))
 
+        self.document.modifiedData(self)
+
     def returnCopy(self):
         """Return version of dataset with no linking."""
         return Dataset(data = _copyOrNone(self.data),
@@ -811,19 +895,80 @@ class Dataset(DatasetBase):
                        perr = _copyOrNone(self.perr),
                        nerr = _copyOrNone(self.nerr))
 
+class DatasetDateTime(Dataset):
+    """Dataset holding dates and times."""
+
+    columns = ('data',)
+    column_descriptions = ('Data',)
+    isstable = True
+
+    dstype = 'Date'
+    displaytype = 'date'
+
+    def __init__(self, data=None, linked=None):
+        Dataset.__init__(self, data=data, linked=linked)
+
+    def description(self, showlinked=True):
+        text = '%s (%i date/times)' % (self.name(), len(self.data))
+        if self.linked and showlinked:
+            text += ', linked to %s' % self.linked.filename
+        return text
+
+    def uiConvertToDataItem(self, val):
+        """Return a value cast to this dataset data type."""
+        if isinstance(val, basestring) or isinstance(val, qt4.QString):
+            v = utils.dateStringToDate( unicode(val) )
+            if not N.isfinite(v):
+                try:
+                    v = float(val)
+                except ValueError:
+                    pass
+            return v
+        else:
+            return N.nan
+
+    def uiDataItemToQVariant(self, val):
+        """Return val converted to QVariant."""
+        return qt4.QVariant(utils.dateFloatToString(val))
+
+    def saveToFile(self, fileobj, name):
+        '''Save data to file.
+        '''
+
+        if self.linked is not None:
+            # do not save if linked to a file
+            return
+
+        descriptor = datasetNameToDescriptorName(name) + '(date)'
+        fileobj.write( "ImportString(%s,'''\n" % repr(descriptor) )
+        fileobj.write( self.datasetAsText() )
+        fileobj.write( "''')\n" )
+
+    def datasetAsText(self, fmt=None, join=None):
+        """Return data as text."""
+        lines = [ utils.dateFloatToString(val) for val in self.data ]
+        lines.append('')
+        return '\n'.join(lines)
+
+    def returnCopy(self):
+        """Returns version of dataset with no linking."""
+        return DatasetDateTime(data=N.array(self.data))
+
 class DatasetText(DatasetBase):
     """Represents a text dataset: holding an array of strings."""
 
     dimensions = 1
-    datatype = 'text'
+    datatype = displaytype = 'text'
     columns = ('data',)
     column_descriptions = ('Data',)
+    dstype = 'Text'
+    isstable = True
 
     def __init__(self, data=None, linked=None):
         """Initialise dataset with data given. Data are a list of strings."""
 
+        DatasetBase.__init__(self, linked=linked)
         self.data = list(data)
-        self.linked = linked
 
     def description(self, showlinked=True):
         text = '%s (%i items)' % (self.name(), len(self.data))
@@ -831,18 +976,26 @@ class DatasetText(DatasetBase):
             text += ', linked to %s' % self.linked.filename
         return text
 
+    def userSize(self):
+        """Size of dataset."""
+        return str( len(self.data) )
+
     def changeValues(self, type, vals):
         if type == 'data':
             self.data = list(vals)
         else:
             raise ValueError, 'type does not contain an allowed value'
 
-        self.document.setModified(True)
+        self.document.modifiedData(self)
     
-    def convertToDataItem(self, val):
+    def uiConvertToDataItem(self, val):
         """Return a value cast to this dataset data type."""
         return unicode(val)
 
+    def uiDataItemToQVariant(self, val):
+        """Return val converted to QVariant."""
+        return qt4.QVariant(unicode(val))
+
     def saveToFile(self, fileobj, name):
         '''Save data to file.
         '''
@@ -859,12 +1012,20 @@ class DatasetText(DatasetBase):
             fileobj.write(r)
         fileobj.write( "''')\n" )
 
+    def datasetAsText(self, fmt=None, join=None):
+        """Return data as text."""
+        lines = list(self.data)
+        lines.append('')
+        return '\n'.join(lines)
+
     def deleteRows(self, row, numrows):
         """Delete numrows rows starting from row.
         Returns deleted rows as a dict of {column:data, ...}
         """
         retn = {'data': self.data[row:row+numrows]}
         del self.data[row:row+numrows]
+        
+        self.document.modifiedData(self)
         return retn
 
     def insertRows(self, row, numrows, rowdata):
@@ -877,6 +1038,8 @@ class DatasetText(DatasetBase):
         for d in insdata[::-1]:
             self.data.insert(row, d)
 
+        self.document.modifiedData(self)
+
     def returnCopy(self):
         """Returns version of dataset with no linking."""
         return DatasetText(self.data)
@@ -976,6 +1139,8 @@ def simpleEvalExpression(doc, expr, part='data'):
 class DatasetExpression(Dataset):
     """A dataset which is linked to another dataset by an expression."""
 
+    dstype = 'Expression'
+
     def __init__(self, data=None, serr=None, nerr=None, perr=None,
                  parametric=None):
         """Initialise the dataset with the expressions given.
@@ -983,9 +1148,7 @@ class DatasetExpression(Dataset):
         parametric is option and can be (minval, maxval, steps) or None
         """
 
-        self.document = None
-        self.linked = None
-        self._invalidpoints = None
+        Dataset.__init__(self, data=[])
 
         # store the expressions to use to generate the dataset
         self.expr = {}
@@ -1160,11 +1323,17 @@ class DatasetExpression(Dataset):
 class DatasetRange(Dataset):
     """Dataset consisting of a range of values e.g. 1 to 10 in 10 steps."""
 
+    dstype = 'Range'
+    isstable = True
+
     def __init__(self, numsteps, data, serr=None, perr=None, nerr=None):
         """Construct dataset.
 
         numsteps: number of steps in range
         data, serr, perr and nerr are tuples containing (start, stop) values."""
+
+        Dataset.__init__(self, data=[])
+
         self.range_data = data
         self.range_serr = serr
         self.range_perr = perr
@@ -1184,9 +1353,17 @@ class DatasetRange(Dataset):
                 vals = None
             setattr(self, name, vals)
 
-        self.document = None
-        self.linked = None
-        self._invalidpoints = None
+    def __getitem__(self, key):
+        """Return a dataset based on this dataset
+
+        We override this from DatasetBase as it would return a
+        DatsetExpression otherwise, not chopped sets of data.
+        """
+        return Dataset(**self._getItemHelper(key))
+
+    def userSize(self):
+        """Size of dataset."""
+        return str( self.numsteps )
 
     def saveToFile(self, fileobj, name):
         """Save dataset to file."""
@@ -1226,11 +1403,12 @@ def getSpacing(data):
     (i.e. steps are not all multiples of some mininimum)
     """
 
-    uniquesorted = N.unique1d(data)
+    uniquesorted = N.unique(data)
+
     sigfactor = (uniquesorted[-1]-uniquesorted[0])*1e-13
 
     # differences between elements
-    deltas = N.unique1d( N.ediff1d(uniquesorted) )
+    deltas = N.unique( N.ediff1d(uniquesorted) )
 
     mindelta = None
     for delta in deltas:
@@ -1251,14 +1429,14 @@ def getSpacing(data):
 class Dataset2DXYZExpression(Dataset2D):
     '''A 2d dataset with expressions for x, y and z.'''
 
+    dstype = '2D XYZ'
+
     def __init__(self, exprx, expry, exprz):
         """Initialise dataset.
 
         Parameters are mathematical expressions based on datasets."""
+        Dataset2D.__init__(self, [])
 
-        self.document = None
-        self.linked = None
-        self._invalidpoints = None
         self.lastchangeset = -1
         self.cacheddata = None
         
@@ -1308,7 +1486,7 @@ class Dataset2DXYZExpression(Dataset2D):
             except Exception, e:
                 raise DatasetExpressionException(
                     "Error evaluating expression: %s\n"
-                    "Error: %s" % (expr, str(e)) )
+                    "Error: %s" % (expr, unicode(e)) )
 
         minx, maxx, stepx, stepsx = getSpacing(evaluated['exprx'])
         miny, maxy, stepy, stepsy = getSpacing(evaluated['expry'])
@@ -1323,7 +1501,12 @@ class Dataset2DXYZExpression(Dataset2D):
         ypts = ((1./stepy)*(evaluated['expry']-miny)).astype('int32')
 
         # this is ugly - is this really the way to do it?
-        self.cacheddata.flat [ xpts + ypts*stepsx ] = evaluated['exprz']
+        try:
+            self.cacheddata.flat [ xpts + ypts*stepsx ] = evaluated['exprz']
+        except Exception, e:
+            raise DatasetExpressionException(
+                "Shape mismatch when constructing dataset\n"
+                "Error: %s" % unicode(e) )
 
         # update changeset
         self.lastchangeset = self.document.changeset
@@ -1384,10 +1567,13 @@ class Dataset2DXYZExpression(Dataset2D):
 class Dataset2DExpression(Dataset2D):
     """Evaluate an expression of 2d datasets."""
 
+    dstype = '2D Expr'
+
     def __init__(self, expr):
         """Create 2d expression dataset."""
-        self.document = None
-        self.linked = None
+
+        Dataset2D.__init__(self, None)
+
         self.expr = expr
         self.lastchangeset = -1
         self.cachedexpr = None
@@ -1490,12 +1676,13 @@ class Dataset2DExpression(Dataset2D):
         """Return linking information."""
         return 'Linked 2D expression: %s' % self.expr
 
-
 class Dataset2DXYFunc(Dataset2D):
     """Given a range of x and y, this is a dataset which is a function of
     this.
     """
 
+    dstype = '2D f(x,y)'
+
     def __init__(self, xstep, ystep, expr):
         """Create 2d dataset:
 
@@ -1504,9 +1691,7 @@ class Dataset2DXYFunc(Dataset2D):
         expr: expression of x and y
         """
 
-        self.document = None
-        self.linked = None
-        self._invalidpoints = None
+        Dataset2D.__init__(self, [])
 
         self.xstep = xstep
         self.ystep = ystep
@@ -1625,6 +1810,11 @@ class _DatasetPlugin(object):
                     self.pluginmanager.saveToFile(fileobj)
                 return
 
+    @property
+    def dstype(self):
+        """Return type of plugin."""
+        return self.pluginmanager.plugin.name
+
 class Dataset1DPlugin(_DatasetPlugin, Dataset):
     """Return 1D dataset from a plugin."""
 
@@ -1635,6 +1825,18 @@ class Dataset1DPlugin(_DatasetPlugin, Dataset):
     def __getitem__(self, key):
         return Dataset(**self._getItemHelper(key))
 
+    def userSize(self):
+        """Size of dataset."""
+        return str( self.data.shape[0] )
+
+    def __getitem__(self, key):
+        """Return a dataset based on this dataset
+
+        We override this from DatasetBase as it would return a
+        DatsetExpression otherwise, not chopped sets of data.
+        """
+        return Dataset(**self._getItemHelper(key))
+
     # parent class sets these attributes, so override setattr to do nothing
     data = property( lambda self: self.getPluginData('data'),
                      lambda self, val: None )
@@ -1674,3 +1876,16 @@ class DatasetTextPlugin(_DatasetPlugin, DatasetText):
 
     data = property( lambda self: self.getPluginData('data'),
                      lambda self, val: None )
+
+class DatasetDateTimePlugin(_DatasetPlugin, DatasetDateTime):
+    """Return date dataset from plugin."""
+
+    def __init__(self, manager, ds):
+        _DatasetPlugin.__init__(self, manager, ds)
+        DatasetDateTime.__init__(self, [])
+
+    def __getitem__(self, key):
+        return DatasetDateTime(self.data[key])
+
+    data = property( lambda self: self.getPluginData('data'),
+                     lambda self, val: None )
