@@ -19,16 +19,12 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: doc.py 1466 2010-12-02 13:58:20Z jeremysanders $
-
 """A class to represent Veusz documents, with dataset classes."""
 
 import os.path
-import time
-import random
-import math
 import re
 import traceback
+import datetime
 from collections import defaultdict
 
 import numpy as N
@@ -37,19 +33,11 @@ import veusz.qtall as qt4
 
 import widgetfactory
 import datasets
+import painthelper
 
 import veusz.utils as utils
 import veusz.setting as setting
 
-try:
-    import emf_export
-    hasemf = True
-except ImportError:
-    hasemf = False
-
-import svg_export
-import selftest_export
-
 # python identifier
 identifier_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_]*$')
 # for splitting
@@ -94,8 +82,20 @@ class Document( qt4.QObject ):
             Document.loadPlugins()
             Document.pluginsloaded = True
 
-        self.changeset = 0          # increased when the document changes
-        self.suspendupdates = False # if True then do not notify listeners of updates
+        # change tracking of document as a whole
+        self.changeset = 0            # increased when the document changes
+
+        # change tracking of datasets
+        self.datachangeset = 0        # increased whan any dataset changes
+        self.datachangesets = dict()  # each ds has an associated change set
+
+        # if set, do not notify listeners of updates
+        # wait under enableUpdates
+        self.suspendupdates = []
+
+        # default document locale
+        self.locale = qt4.QLocale()
+
         self.clearHistory()
         self.wipe()
 
@@ -105,9 +105,13 @@ class Document( qt4.QObject ):
         # store custom functions and constants
         # consists of tuples of (name, type, value)
         # type is constant or function
+        # we use this format to preserve evaluation order
         self.customs = []
         self.updateEvalContext()
 
+        # copy default colormaps
+        self.colormaps = dict(utils.defaultcolormaps)
+
     def wipe(self):
         """Wipe out any stored data."""
         self.data = {}
@@ -124,16 +128,17 @@ class Document( qt4.QObject ):
         self.historyredo = []
         
     def suspendUpdates(self):
-        """Holds sending update messages. This speeds up modification of the document."""
-        assert not self.suspendupdates
-        self.suspendchangeset = self.changeset
-        self.suspendupdates = True
+        """Holds sending update messages.
+        This speeds up modification of the document and prevents the document
+        from being updated on the screen."""
+        self.suspendupdates.append(self.changeset)
 
     def enableUpdates(self):
         """Reenables document updates."""
-        assert self.suspendupdates
-        self.suspendupdates = False
-        if self.suspendchangeset != self.changeset:
+        changeset = self.suspendupdates.pop()
+        if len(self.suspendupdates) == 0 and changeset != self.changeset:
+            # bump this up as some watchers might ignore this otherwise
+            self.changeset += 1
             self.setModified()
 
     def makeDefaultDoc(self):
@@ -153,9 +158,18 @@ class Document( qt4.QObject ):
         
         Operations represent atomic actions which can be done to the document
         and undone.
+
+        Updates are suspended during the operation.
         """
-        
-        retn = operation.do(self)
+
+        self.suspendUpdates()
+        try:
+            retn = operation.do(self)
+            self.changeset += 1
+        except:
+            self.enableUpdates()
+            raise
+        self.enableUpdates()
 
         if self.historybatch:
             # in batch mode, create an OperationMultiple for all changes
@@ -165,7 +179,6 @@ class Document( qt4.QObject ):
             self.historyundo = self.historyundo[-9:] + [operation]
         self.historyredo = []
 
-        self.setModified()
         return retn
 
     def batchHistory(self, batch):
@@ -185,11 +198,17 @@ class Document( qt4.QObject ):
         
     def undoOperation(self):
         """Undo the previous operation."""
-        
+
         operation = self.historyundo.pop()
-        operation.undo(self)
+        self.suspendUpdates()
+        try:
+            operation.undo(self)
+            self.changeset += 1
+        except:
+            self.enableUpdates()
+            raise
+        self.enableUpdates()
         self.historyredo.append(operation)
-        self.setModified()
         
     def canUndo(self):
         """Returns True if previous operation can be removed."""
@@ -197,11 +216,8 @@ class Document( qt4.QObject ):
 
     def redoOperation(self):
         """Redo undone operations."""
-        
         operation = self.historyredo.pop()
-        operation.do(self)
-        self.historyundo.append(operation)
-        self.setModified()
+        return self.applyOperation(operation)
 
     def canRedo(self):
         """Returns True if previous operation can be redone."""
@@ -254,25 +270,52 @@ class Document( qt4.QObject ):
         """Set data to val, with symmetric or negative and positive errors."""
         self.data[name] = dataset
         dataset.document = self
+        
+        # update the change tracking
+        cs = self.datachangesets.get(name, 0)
+        self.datachangesets[name] = cs + 1
+        self.datachangeset += 1
         self.setModified()
+    
+    def deleteData(self, name):
+        """Remove a dataset"""
+        if name in self.data:
+            del self.data[name]
+            
+            # don't remove the changeset tracker, in case this action is later undone
+            self.datachangesets[name] += 1
+            self.datachangeset += 1
+            self.setModified()
 
-    def getLinkedFiles(self):
-        """Get a list of LinkedFile objects used by the document."""
+    def modifiedData(self, dataset):
+        """The named dataset was modified"""
+        for name, ds in self.data.iteritems():
+            if ds is dataset:
+                self.datachangesets[name] += 1
+                self.datachangeset += 1
+                self.setModified()
+
+    def getLinkedFiles(self, filenames=None):
+        """Get a list of LinkedFile objects used by the document.
+        if filenames is a set, only get the objects with filenames given
+        """
         links = set()
         for ds in self.data.itervalues():
-            if ds.linked:
+            if ds.linked and (filenames is None or
+                              ds.linked.filename in filenames):
                 links.add(ds.linked)
         return list(links)
 
-    def reloadLinkedDatasets(self):
+    def reloadLinkedDatasets(self, filenames=None):
         """Reload linked datasets from their files.
+        If filenames is a set(), only reload from these filenames
 
         Returns a tuple of
         - List of datasets read
         - Dict of tuples containing dataset names and number of errors
         """
 
-        links = self.getLinkedFiles()
+        links = self.getLinkedFiles(filenames=filenames)
 
         read = []
         errors = {}
@@ -305,6 +348,8 @@ class Document( qt4.QObject ):
         d = self.data[oldname]
         del self.data[oldname]
         self.data[newname] = d
+        # transfer change set to new name
+        self.datachangesets[newname] = self.datachangesets[oldname]
 
         self.setModified()
 
@@ -326,7 +371,7 @@ class Document( qt4.QObject ):
         self.modified = ismodified
         self.changeset += 1
 
-        if not self.suspendupdates:
+        if len(self.suspendupdates) == 0:
             self.emit( qt4.SIGNAL("sigModified"), ismodified )
 
     def isModified(self):
@@ -334,33 +379,36 @@ class Document( qt4.QObject ):
         return self.modified
 
     @classmethod
-    def loadPlugins(kls):
+    def loadPlugins(kls, pluginlist=None):
         """Load plugins and catch exceptions."""
-        for plugin in setting.settingdb.get('plugins', []):
+        if pluginlist is None:
+            pluginlist = setting.settingdb.get('plugins', [])
+
+        for plugin in pluginlist:
             try:
                 execfile(plugin, dict())
             except Exception, ex:
                 err = ('Error loading plugin ' + plugin + '\n\n' +
                        traceback.format_exc())
-                qt4.QMessageBox.critical(None, "Error loading plugin",
-                                         err)
+                qt4.QMessageBox.critical(None, "Error loading plugin", err)
 
     def printTo(self, printer, pages, scaling = 1., dpi = None,
                 antialias = False):
         """Print onto printing device."""
 
-        painter = Painter(scaling=scaling, dpi=dpi)
-       
-        painter.begin( printer )
-        painter.setRenderHint(qt4.QPainter.Antialiasing,
-                              antialias)
-        painter.setRenderHint(qt4.QPainter.TextAntialiasing,
-                              antialias)
-
+        dpi = (printer.logicalDpiX(), printer.logicalDpiY())
+        painter = qt4.QPainter(printer)
+        if antialias:
+            painter.setRenderHint(qt4.QPainter.Antialiasing, True)
+            painter.setRenderHint(qt4.QPainter.TextAntialiasing, True)
+   
         # This all assumes that only pages can go into the root widget
         num = len(pages)
         for count, page in enumerate(pages):
-            self.basewidget.draw(painter, page)
+            size = self.pageSize(page, dpi=dpi)
+            helper = painthelper.PaintHelper(size, dpi=dpi, directpaint=painter)
+            self.paintTo(helper, page)
+            painter.restore()
 
             # start new pages between each page
             if count < num-1:
@@ -368,13 +416,9 @@ class Document( qt4.QObject ):
 
         painter.end()
 
-    def paintTo(self, painter, page, scaling = 1., dpi = None):
-        """Paint page specified to the painter."""
-        
-        painter.veusz_scaling = scaling
-        if dpi is not None:
-            painter.veusz_pixperpt = dpi / 72.
-        self.basewidget.draw(painter, page)
+    def paintTo(self, painthelper, page):
+        """Paint page specified to the paint helper."""
+        self.basewidget.draw(painthelper, page)
 
     def getNumberPages(self):
         """Return the number of pages in the document."""
@@ -388,14 +432,9 @@ class Document( qt4.QObject ):
         """Write a header to a saved file of type."""
 
         fileobj.write('# Veusz %s (version %s)\n' % (type, utils.version()))
-        try:
-            fileobj.write('# User: %s\n' % os.environ['LOGNAME'] )
-        except KeyError:
-            pass
-        fileobj.write('# Date: %s\n\n' %
-                      time.strftime("%a, %d %b %Y %H:%M:%S +0000",
-                                    time.gmtime()) )
-        
+        fileobj.write('# Saved at %s\n\n' %
+                      datetime.datetime.utcnow().isoformat())
+
     def saveCustomDefinitions(self, fileobj):
         """Save custom constants and functions."""
 
@@ -447,242 +486,31 @@ class Document( qt4.QObject ):
 
         fileobj.write( stylesheet.saveText(True, rootname='') )
 
-    def _exportBitmap(self, filename, pagenumber, dpi=100, antialias=True,
-                      quality=85, backcolor='#ffffff00'):
-        """Export the pagenumber to the requested bitmap filename."""
-
-        # firstly have to convert dpi to image size
-        # have to use a temporary bitmap first
-        tmp = qt4.QPixmap(1, 1)
-        tmppainter = Painter(tmp)
-        realdpi = tmppainter.device().logicalDpiY()
-        width, height = self.basewidget.getSize(tmppainter)
-        scaling = dpi/float(realdpi)
-        tmppainter.end()
-        del tmp, tmppainter
-
-        # work out format
-        format = os.path.splitext(filename)[1]
-        if not format:
-            format = '.png'
-        # str is required as unicode not supported
-        format = str(format[1:].lower())
-
-        # create real output image
-        pixmap = qt4.QPixmap(int(width*scaling), int(height*scaling))
-        backqcolor = utils.extendedColorToQColor(backcolor)
-        if format != 'png':
-            # not transparent
-            backqcolor.setAlpha(255)
-        pixmap.fill(backqcolor)
-
-        # paint to the image
-        painter = Painter(pixmap)
-        painter.setRenderHint(qt4.QPainter.Antialiasing,
-                              antialias)
-        painter.setRenderHint(qt4.QPainter.TextAntialiasing,
-                              antialias)
-        self.paintTo(painter, pagenumber, scaling=scaling, dpi=realdpi)
-        painter.end()
-
-        # write image to disk
-        writer = qt4.QImageWriter()
-        writer.setFormat(qt4.QByteArray(format))
-
-        writer.setFileName(filename)
-
-        if format == 'png':
-            # min quality for png as it makes no difference to output
-            # and makes file size smaller
-            writer.setQuality(0)
+    def _pagedocsize(self, widget, dpi, scaling, integer):
+        """Helper for page or doc size."""
+        if dpi is None:
+            p = qt4.QPixmap(1, 1)
+            dpi = (p.logicalDpiX(), p.logicalDpiY())
+        helper = painthelper.PaintHelper( (1,1), dpi=dpi, scaling=scaling )
+        w = widget.settings.get('width').convert(helper)
+        h = widget.settings.get('height').convert(helper)
+        if integer:
+            return int(w), int(h)
         else:
-            writer.setQuality(quality)
+            return w, h        
 
-        writer.write( pixmap.toImage() )
-        
-    def _exportPS(self, filename, page, color=True):
-        """Postscript or eps format."""
+    def pageSize(self, pagenum, dpi=None, scaling=1., integer=True):
+        """Get the size of a particular page in pixels.
 
-        ext = os.path.splitext(filename)[1]
+        If dpi is None, use the default Qt screen dpi
+        Use dpi if given."""
+        return self._pagedocsize(self.basewidget.getPage(pagenum),
+                                 dpi=dpi, scaling=scaling, integer=integer)
 
-        printer = qt4.QPrinter()
-        printer.setFullPage(True)
-
-        # set printer parameters
-        printer.setColorMode( (qt4.QPrinter.GrayScale, qt4.QPrinter.Color)[color] )
-
-        if ext == '.pdf':
-            f = qt4.QPrinter.PdfFormat
-        else:
-            f = qt4.QPrinter.PostScriptFormat
-        printer.setOutputFormat(f)
-        printer.setOutputFileName(filename)
-        printer.setCreator('Veusz %s' % utils.version())
-
-        # draw the page
-        printer.newPage()
-        painter = Painter(printer)
-        width, height = self.basewidget.getSize(painter)
-        self.basewidget.draw(painter, page)
-        painter.end()
-
-        # fixup eps/pdf file - yuck HACK! - hope qt gets fixed
-        # this makes the bounding box correct
-        if ext == '.eps' or ext == '.pdf':
-            # copy eps to a temporary file
-            tmpfile = "%s.tmp.%i" % (filename, random.randint(0,1000000))
-            fout = open(tmpfile, 'wb')
-            fin = open(filename, 'rb')
-
-            if ext == '.eps':
-                # adjust bounding box
-                for line in fin:
-                    if line[:14] == '%%BoundingBox:':
-                        # replace bounding box line by calculated one
-                        parts = line.split()
-                        widthfactor = float(parts[3]) / printer.width()
-                        origheight = float(parts[4])
-                        line = "%s %i %i %i %i\n" % (
-                            parts[0], 0,
-                            int(math.floor(origheight-widthfactor*height)),
-                            int(math.ceil(widthfactor*width)),
-                            int(math.ceil(origheight)) )
-                    fout.write(line)
-
-            elif ext == '.pdf':
-                # change pdf bounding box and correct pdf index
-                text = fin.read()
-                text = utils.scalePDFMediaBox(text,
-                                              printer.width(),
-                                              width, height)
-                text = utils.fixupPDFIndices(text)
-                fout.write(text)
-
-            fout.close()
-            fin.close()
-            os.remove(filename)
-            os.rename(tmpfile, filename)
-
-    def _getDocSize(self, dpi):
-        """Get size of document in pixels given dpi."""
-        pixmap = qt4.QPixmap(1, 1)
-        painter = Painter(pixmap, scaling=1., dpi=dpi)
-        width, height = self.basewidget.getSize(painter)
-        painter.end()
-        return width, height
-
-    def _exportSVG(self, filename, page):
-        """Export document as SVG"""
-
-        dpi = 90.
-        width, height = self._getDocSize(dpi)
-
-        if qt4.PYQT_VERSION >= 0x40600:
-            # custom paint devices don't work in old PyQt versions
-
-            f = open(filename, 'w')
-            paintdev = svg_export.SVGPaintDevice(f, width/dpi, height/dpi)
-            painter = Painter(paintdev)
-            self.basewidget.draw(painter, page)
-            painter.end()
-            f.close()
-
-        else:
-            # use built-in svg generation, which doesn't work very well
-            # (no clipping, font size problems)
-            import PyQt4.QtSvg
-
-            # actually paint the image
-            gen = PyQt4.QtSvg.QSvgGenerator()
-            gen.setFileName(filename)
-            gen.setResolution(dpi)
-            gen.setSize( qt4.QSize(int(width), int(height)) )
-            painter = Painter(gen)
-            self.basewidget.draw(painter, page)
-            painter.end()
-
-    def _exportSelfTest(self, filename, page):
-        """Export document for testing"""
-
-        dpi = 90.
-        width, height = self._getDocSize(dpi)
-
-        f = open(filename, 'w')
-        paintdev = selftest_export.SelfTestPaintDevice(f, width/dpi, height/dpi)
-        painter = Painter(paintdev)
-        self.basewidget.draw(painter, page)
-        painter.end()
-        f.close()
-
-    def _exportPIC(self, filename, page):
-        """Export document as SVG"""
-
-        pic = qt4.QPicture()
-        painter = Painter(pic)
-        self.basewidget.draw( painter, page )
-        painter.end()
-        pic.save(filename)
-
-    def _exportEMF(self, filename, page):
-        """Export document as EMF."""
-
-        dpi = 90.
-        width, height = self._getDocSize(dpi)
-
-        paintdev = emf_export.EMFPaintDevice(width/dpi, height/dpi,
-                                             dpi=dpi)
-        painter = Painter(paintdev)
-        self.basewidget.draw( painter, page )
-        painter.end()
-        paintdev.paintEngine().saveFile(filename)
-
-    def export(self, filename, pagenumber, color=True, dpi=100,
-               antialias=True, quality=85, backcolor='#ffffff00'):
-        """Export the figure to the filename."""
-
-        ext = os.path.splitext(filename)[1]
-
-        if ext in ('.eps', '.pdf'):
-            self._exportPS(filename, pagenumber, color=color)
-
-        elif ext in ('.png', '.jpg', '.jpeg', '.bmp'):
-            self._exportBitmap(filename, pagenumber, dpi=dpi,
-                               antialias=antialias, quality=quality,
-                               backcolor=backcolor)
-
-        elif ext == '.svg':
-            self._exportSVG(filename, pagenumber)
-
-        elif ext == '.selftest':
-            self._exportSelfTest(filename, pagenumber)
-
-        elif ext == '.pic':
-            self._exportPIC(filename, pagenumber)
-
-        elif ext == '.emf' and hasemf:
-            self._exportEMF(filename, pagenumber)
-
-        else:
-            raise RuntimeError, "File type '%s' not supported" % ext
-            
-    def getExportFormats(self):
-        """Get list of export formats:
-        [
-        (['ext'], 'Filename type'),
-        ...
-        ]
-        """
-        formats = [(["eps"], "Encapsulated Postscript"),
-                   (["png"], "Portable Network Graphics"),
-                   (["jpg"], "Jpeg bitmap"),
-                   (["bmp"], "Windows bitmap"),
-                   (["pdf"], "Portable Document Format"),
-                   (["svg"], "Scalable Vector Graphics"),
-                   #(["pic"], "QT Pic format"),
-                   ]
-        if hasemf:
-            formats.append( (["emf"], "Windows Enhanced Metafile") )
-        return formats
+    def docSize(self, dpi=None, scaling=1., integer=True):
+        """Get size for document."""
+        return self._pagedocsize(self.basewidget,
+                                 dpi=dpi, scaling=scaling, integer=integer)
 
     def resolveItem(self, fromwidget, where):
         """Resolve item relative to fromwidget.
@@ -902,6 +730,13 @@ class Document( qt4.QObject ):
             else:
                 raise ValueError, 'Invalid custom type'
 
+    def customDict(self):
+        """Return a dictionary mapping custom names to (idx, type, value)."""
+        retn = {}
+        for i, (ctype, name, val) in enumerate(self.customs):
+            retn[name] = (i, ctype, val)
+        return retn
+
     def evalDatasetExpression(self, expr, part='data'):
         """Return results of evaluating a 1D dataset expression.
         part is 'data', 'serr', 'perr' or 'nerr' - these are the
@@ -909,22 +744,44 @@ class Document( qt4.QObject ):
         """
         return datasets.simpleEvalExpression(self, expr, part=part)
 
-class Painter(qt4.QPainter):
-    """A painter which allows the program to know which widget it is
-    currently drawing."""
-    
-    def __init__(self, *args, **argsv):
-        qt4.QPainter.__init__(self, *args)
+    def walkNodes(self, tocall, root=None,
+                  nodetypes=('widget', 'setting', 'settings'),
+                  _path=None):
+        """Walk the widget/settings/setting nodes in the document.
+        For each one call tocall(path, node).
+        nodetypes is tuple of possible node types
+        """
 
-        self.veusz_scaling = argsv.get('scaling', 1.)
-        if 'dpi' in argsv and argsv['dpi'] is not None:
-            self.veusz_pixperpt = argsv['dpi'] / 72.
+        if root is None:
+            root = self.basewidget
+        if _path is None:
+            _path = root.path
 
-    def beginPaintingWidget(self, widget, bounds):
-        """Keep track of the widget currently being painted."""
-        pass
+        if root.nodetype in nodetypes:
+            tocall(_path, root)
 
-    def endPaintingWidget(self):
-        """Widget is now finished."""
-        pass
+        if root.nodetype == 'widget':
+            # get rid of // at start of path
+            if _path == '/':
+                _path = ''
 
+            # do the widget's children
+            for w in root.children:
+                self.walkNodes(tocall, root=w, nodetypes=nodetypes,
+                               _path = _path + '/' + w.name)
+            # then do the widget's settings
+            self.walkNodes(tocall, root=root.settings,
+                           nodetypes=nodetypes, _path=_path)
+        elif root.nodetype == 'settings':
+            # do the settings of the settings
+            for name, s in sorted(root.setdict.iteritems()):
+                self.walkNodes(tocall, root=s, nodetypes=nodetypes,
+                               _path = _path + '/' + s.name)
+        # elif root.nodetype == 'setting': pass
+
+    def getColormap(self, name, invert):
+        """Get colormap with name given (returning grey if does not exist)."""
+        cmap = self.colormaps.get(name, self.colormaps['grey'])
+        if invert:
+            return cmap[::-1]
+        return cmap
