@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: operations.py 1471 2010-12-10 22:18:24Z jeremysanders $
-
 """Represents atomic operations to take place on a document which can be undone.
 Rather than the document modified directly, this interface should be used.
 
@@ -62,7 +60,6 @@ class OperationSettingSet(object):
         
     def do(self, document):
         """Apply setting variable."""
-        
         setting = document.resolveFullSettingPath(self.settingpath)
         if setting.isReference():
             self.oldvalue = setting.getReference()
@@ -72,7 +69,6 @@ class OperationSettingSet(object):
         
     def undo(self, document):
         """Return old value back..."""
-        
         setting = document.resolveFullSettingPath(self.settingpath)
         setting.set(self.oldvalue)
 
@@ -207,6 +203,7 @@ class OperationWidgetDelete(object):
         
         self.oldwidget = document.resolveFullWidgetPath(self.widgetpath)
         oldparent = self.oldwidget.parent
+        self.oldwidget.parent = None
         self.oldparentpath = oldparent.path
         self.oldindex = oldparent.children.index(self.oldwidget)
         oldparent.removeChild(self.oldwidget.name)
@@ -215,8 +212,55 @@ class OperationWidgetDelete(object):
         """Restore deleted widget."""
         
         oldparent = document.resolveFullWidgetPath(self.oldparentpath)
+        self.oldwidget.parent = oldparent
         oldparent.addChild(self.oldwidget, index=self.oldindex)
-                
+
+class OperationWidgetsDelete(object):
+    """Delete mutliple widget."""
+    
+    descr = 'delete'
+    
+    def __init__(self, widgets):
+        """Delete the widget."""
+        self.widgetpaths = [w.path for w in widgets]
+        
+    def do(self, document):
+        """Delete widget."""
+        
+        # ignore widgets which share ancestry
+        # as deleting the parent deletes the child
+        widgetpaths = list(self.widgetpaths)
+        widgetpaths.sort( cmp=lambda a, b: len(a)-len(b) )
+        i = 0
+        while i < len(widgetpaths):
+            wp = widgetpaths[i]
+            for j in xrange(i):
+                if wp[:len(widgetpaths[j])+1] == widgetpaths[j]+'/':
+                    del widgetpaths[i]
+                    break
+            else:
+                i += 1
+
+        self.oldwidgets = []
+        self.oldparentpaths = []
+        self.oldindexes = []
+
+        # delete each widget keeping track of details
+        for path in widgetpaths:
+            self.oldwidgets.append( document.resolveFullWidgetPath(path) )
+            oldparent = self.oldwidgets[-1].parent
+            self.oldparentpaths.append( oldparent.path )
+            self.oldindexes.append( oldparent.children.index(self.oldwidgets[-1]) )
+            oldparent.removeChild(self.oldwidgets[-1].name)
+
+    def undo(self, document):
+        """Restore deleted widget."""
+        
+        # put back widgets in reverse order so that indexes are corrent
+        for i in xrange(len(self.oldwidgets)-1,-1,-1):
+            oldparent = document.resolveFullWidgetPath(self.oldparentpaths[i])
+            oldparent.addChild(self.oldwidgets[i], index=self.oldindexes[i])
+        
 class OperationWidgetMoveUpDown(object):
     """Move a widget up or down in the hierarchy."""
 
@@ -266,6 +310,7 @@ class OperationWidgetMove(object):
         newparent = document.resolveFullWidgetPath(self.newparentpath)
         self.oldchildindex = oldparent.children.index(child)
         self.oldparentpath = oldparent.path
+        self.oldname = None
 
         if self.newindex < 0:
             # convert negative index to normal index
@@ -281,12 +326,22 @@ class OperationWidgetMove(object):
         else:
             # moving to different parent
             self.movemode = 'differentparent'
-            # record previous parent and position
 
+            # remove from old parent
             del oldparent.children[self.oldchildindex]
+
+            # current names of children
+            childnames = newparent.childnames
+
+            # record previous parent and position
             newparent.children.insert(self.newindex, child)
             child.parent = newparent
 
+            # set a new name, if required
+            if child.name in childnames:
+                self.oldname = child.name
+                child.name = child.chooseName()
+
         self.newchildpath = child.path
 
     def undo(self, document):
@@ -296,10 +351,16 @@ class OperationWidgetMove(object):
         child = document.resolveFullWidgetPath(self.newchildpath)
         oldparent = document.resolveFullWidgetPath(self.oldparentpath)
 
+        # remove from new parent
         del newparent.children[self.newindex]
+        # restore parent
         oldparent.children.insert(self.oldchildindex, child)
         child.parent = oldparent
 
+        # restore name
+        if self.oldname is not None:
+            child.name = self.oldname
+
 class OperationWidgetAdd(object):
     """Add a widget of specified type to parent."""
 
@@ -370,7 +431,7 @@ class OperationDatasetSet(object):
     def undo(self, document):
         """Undo the data setting."""
         
-        del document.data[self.datasetname]
+        document.deleteData(self.datasetname)
         if self.olddata is not None:
             document.setData(self.datasetname, self.olddata)
     
@@ -385,7 +446,7 @@ class OperationDatasetDelete(object):
     def do(self, document):
         """Remove dataset from document, but preserve for undo."""
         self.olddata = document.data[self.datasetname]
-        del document.data[self.datasetname]
+        document.deleteData(self.datasetname)
         
     def undo(self, document):
         """Put dataset back"""
@@ -437,9 +498,9 @@ class OperationDatasetDuplicate(object):
         """Delete the duplicate"""
         
         if self.olddata is None:
-            del document.data[self.duplname]
+            document.deleteData(self.duplname)
         else:
-            document.data[self.duplname] = self.olddata
+            document.setData(self.duplname, self.olddata)
         
 class OperationDatasetUnlinkFile(object):
     """Remove association between dataset and file."""
@@ -491,9 +552,9 @@ class OperationDatasetCreate(object):
         
     def undo(self, document):
         """Delete the created dataset."""
-        del document.data[self.datasetname]
+        document.deleteData(self.datasetname)
         if self.olddataset is not None:
-            document.data[self.datasetname] = self.olddataset
+            document.setData(self.datasetname, self.olddataset)
         
 class OperationDatasetCreateRange(OperationDatasetCreate):
     """Create a dataset in a specfied range."""
@@ -658,7 +719,7 @@ class OperationDataset2DBase(object):
 
     def undo(self, document):
         """Undo dataset creation."""
-        del document.data[self.datasetname]
+        document.deleteData(self.datasetname)
         if self.olddataset:
             document.setData(self.datasetname, self.olddataset)
 
@@ -704,6 +765,53 @@ class OperationDataset2DXYFunc(OperationDataset2DBase):
     def makeDSClass(self):
         return datasets.Dataset2DXYFunc(self.xstep, self.ystep, self.expr)
 
+class OperationDatasetUnlinkByFile(object):
+    """Unlink all datasets associated with file."""
+
+    descr = "unlink datasets"
+
+    def __init__(self, filename):
+        """Unlink all datasets associated with filename."""
+        self.filename = filename
+
+    def do(self, document):
+        """Remove links."""
+        self.oldlinks = {}
+        for name, ds in document.data.iteritems():
+            if ds.linked is not None and ds.linked.filename == self.filename:
+                self.oldlinks[name] = ds.linked
+                ds.linked = None
+
+    def undo(self, document):
+        """Restore links."""
+        for name, link in self.oldlinks.iteritems():
+            try:
+                document.data[name].linked = link
+            except KeyError:
+                pass
+
+class OperationDatasetDeleteByFile(object):
+    """Delete all datasets associated with file."""
+
+    descr = "delete datasets"
+
+    def __init__(self, filename):
+        """Delete all datasets associated with filename."""
+        self.filename = filename
+
+    def do(self, document):
+        """Remove datasets."""
+        self.olddatasets = {}
+        for name, ds in document.data.items():
+            if ds.linked is not None and ds.linked.filename == self.filename:
+                self.olddatasets[name] = ds
+                document.deleteData(name)
+
+    def undo(self, document):
+        """Restore datasets."""
+        for name, ds in self.olddatasets.iteritems():
+            document.setData(name, ds)
+
 ###############################################################################
 # Import datasets
         
@@ -773,84 +881,69 @@ class OperationDataImport(object):
         else:
             LF = None
 
-        # backup datasets in document for undo
-        # this has possible space issues!
-        self.olddatasets = dict(document.data)
+        # remember datasets in document for undo
+        olddatasets = dict(document.data)
         
         # actually set the data in the document
-        names = self.simpleread.setInDocument(document, linkedfile=LF,
-                                              prefix=self.prefix,
-                                              suffix=self.suffix)
+        names = self.simpleread.setInDocument(
+            document, linkedfile=LF, prefix=self.prefix, suffix=self.suffix)
+
+        # only remember the parts we need
+        self.olddatasets = dict([ (n, olddatasets.get(n, None)) for n in names])
+        
         return names
         
     def undo(self, document):
         """Undo import."""
         
-        # restore old datasets
-        document.data = self.olddatasets
+        for name, ds in self.olddatasets.iteritems():
+            if ds is None:
+                document.deleteData(name)
+            else:
+                document.setData(name, ds)
 
 class OperationDataImportCSV(object):
     """Import data from a CSV file."""
 
     descr = 'import CSV data'
 
-    def __init__(self, filename, readrows=False,
-                 delimiter=',', textdelimiter='"',
-                 encoding='utf_8',
-                 prefix='', suffix='',
-                 headerignore=0,
-                 linked=False):
-        """Import CSV data from filename
-
-        If readrows, then read in rows rather than columns.
-        Prefix is appended to each dataset name.
+    def __init__(self, params, linked=False):
+        """Import CSV data
+        params is a ParamsCSV object
         Data are linked to file if linked is True.
         """
-
-        self.filename = filename
-        self.readrows = readrows
-        self.delimiter = delimiter
-        self.textdelimiter = textdelimiter
-        self.encoding = encoding
-        self.prefix = prefix
-        self.suffix = suffix
-        self.headerignore = headerignore
         self.linked = linked
+        self.params = params
 
     def do(self, document):
         """Do the data import."""
         
-        csvr = readcsv.ReadCSV(self.filename, readrows=self.readrows,
-                               delimiter=self.delimiter,
-                               textdelimiter=self.textdelimiter,
-                               encoding=self.encoding,
-                               headerignore=self.headerignore,
-                               prefix=self.prefix, suffix=self.suffix)
+        csvr = readcsv.ReadCSV(self.params)
         csvr.readData()
 
         if self.linked:
-            LF = datasets.LinkedCSVFile(self.filename, readrows=self.readrows,
-                                        delimiter=self.delimiter,
-                                        textdelimiter=self.textdelimiter,
-                                        encoding=self.encoding,
-                                        headerignore=self.headerignore,
-                                        prefix=self.prefix, suffix=self.suffix)
+            LF = datasets.LinkedCSVFile(self.params)
         else:
             LF = None
 
-        # backup datasets in document for undo
-        # this has possible space issues!
-        self.olddatasets = dict(document.data)
+        # remember the old datasets for undo
+        olddatasets = dict(document.data)
         
         # set the data
         names = csvr.setData(document, linkedfile=LF)
+        
+        # only remember the parts we need
+        self.olddatasets = dict([ (n, olddatasets.get(n, None)) for n in names])
         return names
 
     def undo(self, document):
         """Undo import."""
         
-        # restore old datasets
-        document.data = self.olddatasets
+        for name, ds in self.olddatasets.iteritems():
+            if ds is None:
+                document.deleteData(name)
+            else:
+                document.setData(name, ds)
         
 class OperationDataImport2D(object):
     """Import a 2D matrix from a file."""
@@ -922,9 +1015,9 @@ class OperationDataImport2D(object):
         else:
             LF = None
 
-        # backup datasets in document for undo
-        self.olddatasets = dict(document.data)
-            
+        # remember datasets in document for undo
+        olddatasets = dict(document.data)
+        
         readds = []
         for name in self.datasets:
             sr = simpleread.SimpleRead2D(name)
@@ -943,13 +1036,22 @@ class OperationDataImport2D(object):
             readds += sr.setInDocument(document, linkedfile=LF,
                                        prefix=self.prefix,
                                        suffix=self.suffix)
+        
+        # only remember the parts we need
+        self.olddatasets = dict([ (n, olddatasets.get(n, None))
+                                  for n in self.datasets ])
+        
         return readds
 
     def undo(self, document):
         """Undo import."""
         
         # restore old datasets
-        document.data = self.olddatasets
+        for name, ds in self.olddatasets.iteritems():
+            if ds is None:
+                document.deleteData(name)
+            else:
+                document.setData(name, ds)
     
 class OperationDataImportFITS(object):
     """Import 1d or 2d data from a fits file."""
@@ -1004,8 +1106,12 @@ class OperationDataImportFITS(object):
         # actually create the dataset
         return datasets.Dataset(data=datav, serr=symv, perr=posv, nerr=negv)
 
-    def _import2d(self, hdu):
-        """Import 2d data from hdu."""
+    def _import1dimage(self, hdu):
+        """Import 1d image data form hdu."""
+        return datasets.Dataset(data=hdu.data)
+
+    def _import2dimage(self, hdu):
+        """Import 2d image data from hdu."""
     
         if ( self.datacol is not None or self.symerrcol is not None or self.poserrcol is not None or
              self.negerrcol is not None ):
@@ -1052,8 +1158,13 @@ class OperationDataImportFITS(object):
             ds = self._import1d(hdu)
 
         except AttributeError:
-            ds = self._import2d(hdu)
-
+            naxis = hdu.header.get('NAXIS')
+            if naxis == 1:
+                ds = self._import1dimage(hdu)
+            elif naxis == 2:
+                ds = self._import2dimage(hdu)
+            else:
+                raise RuntimeError, "Cannot import images with %i dimensions" % naxis
         f.close()
             
         if self.linked:
@@ -1069,12 +1180,10 @@ class OperationDataImportFITS(object):
 
     def undo(self, document):
         """Undo the import."""
-        
-        if self.dsname in document.data:
-            del document.data[self.dsname]
-            
         if self.olddataset is not None:
             document.setData(self.dsname, self.olddataset)
+        else:
+            document.deleteData(self.dsname)
 
 class OperationDataImportPlugin(object):
     """Import data using a plugin."""
@@ -1107,11 +1216,32 @@ class OperationDataImportPlugin(object):
             except KeyError:
                 pass
 
+    def addCustoms(self, document, consts):
+        """Add the customs return by plugins to document."""
+
+        self.oldconst = None
+        if len(consts) > 0:
+            self.oldconst = list(document.customs)
+            cd = document.customDict()
+            for item in consts:
+                if item[1] in cd:
+                    idx, ctype, val = cd[item[1]]
+                    document.customs[idx] = item
+                else:
+                    document.customs.append(item)
+            document.updateEvalContext()
+
     def do(self, document):
         """Do import."""
 
         names = [p.name for p in plugins.importpluginregistry]
         plugin = plugins.importpluginregistry[names.index(self.pluginname)]
+
+        # if the plugin is a class, make an instance
+        # the old API is for the plugin to be instances
+        if isinstance(plugin, type):
+            plugin = plugin()
+
         plugparams = plugins.ImportPluginParams(self.filename, self.encoding,
                                                 self.params)
 
@@ -1127,17 +1257,28 @@ class OperationDataImportPlugin(object):
                 self.pluginname, self.filename, self.params,
                 encoding=self.encoding, prefix=self.prefix,
                 suffix=self.suffix)
-            
+
+        customs = []
+
         # convert results to real datasets
+        names = []
         for d in results:
-            if isinstance(d, plugins.ImportDataset1D):
+            if isinstance(d, plugins.Dataset1D):
                 ds = datasets.Dataset(data=d.data, serr=d.serr, perr=d.perr,
                                       nerr=d.nerr)
-            elif isinstance(d, plugins.ImportDataset2D):
+            elif isinstance(d, plugins.Dataset2D):
                 ds = datasets.Dataset2D(data=d.data, xrange=d.rangex,
                                         yrange=d.rangey)
-            elif isinstance(d, plugins.ImportDatasetText):
+            elif isinstance(d, plugins.DatasetText):
                 ds = datasets.DatasetText(data=d.data)
+            elif isinstance(d, plugins.DatasetDateTime):
+                ds = datasets.DatasetDateTime(data=d.data)
+            elif isinstance(d, plugins.Constant):
+                customs.append( ['constant', d.name, d.val] )
+                continue
+            elif isinstance(d, plugins.Function):
+                customs.append( ['function', d.name, d.val] )
+                continue
             else:
                 raise RuntimeError("Invalid data set in plugin results")
 
@@ -1153,16 +1294,27 @@ class OperationDataImportPlugin(object):
             # actually make dataset
             document.setData(d.name, ds)
 
-        self.datasetnames = [d.name for d in results]
-        return self.datasetnames
+            names.append(d.name)
+
+        # add constants, functions to doc, if any
+        self.addCustoms(document, customs)
+
+        # custom strings
+        custstr = ['%s %s=%s' % tuple(c) for c in customs]
+
+        self.datasetnames = names
+        return self.datasetnames, custstr
 
     def undo(self, document):
         """Undo import."""
 
         for name in self.datasetnames:
-            del document.data[name]
+            document.deleteData(name)
         for name, dataset in self.olddata.iteritems():
             document.setData(name, dataset)
+        if self.oldconst is not None:
+            document.customs = self.oldconst
+            document.updateEvalContext()
 
 class OperationDataCaptureSet(object):
     """An operation for setting the results from a SimpleRead into the
@@ -1197,10 +1349,10 @@ class OperationDataCaptureSet(object):
         for name in self.nameschanged:
             if name in self.olddata:
                 # replace datasets with what was there previously
-                document.data[name] = self.olddata[name]
+                document.setData(name, self.olddata[name])
             else:
                 # or delete datasets that weren't there before
-                del document.data[name]
+                document.deleteData(name)
 
 ###############################################################################
 # Alter dataset
@@ -1258,6 +1410,31 @@ class OperationDatasetSetVal(object):
         datacol[self.row] = self.oldval
         ds.changeValues(self.columnname, datacol)
     
+class OperationDatasetSetVal2D(object):
+    """Set a value in a 2D dataset."""
+
+    descr = 'change 2D dataset value'
+
+    def __init__(self, datasetname, row, col, val):
+        """Set row in column columnname to val."""
+        self.datasetname = datasetname
+        self.row = row
+        self.col = col
+        self.val = val
+
+    def do(self, document):
+        """Set the value."""
+        ds = document.data[self.datasetname]
+        self.oldval = ds.data[self.row, self.col]
+        ds.data[self.row, self.col] = self.val
+        document.modifiedData(ds)
+
+    def undo(self, document):
+        """Restore the value."""
+        ds = document.data[self.datasetname]
+        ds.data[self.row, self.col] = self.oldval
+        document.modifiedData(ds)
+
 class OperationDatasetDeleteRow(object):
     """Delete a row or several in the dataset."""
 
@@ -1380,11 +1557,9 @@ class OperationLoadStyleSheet(OperationMultiple):
         e = None
         try:
             interpreter.runFile( open(self.filename) )
-        except Exception, e:
-            pass
-        document.batchHistory(None)
-        if e:
-            raise e
+        except:
+            document.batchHistory(None)
+            raise
         
 class OperationLoadCustom(OperationLoadStyleSheet):
     descr = 'load custom definitions'
@@ -1411,7 +1586,7 @@ class OperationToolsPlugin(OperationMultiple):
         ifc = commandinterface.CommandInterface(document)
         try:
             self.plugin.apply(ifc, self.fields)
-        except Exception:
+        except:
             document.batchHistory(None)
             raise
         document.batchHistory(None)
@@ -1451,7 +1626,7 @@ class OperationDatasetPlugin(object):
         # add new datasets to document
         for name, ds in izip(names, manager.veuszdatasets):
             if name is not None:
-                document.data[name] = ds
+                document.setData(name, ds)
 
         return names
 
@@ -1465,7 +1640,8 @@ class OperationDatasetPlugin(object):
         # delete datasets which were created
         for name in self.datasetnames:
             if name is not None:
-                del document.data[name]
+                document.deleteData(name)
 
         # put back old datasets
-        document.data.update(self.olddata)
+        for name, ds in self.olddata.iteritems():
+            document.setData(name, ds)
