@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: simpleread.py 1355 2010-08-12 19:38:13Z jeremysanders $
-
 """SimpleRead: a class for the reading of data formatted in a simple way
 
 To read the data it takes a descriptor which takes the form of
@@ -240,7 +238,6 @@ class DescriptorPart(object):
                 val = stream.nextColumn()
                 if val is None:
                     return
-
                 # append a suffix to specify whether error or value
                 # \0 is used as the user cannot enter it
                 fullname = '%s\0%s' % (name, col)
@@ -314,7 +311,7 @@ class DescriptorPart(object):
                         minlength = len(ds)
                 for ds in vals, pos, neg, sym:
                     if ds is not None and len(ds) != minlength:
-                        ds = ds[:minlength]
+                        del ds[minlength:]
 
                 # only remember last N values
                 if tail is not None:
@@ -324,10 +321,13 @@ class DescriptorPart(object):
                     if neg is not None: neg = neg[-tail:]
 
                 # create the dataset
-                if self.datatype in ('float', 'date'):
-                    ds = datasets.Dataset(data = vals, serr = sym,
-                                          nerr = neg, perr = pos,
-                                          linked = linkedfile)
+                if self.datatype == 'float':
+                    ds = datasets.Dataset( data = vals, serr = sym,
+                                           nerr = neg, perr = pos,
+                                           linked = linkedfile )
+                elif self.datatype == 'date':
+                    ds = datasets.DatasetDateTime( data=vals,
+                                                   linked=linkedfile )
                 elif self.datatype == 'string':
                     ds = datasets.DatasetText( data=vals,
                                                linked = linkedfile )
@@ -343,19 +343,22 @@ class DescriptorPart(object):
         return names
 
 class Stream(object):
+    """This object reads through an input data source (override
+    readLine) and interprets data from the source."""
                                     
-    # this regular expression is for splitting up the stream into words
+    # this is a regular expression for finding data items in data stream
     # I'll try to explain this bit-by-bit (these are ORd, and matched in order)
-    split_re = re.compile( r'''
-    `.+?`[^ \t\n#!%;]* | # match dataset name quoted in back-ticks
-                         # we also need to match following characters to catch
-                         # corner cases in the descriptor
+    find_re = re.compile( r'''
+    `.+?`[^ \t\n\r#!%;]* | # match dataset name quoted in back-ticks
+                           # we also need to match following characters to catch
+                           # corner cases in the descriptor
     u?"" |          # match empty double-quoted string
     u?".*?[^\\]" |  # match double-quoted string, ignoring escaped quotes
     u?'' |          # match empty single-quoted string
     u?'.*?[^\\]' |  # match single-quoted string, ignoring escaped quotes
+    [#!%;](?=descriptor) | # match separately comment char before descriptor
     [#!%;].* |      # match comment to end of line
-    [^ \t\n#!%;]+   # match normal space/tab separated items
+    [^ \t\n\r#!%;]+ # match normal space/tab separated items
     ''', re.VERBOSE )
 
     def __init__(self):
@@ -394,8 +397,8 @@ class Stream(object):
                 return False
 
             # break up and append to buffer (removing comments)
-            self.remainingline += [ x for x in self.split_re.findall(line) if
-                                    x[0] not in '#!%;' ]
+            cmpts = self.find_re.findall(line)
+            self.remainingline += [ x for x in cmpts if x[0] not in '#!%;']
 
             if self.remainingline and self.remainingline[-1] == '\\':
                 # this is a continuation: drop this item and read next line
@@ -432,9 +435,16 @@ class SimpleRead(object):
 
     tail attribute if set says to only use last tail data points when setting
     '''
-    
+
     def __init__(self, descriptor):
+        # convert descriptor to part objects
+        descriptor = descriptor.strip()
         self._parseDescriptor(descriptor)
+
+        # construct data names automatically
+        self.autodescr = (descriptor == '')
+
+        # get read for reading data
         self.clearState()
 
     def clearState(self):
@@ -442,10 +452,9 @@ class SimpleRead(object):
         self.datasets = {}
         self.blocks = None
         self.tail = None
-        
+
     def _parseDescriptor(self, descriptor):
         """Take a descriptor, and parse it into its individual parts."""
-
         self.parts = interpretDescriptor(descriptor)
 
     def readData(self, stream, useblocks=False, ignoretext=False):
@@ -465,7 +474,7 @@ class SimpleRead(object):
     def _readDataUnblocked(self, stream, ignoretext):
         """Read in that data from the stream."""
 
-        allparts = self.parts
+        allparts = list(self.parts)
 
         # loop over lines
         while stream.newLine():
@@ -474,6 +483,7 @@ class SimpleRead(object):
                 descriptor =  ' '.join(stream.remainingline[1:])
                 self._parseDescriptor(descriptor)
                 allparts += self.parts
+                self.autodescr = False
             elif ( self.ignoretext and len(stream.remainingline) > 0 and 
                    text_start_re.match(stream.remainingline[0]) and
                    len(self.parts) > 0 and
@@ -486,6 +496,16 @@ class SimpleRead(object):
                 # normal text
                 for p in self.parts:
                     p.readFromStream(stream, self.datasets)
+
+                # automatically create parts if data are remaining
+                if self.autodescr:
+                    while len(stream.remainingline) > 0:
+                        p = DescriptorPart(
+                            str(len(self.parts)+1), None, 'D', None )
+                        p.readFromStream(stream, self.datasets)
+                        self.parts.append(p)
+                        allparts.append(p)
+
             stream.flushLine()
 
         self.parts = allparts
@@ -497,11 +517,11 @@ class SimpleRead(object):
         blocks = {}
         block = 1
         while stream.newLine():
-            l = stream.remainingline
+            line = stream.remainingline
 
             # if this is a blank line, separating data then advance to a new
             # block
-            if len(l) == 0 or l[0].lower() == 'no':
+            if len(line) == 0 or line[0].lower() == 'no':
                 # blank lines separate blocks
                 if block in blocks:
                     block += 1
@@ -509,6 +529,16 @@ class SimpleRead(object):
                 # read in data
                 for p in self.parts:
                     p.readFromStream(stream, self.datasets, block=block)
+
+                # automatically create parts if data are remaining
+                if self.autodescr:
+                    while len(stream.remainingline) > 0:
+                        p = DescriptorPart(
+                            str(len(self.parts)+1), None, 'D', None )
+                        p.readFromStream(stream, self.datasets, block=block)
+                        self.parts.append(p)
+                        allparts.append(p)
+
                 blocks[block] = True
 
             # lose remaining data
@@ -548,14 +578,19 @@ class SimpleRead(object):
         else:
             blocks = self.blocks
 
+        # if automatically making parts, use a prefix/suffix if not set
+        if self.autodescr and prefix == '' and suffix == '':
+            prefix = 'col'
+
         names = []
         for block in blocks:
             for part in self.parts:
-                names += part.setInDocument(self.datasets, document,
-                                            block=block,
-                                            linkedfile=linkedfile,
-                                            prefix=prefix, suffix=suffix,
-                                            tail=self.tail)
+                names += part.setInDocument(
+                    self.datasets, document,
+                    block=block,
+                    linkedfile=linkedfile,
+                    prefix=prefix, suffix=suffix,
+                    tail=self.tail)
 
         return names
 
