@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #    Copyright (C) 2005 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -16,8 +17,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: controls.py 1469 2010-12-08 22:15:00Z jeremysanders $
-
 """Module for creating QWidgets for the settings, to enable their values
    to be changed.
 
@@ -27,10 +26,12 @@
 
 from itertools import izip
 import re
+import numpy as N
 
 import veusz.qtall as qt4
 
 import setting
+
 import veusz.utils as utils
 
 def styleClear(widget):
@@ -42,6 +43,15 @@ def styleError(widget):
     widget.setStyleSheet("background-color: " +
                          setting.settingdb.color('error').name() )
 
+class DotDotButton(qt4.QPushButton):
+    """A button for opening up more complex editor."""
+    def __init__(self, tooltip=None, checkable=True):
+        qt4.QPushButton.__init__(self, "..", flat=True, checkable=checkable,
+                                 maximumWidth=16)
+        if tooltip:
+            self.setToolTip(tooltip)
+        self.setSizePolicy(qt4.QSizePolicy.Maximum, qt4.QSizePolicy.Maximum)
+
 class Edit(qt4.QLineEdit):
     """Main control for editing settings which are text."""
 
@@ -69,11 +79,7 @@ class Edit(qt4.QLineEdit):
         try:
             val = self.setting.fromText(text)
             styleClear(self)
-
-            # value has changed
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'),
-                           self, self.setting, val )
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
         except setting.InvalidType:
             styleError(self)
@@ -107,7 +113,7 @@ class _EditBox(qt4.QTextEdit):
         if readonly:
             self.setReadOnly(True)
 
-        self.positionSelf(parent)
+        utils.positionFloatingPopup(self, parent)
 
         self.installEventFilter(self)
 
@@ -138,42 +144,13 @@ class _EditBox(qt4.QTextEdit):
         """A reasonable size for the text editor."""
         return qt4.QSize(self.spacing*40, self.spacing*3)
 
-    def positionSelf(self, widget):
-        """Open the edit box below the widget."""
-
-        pos = widget.parentWidget().mapToGlobal( widget.pos() )
-        desktop = qt4.QApplication.desktop()
-
-        # recalculates out position so that size is correct below
-        self.adjustSize()
-
-        # is there room to put this widget besides the widget?
-        if pos.y() + self.height() + 1 < desktop.height():
-            # put below
-            y = pos.y() + 1
-        else:
-            # put above
-            y = pos.y() - self.height() - 1
-        
-        # is there room to the left for us?
-        if ( (pos.x() + widget.width() + self.width() < desktop.width()) or
-             (pos.x() + widget.width() < desktop.width()/2) ):
-            # put left justified with widget
-            x = pos.x() + widget.width()
-        else:
-            # put extending to left
-            x = pos.x() - self.width() - 1
-
-        self.move(x, y)
-        self.setFocus()
-
     def closeEvent(self, event):
         """Tell the calling widget that we are closing, and provide
         the new text."""
 
         text = unicode(self.toPlainText())
         text = text.replace('\n', '')
-        self.emit( qt4.SIGNAL('closing'), text)
+        self.emit(qt4.SIGNAL('closing'), text)
         event.accept()
 
 class String(qt4.QWidget):
@@ -191,11 +168,7 @@ class String(qt4.QWidget):
         self.edit = qt4.QLineEdit()
         layout.addWidget(self.edit)
 
-        b = self.button = qt4.QPushButton('..')
-        b.setFlat(True)
-        b.setSizePolicy(qt4.QSizePolicy.Maximum, qt4.QSizePolicy.Maximum)
-        b.setMaximumWidth(16)
-        b.setCheckable(True)
+        b = self.button = DotDotButton(tooltip="Edit text")
         layout.addWidget(b)
 
         # set the text of the widget to the 
@@ -242,10 +215,7 @@ class String(qt4.QWidget):
         try:
             val = self.setting.fromText(text)
             styleClear(self.edit)
-
-            # value has changed
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val)
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
         except setting.InvalidType:
             styleError(self.edit)
@@ -260,6 +230,7 @@ class Int(qt4.QSpinBox):
     def __init__(self, setting, parent):
         qt4.QSpinBox.__init__(self, parent)
 
+        self.ignorechange = False
         self.setting = setting
         self.setMinimum(setting.minval)
         self.setMaximum(setting.maxval)
@@ -273,11 +244,15 @@ class Int(qt4.QSpinBox):
 
     def slotChanged(self, value):
         """If check box changes."""
-        self.emit(qt4.SIGNAL('settingChanged'), self, self.setting, value)
+        # this is emitted by setValue, so ignore onModified doing this
+        if not self.ignorechange:
+            self.emit(qt4.SIGNAL('settingChanged'), self, self.setting, value)
 
     def onModified(self, mod):
         """called when the setting is changed remotely"""
+        self.ignorechange = True
         self.setValue( self.setting.val )
+        self.ignorechange = False
 
 class Bool(qt4.QCheckBox):
     """A check box for changing a bool setting."""
@@ -285,6 +260,7 @@ class Bool(qt4.QCheckBox):
     def __init__(self, setting, parent):
         qt4.QCheckBox.__init__(self, parent)
 
+        self.ignorechange = False
         self.setting = setting
         self.setChecked(setting.val)
 
@@ -299,11 +275,15 @@ class Bool(qt4.QCheckBox):
 
     def slotToggled(self, state):
         """Emitted when checkbox toggled."""
-        self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, state )
+        # this is emitted by setChecked, so ignore onModified doing this
+        if not self.ignorechange:
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, state )
         
     def onModified(self, mod):
         """called when the setting is changed remotely"""
+        self.ignorechange = True
         self.setChecked( self.setting.val )
+        self.ignorechange = False
 
 class BoolSwitch(Bool):
     """Bool for switching off/on other settings."""
@@ -374,6 +354,10 @@ class Choice(qt4.QComboBox):
         if setting.readonly:
             self.setEnabled(False)
 
+        # make completion case sensitive (to help fix case typos)
+        if self.completer():
+            self.completer().setCaseSensitivity(qt4.Qt.CaseSensitive)
+
     def focusOutEvent(self, *args):
         """Allows us to check the contents of the widget."""
         qt4.QComboBox.focusOutEvent(self, *args)
@@ -386,10 +370,7 @@ class Choice(qt4.QComboBox):
         try:
             val = self.setting.fromText(text)
             styleClear(self)
-            
-            # value has changed
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
         except setting.InvalidType:
             styleError(self)
@@ -431,10 +412,7 @@ class MultiLine(qt4.QTextEdit):
         try:
             val = self.setting.fromText(text)
             styleClear(self)
-            
-            # value has changed
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
         except setting.InvalidType:
             styleError(self)
@@ -448,7 +426,7 @@ class Distance(Choice):
 
     # used to remove non-numerics from the string
     # we also remove X/ from X/num
-    stripnumre = re.compile(r"[0-9]*/|[^0-9.]")
+    stripnumre = re.compile(r"[0-9]*/|[^0-9.,]")
 
     # remove spaces
     stripspcre = re.compile(r"\s")
@@ -519,7 +497,7 @@ class DistancePt(Choice):
         '''Initialise with blank list, then populate with sensible units.'''
         Choice.__init__(self, setting, True, DistancePt.points, parent)
         
-class Dataset(Choice):
+class Dataset(qt4.QWidget):
     """Allow the user to choose between the possible datasets."""
 
     def __init__(self, setting, document, dimensions, datatype, parent):
@@ -529,13 +507,33 @@ class Dataset(Choice):
 
         Changes on the document refresh the list of datasets."""
         
-        Choice.__init__(self, setting, True, [], parent)
+        qt4.QWidget.__init__(self, parent)
+
+        self.choice = Choice(setting, True, [], None)
+        self.connect( self.choice, qt4.SIGNAL("settingChanged"),
+                      self.slotSettingChanged )
+
+        b = self.button = DotDotButton(tooltip="Select using dataset browser")
+        self.connect(b, qt4.SIGNAL("toggled(bool)"),
+                     self.slotButtonToggled)
+
         self.document = document
         self.dimensions = dimensions
         self.datatype = datatype
         self.lastdatasets = None
         self._populateEntries()
-        self.connect(document, qt4.SIGNAL('sigModified'), self.slotModified)
+        self.connect(document, qt4.SIGNAL("sigModified"), self.slotModified)
+
+        layout = qt4.QHBoxLayout()
+        layout.setSpacing(0)
+        layout.setMargin(0)
+        layout.addWidget(self.choice)
+        layout.addWidget(b)
+        self.setLayout(layout)
+
+    def slotSettingChanged(self, *args):
+        """Reemit setting changed signal if combo box changes."""
+        self.emit( qt4.SIGNAL("settingChanged"), *args )
 
     def _populateEntries(self):
         """Put the list of datasets into the combobox."""
@@ -548,67 +546,69 @@ class Dataset(Choice):
         datasets.sort()
 
         if datasets != self.lastdatasets:
-            utils.populateCombo(self, datasets)
+            utils.populateCombo(self.choice, datasets)
             self.lastdatasets = datasets
 
     def slotModified(self, modified):
         """Update the list of datasets if the document is modified."""
         self._populateEntries()
 
-class DatasetOrString(qt4.QWidget):
+    def slotButtonToggled(self, on):
+        """Bring up list of datasets."""
+        if on:
+            from veusz.qtwidgets.datasetbrowser import DatasetBrowserPopup
+            d = DatasetBrowserPopup(self.document,
+                                    unicode(self.choice.currentText()),
+                                    self.button,
+                                    filterdims=set((self.dimensions,)),
+                                    filterdtype=set((self.datatype,)) )
+            self.connect(d, qt4.SIGNAL("closing"), self.boxClosing)
+            self.connect(d, qt4.SIGNAL("newdataset"), self.newDataset)
+            d.show()
+
+    def boxClosing(self):
+        """Called when the popup edit box closes."""
+        self.button.setChecked(False)
+
+    def newDataset(self, dsname):
+        """New dataset selected."""
+        self.emit( qt4.SIGNAL("settingChanged"), self,
+                   self.choice.setting, dsname )
+
+class DatasetOrString(Dataset):
     """Allow use to choose a dataset or enter some text."""
 
     def __init__(self, setting, document, dimensions, datatype, parent):
-        qt4.QWidget.__init__(self, parent)
-        self.datachoose = Dataset(setting, document, dimensions, datatype,
-                                  None)
-        
-        b = self.button = qt4.QPushButton('..')
-        b.setFlat(True)
-        b.setSizePolicy(qt4.QSizePolicy.Maximum, qt4.QSizePolicy.Maximum)
-        b.setMaximumHeight(self.datachoose.height())
-        b.setMaximumWidth(16)
+        Dataset.__init__(self, setting, document, dimensions, datatype, parent)
+
+        b = self.textbutton = DotDotButton()
         b.setCheckable(True)
+        self.layout().addWidget(b)
+        self.connect(b, qt4.SIGNAL('toggled(bool)'), self.textButtonToggled)
 
-        layout = qt4.QHBoxLayout()
-        self.setLayout(layout)
-        layout.setSpacing(0)
-        layout.setMargin(0)
-        layout.addWidget(self.datachoose)
-        layout.addWidget(b)
-
-        self.connect(b, qt4.SIGNAL('toggled(bool)'),
-                     self.buttonToggled)
-        self.connect(self.datachoose, qt4.SIGNAL('settingChanged'),
-                     self.slotSettingChanged)
-
-    def slotSettingChanged(self, *args):
-        """When datachoose changes, inform any listeners."""
-        self.emit( qt4.SIGNAL('settingChanged'), *args )
-        
-    def buttonToggled(self, on):
+    def textButtonToggled(self, on):
         """Button is pressed to bring popup up / down."""
 
         # if button is down and there's no existing popup, bring up a new one
         if on:
-            e = _EditBox( unicode(self.datachoose.currentText()),
-                          self.datachoose.setting.readonly, self.button)
+            e = _EditBox( unicode(self.choice.currentText()),
+                          self.choice.setting.readonly, self.textbutton)
 
             # we get notified with text when the popup closes
-            self.connect(e, qt4.SIGNAL('closing'), self.boxClosing)
+            self.connect(e, qt4.SIGNAL("closing"), self.textBoxClosing)
             e.show()
 
-    def boxClosing(self, text):
+    def textBoxClosing(self, text):
         """Called when the popup edit box closes."""
 
-        # update the text if we can
-        if not self.datachoose.setting.readonly:
-            self.datachoose.setEditText(text)
-            self.datachoose.setFocus()
-            self.parentWidget().setFocus()
-            self.datachoose.setFocus()
+        self.textbutton.setChecked(False)
 
-        self.button.setChecked(False)
+        # update the text if we can
+        if not self.choice.setting.readonly:
+            self.choice.setEditText(text)
+            self.choice.setFocus()
+            self.parentWidget().setFocus()
+            self.choice.setFocus()
 
 class FillStyle(Choice):
     """For choosing between fill styles."""
@@ -735,6 +735,7 @@ class LineStyle(Choice):
 
         # import later for dependency issues
         import veusz.setting.collections
+        import veusz.document
 
         icons = []
         size = cls.size
@@ -745,12 +746,15 @@ class LineStyle(Choice):
         for lstyle in cls._lines:
             pix = qt4.QPixmap(*size)
             pix.fill()
+
+            ph = veusz.document.PaintHelper( (1, 1) )
+
             painter = qt4.QPainter(pix)
             painter.setRenderHint(qt4.QPainter.Antialiasing)
 
             setn.get('style').set(lstyle)
             
-            painter.setPen( setn.makeQPen(painter) )
+            painter.setPen( setn.makeQPen(ph) )
             painter.drawLine( int(size[0]*0.1), size[1]/2,
                               int(size[0]*0.9), size[1]/2 )
             painter.end()
@@ -843,19 +847,14 @@ class Color(qt4.QWidget):
         if col.isValid():
             # change setting
             val = unicode( col.name() )
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self,
-                           self.setting, val)
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
     def slotActivated(self, val):
         """A different value is selected."""
         
         text = unicode(self.combo.currentText())
         val = self.setting.fromText(text)
-            
-        # value has changed
-        if self.setting.val != val:
-            self.emit(qt4.SIGNAL('settingChanged'), self, self.setting, val)
+        self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
     def setColor(self, color):
         """Update control with color given."""
@@ -894,8 +893,8 @@ class WidgetSelector(Choice):
         """Update list of axes."""
         self._populateEntries()
 
-class Image(WidgetSelector):
-    """Choose an image."""
+class WidgetChoice(WidgetSelector):
+    """Choose a widget."""
 
     def __init__(self, setting, document, parent):
         """Initialise and populate combobox."""
@@ -904,12 +903,12 @@ class Image(WidgetSelector):
         self._populateEntries()
 
     def _populateEntries(self):
-        """Build up a list of images for combobox."""
+        """Build up a list of widgets for combobox."""
 
-        images = self.setting.getImageList()
+        widgets = self.setting.getWidgetList()
 
         # we only need the list of names
-        names = images.keys()
+        names = widgets.keys()
         names.sort()
 
         utils.populateCombo(self, names)
@@ -1483,12 +1482,9 @@ class Filename(qt4.QWidget):
         self.edit.setText( setting.toText() )
         layout.addWidget(self.edit)
         
-        # get a sensible shape for the button - yawn
-        b = self.button = qt4.QPushButton('..')
-        b.setFlat(True)
+        b = self.button = DotDotButton(checkable=False,
+                                       tooltip="Browse for file")
         layout.addWidget(b)
-        b.setSizePolicy(qt4.QSizePolicy.Maximum, qt4.QSizePolicy.Maximum)
-        b.setMaximumWidth(16)
 
         # connect up signals
         self.connect(self.edit, qt4.SIGNAL('editingFinished()'),
@@ -1524,9 +1520,7 @@ class Filename(qt4.QWidget):
 
         if filename:
             val = unicode(filename)
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self, self.setting,
-                           val )
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
     def validateAndSet(self):
         """Check the text is a valid setting and update it."""
@@ -1535,11 +1529,7 @@ class Filename(qt4.QWidget):
         try:
             val = self.setting.fromText(text)
             styleClear(self.edit)
-
-            # value has changed
-            if self.setting.val != val:
-                self.emit( qt4.SIGNAL('settingChanged'), self, self.setting,
-                           val )
+            self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
         except setting.InvalidType:
             styleError(self.edit)
@@ -1579,8 +1569,7 @@ class FontFamily(qt4.QFontComboBox):
     def slotActivated(self, val):
         """Update setting if a different item is chosen."""
         newval = unicode(self.currentText())
-        if self.setting.val != newval:
-            self.emit(qt4.SIGNAL('settingChanged'), self, self.setting, newval)
+        self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, newval )
 
     def onModified(self, mod):
         """Make control reflect chosen setting."""
@@ -1605,3 +1594,73 @@ class ErrorStyle(Choice):
         cls._icons = []
         for errstyle in cls._errorstyles:
             cls._icons.append( utils.getIcon('error_%s' % errstyle) )
+
+class Colormap(Choice):
+    """Give the user a preview of colormaps.
+
+    Based on Choice to make life easier
+    """
+
+    _icons = {}
+
+    size = (32, 12)
+
+    def __init__(self, setn, document, parent):
+        names = sorted(document.colormaps.keys())
+
+        icons = Colormap._generateIcons(document, names)
+        setting.controls.Choice.__init__(self, setn, True,
+                                         names, parent,
+                                         icons=icons)
+        self.setIconSize( qt4.QSize(*self.size) )
+
+    @classmethod
+    def _generateIcons(kls, document, names):
+        """Generate a list of icons for drop down menu."""
+
+        # create a fake dataset smoothly varying from 0 to size[0]-1
+        size = kls.size
+        fakedataset = N.fromfunction(lambda x, y: y, (size[1], size[0]))
+
+        # keep track of icons to return
+        retn = []
+
+        # iterate over colour maps
+        for name in names:
+            val = document.colormaps.get(name, None)
+            if val in kls._icons:
+                icon = kls._icons[val]
+            else:
+                if val is None:
+                    # empty icon
+                    pixmap = qt4.QPixmap(*size)
+                    pixmap.fill(qt4.Qt.transparent)
+                else:
+                    # generate icon
+                    image = utils.applyColorMap(val, 'linear',
+                                                fakedataset,
+                                                0., size[0]-1., 0)
+                    pixmap = qt4.QPixmap.fromImage(image)
+                icon = qt4.QIcon(pixmap)
+                kls._icons[val] = icon
+            retn.append(icon)
+        return retn
+
+class AxisBound(Choice):
+    """Control for setting bounds of axis.
+
+    This is to allow dates etc
+    """
+
+    def __init__(self, setting, *args):
+        Choice.__init__(self, setting, True, ['Auto'], *args)
+
+        modesetn = setting.parent.get('mode')
+        modesetn.setOnModified(self.modeChange)
+
+    def modeChange(self, changed):
+        """Called if the mode of the axis changes.
+        Re-set text as float or date."""
+
+        if unicode(self.currentText()).lower() != 'auto':
+            self.setEditText( self.setting.toText() )
