@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: setting.py 1469 2010-12-08 22:15:00Z jeremysanders $
-
 """Module for holding setting values.
 
 e.g.
@@ -34,7 +32,7 @@ import numpy as N
 import veusz.qtall as qt4
 
 import controls
-from settingdb import settingdb
+from settingdb import settingdb, uilocale
 from reference import Reference
 
 import veusz.utils as utils
@@ -44,6 +42,11 @@ class InvalidType(Exception):
     pass
 
 class Setting(object):
+    """A class to store a value with a particular type."""
+
+    # differentiate widgets, settings and setting
+    nodetype = 'setting'
+
     typename = 'setting'
 
     def __init__(self, name, value, descr='', usertext='',
@@ -55,6 +58,7 @@ class Setting(object):
         descr:  description of the setting
         usertext: name of setting for user
         formatting: whether setting applies to formatting
+        hidden: hide widget from user
         """
         self.readonly = False
         self.parent = None
@@ -62,6 +66,7 @@ class Setting(object):
         self.descr = descr
         self.usertext = usertext
         self.formatting = formatting
+        self.hidden = hidden
         self.default = value
         self.onmodified = qt4.QObject()
         self._val = None
@@ -92,6 +97,7 @@ class Setting(object):
         opt['descr'] = self.descr
         opt['usertext'] = self.usertext
         opt['formatting'] = self.formatting
+        opt['hidden'] = self.hidden
 
         obj = self.__class__(*args, **opt)
         obj.readonly = self.readonly
@@ -445,7 +451,6 @@ class Bool(Setting):
     def makeControl(self, *args):
         return controls.Bool(self, *args)
 
-
 # Storing integers
 class Int(Setting):
     """Integer settings."""
@@ -481,21 +486,30 @@ class Int(Setting):
         raise InvalidType
 
     def toText(self):
-        return str(self.val)
+        return unicode( uilocale.toString(self.val) )
 
     def fromText(self, text):
-        try:
-            i = int(text)
-            if i >= self.minval and i <= self.maxval:
-                return i
-            else:
-                raise InvalidType, 'Out of range allowed'
-        except ValueError:
-            raise InvalidType
+        i, ok = uilocale.toLongLong(text)
+        if not ok:
+            raise ValueError
+
+        if i >= self.minval and i <= self.maxval:
+            return i
+        else:
+            raise InvalidType, 'Out of range allowed'
 
     def makeControl(self, *args):
         return controls.Int(self, *args)
 
+def _finiteRangeFloat(f, minval=-1e300, maxval=1e300):
+    """Return a finite float in range or raise exception otherwise."""
+    f = float(f)
+    if not N.isfinite(f):
+        raise InvalidType, 'Finite values only allowed'
+    if f < minval or f > maxval:
+        raise InvalidType, 'Out of range allowed'
+    return f
+
 # for storing floats
 class Float(Setting):
     """Float settings."""
@@ -524,19 +538,16 @@ class Float(Setting):
 
     def convertTo(self, val):       
         if isinstance(val, int) or isinstance(val, float):
-            if val >= self.minval and val <= self.maxval:
-                return float(val)
-            else:
-                raise InvalidType, 'Out of range allowed'
+            return _finiteRangeFloat(val,
+                                     minval=self.minval, maxval=self.maxval)
         raise InvalidType
 
     def toText(self):
-        return str(self.val)
+        return unicode(uilocale.toString(self.val))
 
     def fromText(self, text):
-        try:
-            f = float(text)
-        except ValueError:
+        f, ok = uilocale.toDouble(text)
+        if not ok:
             # try to evaluate
             f = self.safeEvalHelper(text)
         return self.convertTo(f)
@@ -551,7 +562,7 @@ class FloatOrAuto(Setting):
 
     def convertTo(self, val):
         if type(val) in (int, float):
-            return float(val)
+            return _finiteRangeFloat(val)
         elif isinstance(val, basestring) and val.strip().lower() == 'auto':
             return None
         else:
@@ -564,18 +575,20 @@ class FloatOrAuto(Setting):
             return val
 
     def toText(self):
-        if self.val is None:
+        if self.val is None or (isinstance(self.val, basestring) and
+                                self.val.lower() == 'auto'):
             return 'Auto'
         else:
-            return str(self.val)
+            return unicode(uilocale.toString(self.val))
 
     def fromText(self, text):
         if text.strip().lower() == 'auto':
             return 'Auto'
         else:
-            try:
-                return float(text)
-            except ValueError:
+            f, ok = uilocale.toDouble(text)
+            if ok:
+                return self.convertTo(f)
+            else:
                 # try to evaluate
                 return self.safeEvalHelper(text)
 
@@ -602,19 +615,20 @@ class IntOrAuto(Setting):
             return val
 
     def toText(self):
-        if self.val is None:
+        if self.val is None or (isinstance(self.val, basestring) and
+                                self.val.lower() == 'auto'):
             return 'Auto'
         else:
-            return str(self.val)
+            return unicode( uilocale.toString(self.val) )
 
     def fromText(self, text):
         if text.strip().lower() == 'auto':
             return 'Auto'
         else:
-            try:
-                return int(text)
-            except ValueError:
+            i, ok = uilocale.toLongLong(text)
+            if not ok:
                 raise InvalidType
+            return i
             
     def makeControl(self, *args):
         return controls.Choice(self, True, ['Auto'], *args)
@@ -622,128 +636,124 @@ class IntOrAuto(Setting):
 # these are functions used by the distance setting below.
 # they don't work as class methods
 
-def _calcPixPerPt(painter):
-    """Calculate the numbers of pixels per point for the painter.
-
-    This is stored in the variable veusz_pixperpt."""
-
-    dpi = painter.device().logicalDpiY()
-    if dpi == 0: dpi = 72
-    painter.veusz_pixperpt = dpi / 72.
-
 def _distPhys(match, painter, mult):
     """Convert a physical unit measure in multiples of points."""
-
-    if not hasattr(painter, 'veusz_pixperpt'):
-        _calcPixPerPt(painter)
-
-    return (painter.veusz_pixperpt * mult *
-            float(match.group(1)) * painter.veusz_scaling)
+    return (painter.pixperpt * mult *
+            float(match.group(1)) * painter.scaling)
 
 def _distInvPhys(pixdist, painter, mult, unit):
     """Convert number of pixels into physical distance."""
-    dist = pixdist / (mult * painter.veusz_pixperpt *
-                      painter.veusz_scaling)
+    dist = pixdist / (mult * painter.pixperpt * painter.scaling)
     return "%.3g%s" % (dist, unit)
 
-def _distPerc(match, painter, maxsize):
+def _distPerc(match, painter):
     """Convert from a percentage of maxsize."""
-    return maxsize * 0.01 * float(match.group(1))
+    return painter.maxsize * 0.01 * float(match.group(1))
 
-def _distInvPerc(pixdist, painter, maxsize):
+def _distInvPerc(pixdist, painter):
     """Convert pixel distance into percentage."""
-    perc = pixdist * 100. / maxsize
+    perc = pixdist * 100. / painter.maxsize
     return "%.3g%%" % perc
 
-def _distFrac(match, painter, maxsize):
+def _distFrac(match, painter):
     """Convert from a fraction a/b of maxsize."""
-    return maxsize * float(match.group(1)) / float(match.group(2))
+    try:
+        return painter.maxsize * float(match.group(1))/float(match.group(4))
+    except ZeroDivisionError:
+        return 0.
 
-def _distRatio(match, painter, maxsize):
+def _distRatio(match, painter):
     """Convert from a simple 0.xx ratio of maxsize."""
 
     # if it's greater than 1 then assume it's a point measurement
     if float(match.group(1)) > 1.:
         return _distPhys(match, painter, 1)
 
-    return maxsize * float(match.group(1))
+    return painter.maxsize * float(match.group(1))
+
+# regular expression to match distances
+distre_expr = r'''^
+ [ ]*                                # optional whitespace
+
+ (\.?[0-9]+|[0-9]+\.[0-9]*)          # a floating point number
+
+ [ ]*                                # whitespace
+
+ (cm|pt|mm|inch|in|"|%||             # ( unit, no unit,
+  (?P<slash>/) )                     # or / )
+
+ (?(slash)[ ]*                       # if it was a slash, match any whitespace
+  (\.?[0-9]+|[0-9]+\.[0-9]*))        # and match following fp number
+
+ [ ]*                                # optional whitespace
+$'''
 
 class Distance(Setting):
     """A veusz distance measure, e.g. 1pt or 3%."""
 
     typename = 'distance'
 
-    # mappings from regular expressions to function to convert distance
-    # the recipient function takes regexp match,
-    # painter and maximum size of frac
+    # match a distance
+    distre = re.compile(distre_expr, re.VERBOSE)
 
-    # the second function is to do the inverse calculation
-    distregexp = [
-        # cm distance
-        ( re.compile('^([0-9\.]+) *cm$'),
-          lambda match, painter, t:
-              _distPhys(match, painter, 28.452756),
-          lambda pixdist, painter, t:
-              _distInvPhys(pixdist, painter, 28.452756, 'cm') ),
+    # functions to convert from unit values to pixels
+    unit_func = {
+        'cm': lambda match, painter:
+            _distPhys(match, painter, 28.452756),
+        'pt': lambda match, painter:
+            _distPhys(match, painter, 1.),
+        'mm': lambda match, painter:
+            _distPhys(match, painter, 2.8452756),
+        'in': lambda match, painter:
+            _distPhys(match, painter, 72.27),
+        'inch': lambda match, painter:
+            _distPhys(match, painter, 72.27),
+        '"': lambda match, painter:
+            _distPhys(match, painter, 72.27),
+        '%': _distPerc,
+        '/': _distFrac,
+        '': _distRatio
+        }
 
-        # point size
-        ( re.compile('^([0-9\.]+) *pt$'),
-          lambda match, painter, t:
-              _distPhys(match, painter, 1.),
-          lambda pixdist, painter, t:
-              _distInvPhys(pixdist, painter, 1., 'pt') ),
-
-        # mm distance
-        ( re.compile('^([0-9\.]+) *mm$'),
-          lambda match, painter, t:
-              _distPhys(match, painter, 2.8452756),
-          lambda pixdist, painter, t:
-              _distInvPhys(pixdist, painter, 2.8452756, 'mm') ),
-
-        # inch distance
-        ( re.compile('^([0-9\.]+) *(inch|in|")$'),
-          lambda match, painter, t:
-              _distPhys(match, painter, 72.27),
-          lambda pixdist, painter, t:
-              _distInvPhys(pixdist, painter, 72.27, 'in') ),
-
-        # plain fraction
-        ( re.compile('^([0-9\.]+)$'),
-          _distRatio,
-          _distInvPerc ),
-
-        # percentage
-        ( re.compile('^([0-9\.]+) *%$'),
-          _distPerc,
-          _distInvPerc ),
-
-        # fractional
-        ( re.compile('^([0-9\.]+) */ *([0-9\.]+)$'),
-          _distFrac,
-          _distInvPerc ),
-        ]
+    # inverse functions for converting pixels to units
+    inv_unit_func = {
+        'cm': lambda match, painter:
+            _distInvPhys(match, painter, 28.452756, 'cm'),
+        'pt': lambda match, painter:
+            _distInvPhys(match, painter, 1., 'pt'),
+        'mm': lambda match, painter:
+            _distInvPhys(match, painter, 2.8452756, 'mm'),
+        'in': lambda match, painter:
+            _distInvPhys(match, painter, 72.27, 'in'),
+        'inch': lambda match, painter:
+            _distInvPhys(match, painter, 72.27, 'in'),
+        '"': lambda match, painter:
+            _distInvPhys(match, painter, 72.27, 'in'),
+        '%': _distInvPerc,
+        '/': _distInvPerc,
+        '': _distInvPerc
+        }
 
     @classmethod
     def isDist(kls, dist):
         """Is the text a valid distance measure?"""
         
-        dist = dist.strip()
-        for reg, fn, fninv in kls.distregexp:
-            if reg.match(dist):
-                return True
-            
-        return False
+        return kls.distre.match(dist) is not None
 
     def convertTo(self, val):
-        if self.isDist(val):
+        if self.distre.match(val) is not None:
             return val
         else:
             raise InvalidType
 
     def toText(self):
-        return self.val
+        # convert decimal point to display locale
+        return self.val.replace('.', qt4.QString(uilocale.decimalPoint()))
 
     def fromText(self, text):
+        # convert decimal point from display locale
+        text = text.replace(qt4.QString(uilocale.decimalPoint()), '.')
+
         if self.isDist(text):
             return text
         else:
@@ -753,7 +763,7 @@ class Distance(Setting):
         return controls.Distance(self, *args)
 
     @classmethod
-    def convertDistance(kls, painter, distance):
+    def convertDistance(kls, painter, dist):
         '''Convert a distance to plotter units.
 
         dist: eg 0.1 (fraction), 10% (percentage), 1/10 (fraction),
@@ -762,26 +772,12 @@ class Distance(Setting):
         painter: painter to get metrics to convert physical sizes
         '''
 
-        # we set a scaling variable in the painter if it's not set
-        if not hasattr(painter, 'veusz_scaling'):
-            painter.veusz_scaling = 1.
-
-        # work out maximum size
-        try:
-            maxsize = max( *painter.veusz_page_size )
-        except AttributeError:
-            w = painter.window()
-            maxsize = max(w.width(), w.height())
-
-        dist = distance.strip()
-
-        # compare string against each regexp
-        for reg, fn, fninv in kls.distregexp:
-            m = reg.match(dist)
-
-            # if there's a match, then call the appropriate conversion fn
-            if m:
-                return fn(m, painter, maxsize)
+        # match distance against expression
+        m = kls.distre.match(dist)
+        if m is not None:
+            # lookup function to call to do conversion
+            func = kls.unit_func[m.group(2)]
+            return func(m, painter)
 
         # none of the regexps match
         raise ValueError( "Cannot convert distance in form '%s'" %
@@ -789,37 +785,26 @@ class Distance(Setting):
 
     def convert(self, painter):
         """Convert this setting's distance as above"""
-        
         return self.convertDistance(painter, self.val)
 
     def convertPts(self, painter):
         """Get the distance in points."""
-        if not hasattr(painter, 'veusz_pixperpt'):
-            _calcPixPerPt(painter)
-
-        return self.convert(painter) / painter.veusz_pixperpt
+        return self.convert(painter) / painter.pixperpt
         
     def convertInverse(self, distpix, painter):
         """Convert distance in pixels into units of this distance.
-
-        Not that great coding as takes "painter" containing veusz
-        scaling parameters. Should be cleaned up.
         """
 
-        # identify units and get inverse mapping
-        v = self.val
-        inversefn = None
-        for reg, fn, fninv in self.distregexp:
-            if reg.match(v):
-                inversefn = fninv
-                break
-        if not inversefn:
-            inversefn = self.distregexp[0][2]
-
-        maxsize = max( *painter.veusz_page_size )
+        m = self.distre.match(self.val)
+        if m is not None:
+            # if it matches convert back
+            inversefn = self.inv_unit_func[m.group(2)]
+        else:
+            # otherwise force unit
+            inversefn = self.inv_unit_func['cm']
 
         # do inverse mapping
-        return inversefn(distpix, painter, maxsize)
+        return inversefn(distpix, painter)
 
 class DistancePt(Distance):
     """For a distance in points."""
@@ -832,7 +817,7 @@ class DistanceOrAuto(Distance):
 
     typename = 'distance-or-auto'
 
-    distregexp = Distance.distregexp + [(re.compile('^Auto$'), None, None)]
+    distre = re.compile( distre_expr + r'|^Auto$', re.VERBOSE )
     
     def isAuto(self):
         return self.val == 'Auto'
@@ -935,7 +920,7 @@ class FloatDict(Setting):
         keys = self.val.keys()
         keys.sort()
         
-        text = ['%s = %g' % (key, self.val[key]) for key in keys]
+        text = ['%s = %s' % (k, uilocale.toString(self.val[k])) for k in keys]
         return '\n'.join(text)
 
     def fromText(self, text):
@@ -954,9 +939,8 @@ class FloatDict(Setting):
             if len(p) != 2:
                 raise InvalidType
 
-            try:
-                v = float(p[1])
-            except ValueError:
+            v, ok = uilocale.toDouble(p[1])
+            if not ok:
                 raise InvalidType
 
             out[ p[0].strip() ] = v
@@ -970,8 +954,6 @@ class FloatList(Setting):
 
     typename = 'float-list'
 
-    list_re = re.compile(r'[\t\n, ]+')
-
     def convertTo(self, val):
         if type(val) not in (list, tuple):
             raise InvalidType
@@ -987,17 +969,28 @@ class FloatList(Setting):
 
     def toText(self):
         """Make a string a, b, c."""
-        return ', '.join( [str(i) for i in self.val] )
+        # can't use the comma for splitting if used as a decimal point
+
+        join = ', '
+        if uilocale.decimalPoint() == qt4.QChar(','):
+            join = '; '
+        return join.join( [unicode(uilocale.toString(x)) for x in self.val] )
 
     def fromText(self, text):
         """Convert from a, b, c or a b c."""
 
+        # don't use commas if it is the decimal separator
+        splitre = r'[\t\n, ]+'
+        if uilocale.decimalPoint() == qt4.QChar(','):
+            splitre = r'[\t\n; ]+'
+
         out = []
-        for x in self.list_re.split(text.strip()):
+        for x in re.split(splitre, text.strip()):
             if x:
-                try:
-                    out.append(float(x))
-                except ValueError:
+                f, ok = uilocale.toDouble(x)
+                if ok:
+                    out.append(f)
+                else:
                     out.append( self.safeEvalHelper(x) )
         return out
 
@@ -1031,7 +1024,7 @@ class WidgetPath(Str):
                                 {'relativetoparent': self.relativetoparent,
                                  'allowedwidgets': self.allowedwidgets})
 
-    def getWidget(self, val = None):
+    def getReferredWidget(self, val = None):
         """Get the widget referred to. We double-check here to make sure
         it's the one.
 
@@ -1194,10 +1187,6 @@ class DatasetOrFloatList(Dataset):
 
     typename = 'dataset-or-floatlist'
 
-    # a list of numbers separated by spaces or tabs
-    # (requires number at end of line)
-    numbers_re = re.compile(r'^([-+]?[0-9]*\.?[0-9]*([eE][-+]?[0-9]+)?[ \t,$]+)+$')
-
     def convertTo(self, val):
         """Check is a string (dataset name) or a list of floats (numbers).
 
@@ -1217,20 +1206,31 @@ class DatasetOrFloatList(Dataset):
         if isinstance(self.val, basestring):
             return self.val
         else:
-            return ', '.join( [str(x) for x in self.val] )
+            # join based on , or ; depending on decimal point
+            join = ', '
+            if uilocale.decimalPoint() == qt4.QChar(','):
+                join = '; '
+            return join.join( [ unicode(uilocale.toString(x))
+                                for x in self.val ] )
 
     def fromText(self, text):
-        text = text.strip()
-        try:
-            out = []
-            for x in FloatList.list_re.split(text):
-                try:
-                    out.append(float(x))
-                except ValueError:
-                    out.append( self.safeEvalHelper(x) )
-            return out
-        except InvalidType:
-            return text
+        """Convert from text."""
+
+        # split based on , or ; depending on decimal point
+        splitre = r'[\t\n, ]+'
+        if uilocale.decimalPoint() == qt4.QChar(','):
+            splitre = r'[\t\n; ]+'
+
+        out = []
+        for x in re.split(splitre, text.strip()):
+            if x:
+                f, ok = uilocale.toDouble(x)
+                if ok:
+                    out.append(f)
+                else:
+                    # fail conversion, so exit with text
+                    return text
+        return out
 
     def getFloatArray(self, doc):
         """Get a numpy of values or None."""
@@ -1250,6 +1250,10 @@ class DatasetOrFloatList(Dataset):
         return (isinstance(self.val, basestring) and
                 doc.data.get(self.val))
 
+    def isEmpty(self):
+        """Is this unset?"""
+        return self.val == []
+
     def getData(self, doc):
         """Return veusz dataset"""
         if isinstance(self.val, basestring):
@@ -1437,16 +1441,25 @@ class Axis(Str):
         """Allows user to choose an axis or enter a name."""
         return controls.Axis(self, self.getDocument(), self.direction, *args)
 
-class Image(Str):
-    """Hold the name of a child image."""
+class WidgetChoice(Str):
+    """Hold the name of a child widget."""
 
-    typename = 'image-widget'
+    typename = 'widget-choice'
 
-    @staticmethod
-    def buildImageList(level, widget, outdict):
-        """A recursive helper to build up a list of possible image widgets.
+    def __init__(self, name, val, widgettypes={}, **args):
+        """Choose widgets from (named) type given."""
+        Setting.__init__(self, name, val, **args)
+        self.widgettypes = widgettypes
 
-        This iterates over widget's children, and adds Image widgets as tuples
+    def copy(self):
+        """Make a copy of the setting."""
+        return self._copyHelper((), (),
+                                {'widgettypes': self.widgettypes})
+
+    def buildWidgetList(self, level, widget, outdict):
+        """A recursive helper to build up a list of possible widgets.
+
+        This iterates over widget's children, and adds widgets as tuples
         to outdict using outdict[name] = (widget, level)
 
         Lower level images of the same name outweigh other images further down
@@ -1454,14 +1467,14 @@ class Image(Str):
         """
 
         for child in widget.children:
-            if child.typename == 'image':
+            if child.typename in self.widgettypes:
                 if (child.name not in outdict) or (outdict[child.name][1]>level):
                     outdict[child.name] = (child, level)
             else:
-                Image.buildImageList(level+1, child, outdict)
+                self.buildWidgetList(level+1, child, outdict)
 
-    def getImageList(self):
-        """Return a dict of valid image names and the corresponding objects."""
+    def getWidgetList(self):
+        """Return a dict of valid widget names and the corresponding objects."""
 
         # find widget which contains setting
         widget = self.parent
@@ -1472,10 +1485,10 @@ class Image(Str):
         if widget is not None:
             widget = widget.parent
 
-        # get list of images from recursive find
+        # get list of widgets from recursive find
         images = {}
         if widget is not None:
-            Image.buildImageList(0, widget, images)
+            self.buildWidgetList(0, widget, images)
 
         # turn (object, level) pairs into object
         outdict = {}
@@ -1484,15 +1497,15 @@ class Image(Str):
 
         return outdict
 
-    def findImage(self):
+    def findWidget(self):
         """Find the image corresponding to this setting.
 
         Returns Image object if succeeds or None if fails
         """
 
-        images = self.getImageList()
+        widgets = self.getWidgetList()
         try:
-            return images[self.get()]
+            return widgets[self.get()]
         except KeyError:
             return None
 
@@ -1502,7 +1515,7 @@ class Image(Str):
 
     def makeControl(self, *args):
         """Allows user to choose an image widget or enter a name."""
-        return controls.Image(self, self.getDocument(), *args)
+        return controls.WidgetChoice(self, self.getDocument(), *args)
     
 class Marker(Choice):
     """Choose a marker type from one allowable."""
@@ -1687,6 +1700,7 @@ class ErrorStyle(Choice):
         'boxfill',
         'fillvert', 'fillhorz',
         'linevert', 'linehorz',
+        'linevertbar', 'linehorzbar'
         )
 
     controls.ErrorStyle._errorstyles  = _errorstyles
@@ -1768,3 +1782,70 @@ class BoolSwitch(Bool):
     def copy(self):
         return self._copyHelper((), (), {'settingsfalse': self.sfalse,
                                          'settingstrue': self.strue})
+
+class RotateInterval(Choice):
+    '''Rotate a label with intervals given.'''
+
+    def __init__(self, name, val, **args):
+        Choice.__init__(self, name,
+                        ('-180', '-135', '-90', '-45',
+                         '0', '45', '90', '135', '180'),
+                        val, **args)
+
+    def convertTo(self, val):
+        """Store rotate angle."""
+        # backward compatibility with rotate option
+        # False: angle 0
+        # True:  angle 90
+        if val == False:
+            val = '0'
+        elif val == True:
+            val = '90'
+        return Choice.convertTo(self, val)
+
+    def copy(self):
+        """Make a copy of the setting."""
+        return self._copyHelper((), (), {})
+
+class Colormap(Str):
+    """A setting to set the color map used in an image.
+    This is based on a Str rather than Choice as the list might
+    change later.
+    """
+
+    def makeControl(self, *args):
+        return controls.Colormap(self, self.getDocument(), *args)
+
+class AxisBound(FloatOrAuto):
+    """Axis bound - either numeric, Auto or date."""
+
+    typename = 'axis-bound'
+
+    def makeControl(self, *args):
+        return controls.AxisBound(self, *args)
+
+    def toText(self):
+        """Convert to text, taking into account mode of Axis.
+        Displays datetimes in date format if used
+        """
+
+        try:
+            mode = self.parent.mode
+        except AttributeError:
+            mode = None
+
+        v = self.val
+        if ( not isinstance(v, basestring) and v is not None and
+             mode == 'datetime' ):
+            return utils.dateFloatToString(v)
+
+        return FloatOrAuto.toText(self)
+
+    def fromText(self, txt):
+        """Convert from text, allowing datetimes."""
+
+        v = utils.dateStringToDate(txt)
+        if N.isfinite(v):
+            return v
+        else:
+            return FloatOrAuto.fromText(self, txt)
