@@ -19,8 +19,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: textrender.py 1403 2010-09-12 13:44:35Z jeremysanders $
-
 import math
 import re
 
@@ -40,11 +38,13 @@ symbols = {
     r'\}': '}',
     r'\[': '[',
     r'\]': ']',
+    r'\backslash' : u'\u005c',
 
     # operators
     r'\pm': u'\u00b1',
     r'\mp': u'\u2213',
     r'\times': u'\u00d7',
+    r'\cdot': u'\u22c5',
     r'\ast': u'\u2217',
     r'\star': u'\u22c6',
     r'\deg': u'\u00b0',
@@ -56,6 +56,7 @@ symbols = {
     r'\uplus': u'\u228e',
     r'\vee': u'\u22c1',
     r'\wedge': u'\u22c0',
+    r'\nabla': u'\u2207',
     r'\lhd': u'\u22b2',
     r'\rhd': u'\u22b3',
     r'\unlhd': u'\u22b4',
@@ -75,9 +76,14 @@ symbols = {
     r'\infty': u'\u221e',
     r'\int': u'\u222b',
     r'\leftarrow': u'\u2190',
+    r'\Leftarrow': u'\u21d0',
     r'\uparrow': u'\u2191',
     r'\rightarrow': u'\u2192',
+    r'\to': u'\u2192',
+    r'\Rightarrow': u'\u21d2',
     r'\downarrow': u'\u2193',
+    r'\leftrightarrow': u'\u2194',
+    r'\Leftrightarrow': u'\u21d4',
     r'\circ': u'\u0e50',
 
     # relations
@@ -190,7 +196,7 @@ class RenderState(object):
         painter = self.painter
         pixperpt = painter.device().logicalDpiY() / 72.
         try:
-            pixperpt *= painter.veusz_scaling
+            pixperpt *= painter.scaling
         except AttributeError:
             pass
         return pixperpt
@@ -209,6 +215,9 @@ class PartText(Part):
     def __init__(self, text):
         self.text = text
     
+    def addText(self, text):
+        self.text += text
+    
     def render(self, state):
         """Render some text."""
 
@@ -387,6 +396,21 @@ class PartSubScript(Part):
         font.setPointSizeF(size)
         state.painter.setFont(font)
 
+class PartMultiScript(Part):
+    """Represents multiple parts with the same starting x, e.g. a combination of
+       super- and subscript parts."""
+    def render(self, state):
+        oldx = state.x
+        newx = oldx
+        for p in self.children:
+            state.x = oldx
+            p.render(state)
+            newx = max([state.x, newx])
+        state.x = newx
+    
+    def append(p):
+        self.children.append(p)
+
 class PartItalic(Part):
     """Represents italic part."""
     def render(self, state):
@@ -548,6 +572,7 @@ part_commands = {
     r'\size': (PartSize, 2),
     r'\frac': (PartFrac, 2),
     r'\bar': (PartBar, 1),
+    r'\overline': (PartBar, 1),
     r'\dot': (PartDot, 1),
     }
 
@@ -555,8 +580,7 @@ part_commands = {
 splitter_re = re.compile(r'''
 (
 \\[A-Za-z]+[ ]* |   # normal latex command
-\\\{ | \\\} |       # escaped {} brackets
-\\\[ | \\\] |       # escaped [] brackets
+\\[\[\]{}_^] |      # escaped special characters
 \\\\ |              # line end
 \{ |                # begin block
 \} |                # end block
@@ -580,14 +604,12 @@ def makePartList(text):
             # we may need to drop excess spaces after \foo commands
             ps = p.rstrip()
             if ps in symbols:
-                # convert to symbol if possible
-                text = symbols[ps]
+                # it will become a symbol, so preserve whitespace
+                doAdd(ps)
                 if ps != p:
-                    # add back spacing
-                    text += p[len(ps)-len(p):]
-                doAdd(text)
+                    doAdd(p[len(ps)-len(p):])
             else:
-                # add as possible command
+                # add as possible command, so drop excess whitespace
                 doAdd(ps)
         elif p == '{':
             # add a new level
@@ -606,6 +628,14 @@ def makePartTree(partlist):
     lines = []
     itemlist = []
     length = len(partlist)
+    
+    def addText(text):
+        """Try to merge consecutive text items for better rendering."""
+        if itemlist and isinstance(itemlist[-1], PartText):
+            itemlist[-1].addText(text)
+        else:
+            itemlist.append( PartText(text) )
+    
     i = 0
     while i < length:
         p = partlist[i]
@@ -613,13 +643,38 @@ def makePartTree(partlist):
             lines.append( Part(itemlist) )
             itemlist = []
         elif isinstance(p, basestring):
-            if p in part_commands:
+            if p in symbols:
+                addText(symbols[p])
+            elif p in part_commands:
                 klass, numargs = part_commands[p]
-                partargs = [makePartTree(k) for k in partlist[i+1:i+numargs+1]]
-                itemlist.append( klass(partargs) )
+                if numargs == 1 and len(partlist) > i+1 and isinstance(partlist[i+1], basestring):
+                    # coerce a single argument to a partlist so that things
+                    # like "A^\dagger" render correctly without needing
+                    # curly brackets
+                    partargs = [makePartTree([partlist[i+1]])]
+                else:
+                    partargs = [makePartTree(k) for k in partlist[i+1:i+numargs+1]]
+                
+                if (p == '^' or p == '_'):
+                    if len(itemlist) > 0 and (
+                        isinstance(itemlist[-1], PartSubScript) or
+                        isinstance(itemlist[-1], PartSuperScript) or
+                        isinstance(itemlist[-1], PartMultiScript)):
+                        # combine sequences of multiple sub-/superscript parts into
+                        # a MultiScript item so that a single text item can have 
+                        # both super and subscript indicies
+                        # e.g. X^{(q)}_{i}
+                        if isinstance(itemlist[-1], PartMultiScript):
+                            itemlist.append( klass(partargs) )
+                        else:
+                            itemlist[-1] = PartMultiScript([itemlist[-1], klass(partargs)])
+                    else:
+                        itemlist.append( klass(partargs) )
+                else:
+                    itemlist.append( klass(partargs) )
                 i += numargs
             else:
-                itemlist.append( PartText(p) )
+                addText(p)
         else:
             itemlist.append( makePartTree(p) )
         i += 1
