@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: axis.py 1412 2010-09-25 08:42:01Z jeremysanders $
-
 """Widget to plot axes, and to handle conversion of coordinates to plot
 positions."""
 
@@ -109,9 +107,10 @@ class AxisLabel(setting.Text):
                                 descr = 'Place axis label close to edge'
                                 ' of graph',
                                 usertext='At edge') )
-        self.add( setting.Bool( 'rotate', False,
-                                descr = 'Rotate the label by 90 degrees',
-                                usertext='Rotate') )
+        self.add( setting.RotateInterval(
+                'rotate', '0',
+                descr = 'Angle by which to rotate label by',
+                usertext='Rotate') )
         self.add( setting.DistancePt( 'offset',
                                       '0pt',
                                       descr = 'Additional offset of axis label'
@@ -133,9 +132,10 @@ class TickLabel(setting.Text):
 
     def __init__(self, name, **args):
         setting.Text.__init__(self, name, **args)
-        self.add( setting.Bool( 'rotate', False,
-                                descr = 'Rotate the label by 90 degrees',
-                                usertext='Rotate') )
+        self.add( setting.RotateInterval(
+                'rotate', '0',
+                descr = 'Angle by which to rotate label by',
+                usertext='Rotate') )
         self.add( setting.ChoiceOrMore( 'format',
                                         TickLabel.formatchoices,
                                         'Auto',
@@ -195,12 +195,12 @@ class Axis(widget.Widget):
         s.add( setting.Str('label', '',
                            descr='Axis label text',
                            usertext='Label') )
-        s.add( setting.FloatOrAuto('min', 'Auto',
-                                   descr='Minimum value of axis',
-                                   usertext='Min') )
-        s.add( setting.FloatOrAuto('max', 'Auto',
-                                   descr='Maximum value of axis',
-                                   usertext='Max') )
+        s.add( setting.AxisBound('min', 'Auto',
+                                 descr='Minimum value of axis',
+                                 usertext='Min') )
+        s.add( setting.AxisBound('max', 'Auto',
+                                 descr='Maximum value of axis',
+                                 usertext='Max') )
         s.add( setting.Bool('log', False,
                             descr = 'Whether axis is logarithmic',
                             usertext='Log') )
@@ -324,7 +324,7 @@ class Axis(widget.Widget):
             # locate widget we're matching
             # this is ensured to be an Axis
             try:
-                widget = s.get('match').getWidget()
+                widget = s.get('match').getReferredWidget()
             except setting.InvalidType:
                 widget = None
 
@@ -359,9 +359,9 @@ class Axis(widget.Widget):
 
         # make sure log axes don't blow up
         if s.log:
-            if self.plottedrange[0] <= 0.:
+            if self.plottedrange[0] < 1e-99:
                 self.plottedrange[0] = 1e-99
-            if self.plottedrange[1] <= 0.:
+            if self.plottedrange[1] < 1e-99:
                 self.plottedrange[1] = 1e-99
             if self.plottedrange[0] == self.plottedrange[1]:
                 self.plottedrange[1] = self.plottedrange[0]*2
@@ -378,9 +378,12 @@ class Axis(widget.Widget):
                         extendzero = s.autoExtendZero,
                         logaxis = s.log )
 
-        (self.plottedrange[0],self.plottedrange[1],
-         self.majortickscalc, self.minortickscalc, 
-         self.autoformat) =  axs.getTicks()
+        axs.getTicks()
+        self.plottedrange[0] = axs.minval
+        self.plottedrange[1] = axs.maxval
+        self.majortickscalc = axs.tickvals
+        self.minortickscalc = axs.minorticks
+        self.autoformat = axs.autoformat
 
         # override values if requested
         if len(s.MajorTicks.manualTicks) > 0:
@@ -426,8 +429,8 @@ class Axis(widget.Widget):
 
             # other axis coordinates
             self.coordPerp  = y2 - dy*otherposition
-            self.coordPerp1 = y2 - dy*p1
-            self.coordPerp2 = y2 - dy*p2
+            self.coordPerp1 = y1
+            self.coordPerp2 = y2
 
         else: # vertical
             self.coordParr1 = y2 - dy*p1
@@ -435,8 +438,8 @@ class Axis(widget.Widget):
 
             # other axis coordinates
             self.coordPerp  = x1 + dx*otherposition
-            self.coordPerp1 = x1 + dx*p1
-            self.coordPerp2 = x1 + dx*p2
+            self.coordPerp1 = x1
+            self.coordPerp2 = x2
 
         # is this axis reflected
         if otherposition > 0.5:
@@ -575,9 +578,21 @@ class Axis(widget.Widget):
             a = (b1, a1, b2, a2)
         utils.plotLinesToPainter(painter, a[0], a[1], a[2], a[3])
 
-    def _drawGridLines(self, subset, painter, coordticks):
+    def _drawGridLines(self, subset, painter, coordticks, parentposn):
         """Draw grid lines on the plot."""
         painter.setPen( self.settings.get(subset).makeQPen(painter) )
+
+        # drop points which overlap with graph box (if used)
+        if self.parent.typename == 'graph':
+            if not self.parent.settings.Border.hide:
+                if self.settings.direction == 'horizontal':
+                    ok = ( (N.abs(coordticks-parentposn[0]) > 1e-3) &
+                           (N.abs(coordticks-parentposn[2]) > 1e-3) )
+                else:
+                    ok = ( (N.abs(coordticks-parentposn[1]) > 1e-3) &
+                           (N.abs(coordticks-parentposn[3]) > 1e-3) )
+                coordticks = coordticks[ok]
+
         self.swaplines(painter,
                        coordticks, coordticks*0.+self.coordPerp1,
                        coordticks, coordticks*0.+self.coordPerp2)
@@ -610,9 +625,9 @@ class Axis(widget.Widget):
             delta *= -1
         
         y = coordminorticks*0.+self.coordPerp
-        self.swaplines(painter,
-                       coordminorticks, y,
-                       coordminorticks, y-delta)
+        self.swaplines( painter,
+                        coordminorticks, y,
+                        coordminorticks, y-delta )
 
     def _drawMajorTicks(self, painter, tickcoords):
         """Draw major ticks on the plot."""
@@ -633,19 +648,19 @@ class Axis(widget.Widget):
             delta *= -1
 
         y = tickcoords*0.+self.coordPerp
-        self.swaplines(painter,
-                       tickcoords, y,
-                       tickcoords, y-delta)
+        self.swaplines( painter,
+                        tickcoords, y,
+                        tickcoords, y-delta )
 
         # account for ticks if they are in the direction of the label
         if s.outerticks and not self.coordReflected:
             self._delta_axis += abs(delta)
 
-    def generateLabelLabels(self, painter):
+    def generateLabelLabels(self, phelper):
         """Generate list of positions and labels from widgets using this
         axis."""
         try:
-            plotters = painter.veusz_axis_plotter_map[self]
+            plotters = phelper.axisplottermap[self]
         except (AttributeError, KeyError):
             return
 
@@ -663,7 +678,7 @@ class Axis(widget.Widget):
                     if N.isfinite(coord) and (minval <= coord <= maxval):
                         yield pcoord, lab
 
-    def _drawTickLabels(self, painter, coordticks, sign, outerbounds,
+    def _drawTickLabels(self, phelper, painter, coordticks, sign, outerbounds,
                         texttorender):
         """Draw tick labels on the plot.
 
@@ -679,13 +694,9 @@ class Axis(widget.Widget):
         tl_spacing = fm.leading() + fm.descent()
 
         # work out font alignment
-        if s.TickLabels.rotate:
-            if self.coordReflected:
-                angle = 90
-            else:
-                angle = 270
-        else:
-            angle = 0
+        angle = int(s.TickLabels.rotate)
+        if not self.coordReflected and angle != 0:
+            angle = 360-angle
         
         if vertical:
             # limit tick labels to be directly below/besides axis
@@ -717,7 +728,8 @@ class Axis(widget.Widget):
 
             # generate positions and labels
             for posn, tickval in izip(coordticks, self.majortickscalc):
-                text = utils.formatNumber(tickval*scale, format)
+                text = utils.formatNumber(tickval*scale, format,
+                                          locale=self.document.locale)
                 yield posn, text
 
         # position of label perpendicular to axis
@@ -725,7 +737,7 @@ class Axis(widget.Widget):
 
         # use generator function to get labels and positions
         if s.mode == 'labels':
-            ticklabels = self.generateLabelLabels(painter)
+            ticklabels = self.generateLabelLabels(phelper)
         else:
             ticklabels = generateTickLabels()
 
@@ -795,15 +807,16 @@ class Axis(widget.Widget):
         if reflected:
             ax, ay = -ax, -ay
 
-        # angle of text
-        if ( (horz and not sl.rotate) or
-             (not horz and sl.rotate) ):
-            angle = 0
+        # angle of text (logic is slightly complex)
+        angle = int(sl.rotate)
+        if horz:
+            if not reflected:
+                angle = 360-angle
         else:
+            angle = angle+270
             if reflected:
-                angle = 90
-            else:
-                angle = 270
+                angle = 360-angle
+        angle = angle % 360
 
         x = 0.5*(self.coordParr1 + self.coordParr2)
         y = self.coordPerp + sign*(self._delta_axis+al_spacing)
@@ -912,10 +925,13 @@ class Axis(widget.Widget):
                 return True
         return False
 
-    def draw(self, parentposn, painter, outerbounds=None):
+    def draw(self, parentposn, phelper, outerbounds=None,
+             useexistingpainter=None):
         """Plot the axis on the painter.
 
-        if suppresstext is True, then we don't number or label the axis
+        useexistingpainter is a hack so that a colorbar can reuse the
+        drawing code here. If set to a painter, it will use this rather
+        than opening a new one.
         """
 
         s = self.settings
@@ -924,17 +940,19 @@ class Axis(widget.Widget):
         if self.docchangeset != self.document.changeset:
             self._computePlottedRange()
 
-        posn = widget.Widget.draw(self, parentposn, painter, outerbounds)
+        posn = widget.Widget.draw(self, parentposn, phelper, outerbounds)
         self._updatePlotRange(posn)
 
+        # get ready to draw
+        if useexistingpainter is not None:
+            painter = useexistingpainter
+        else:
+            painter = phelper.painter(self, posn)
+
         # make control item for axis
-        self.controlgraphitems = [
-            controlgraph.ControlAxisLine(self, s.direction,
-                                         self.coordParr1,
-                                         self.coordParr2,
-                                         self.coordPerp,
-                                         posn)
-            ]
+        phelper.setControlGraph(self, [ controlgraph.ControlAxisLine(
+                    self, s.direction, self.coordParr1,
+                    self.coordParr2, self.coordPerp, posn) ])
 
         # get tick vals
         coordticks = self._graphToPlotter(self.majortickscalc)
@@ -944,10 +962,6 @@ class Axis(widget.Widget):
         if s.hide:
             return
 
-        # save the state of the painter for later
-        painter.beginPaintingWidget(self, posn)
-        painter.save()
-
         texttorender = []
 
         # multiplication factor if reflection on the axis is requested
@@ -959,9 +973,11 @@ class Axis(widget.Widget):
 
         # plot gridlines
         if not s.MinorGridLines.hide:
-            self._drawGridLines('MinorGridLines', painter, coordminorticks)
+            self._drawGridLines('MinorGridLines', painter, coordminorticks,
+                                parentposn)
         if not s.GridLines.hide:
-            self._drawGridLines('GridLines', painter, coordticks)
+            self._drawGridLines('GridLines', painter, coordticks,
+                                parentposn)
 
         # plot the line along the axis
         if not s.Line.hide:
@@ -978,19 +994,11 @@ class Axis(widget.Widget):
         if not s.MajorTicks.hide:
             self._drawMajorTicks(painter, coordticks)
 
-        # debugging
-        #painter.save()
-        #painter.setPen(qt4.QPen(qt4.Qt.blue))
-        #painter.drawRect(
-        #    qt4.QRectF(qt4.QPointF(outerbounds[0], outerbounds[1]),
-        #               qt4.QPointF(outerbounds[2], outerbounds[3])) )
-        #painter.restore()
-
         # plot tick labels
         suppresstext = self._suppressText(painter, parentposn, outerbounds)
         if not s.TickLabels.hide and not suppresstext:
-            self._drawTickLabels(painter, coordticks, sign, outerbounds,
-                                 texttorender)
+            self._drawTickLabels(phelper, painter, coordticks, sign,
+                                 outerbounds, texttorender)
 
         # draw an axis label
         if not s.Label.hide and not suppresstext:
@@ -1012,36 +1020,48 @@ class Axis(widget.Widget):
                 painter.setPen(pen)
                 box = r.render()
                 drawntext.addRect(rect)
-
-        # restore the state of the painter
-        painter.restore()
-
-        painter.endPaintingWidget()
-
+                
     def updateControlItem(self, cgi):
         """Update axis position from control item."""
 
         s = self.settings
         p = cgi.maxposn
-        if s.direction == 'horizontal':
-            minfrac = abs((cgi.minpos - p[0]) / (p[2] - p[0]))
-            maxfrac = abs((cgi.maxpos - p[0]) / (p[2] - p[0]))
-            axisfrac = abs((cgi.axispos - p[3]) / (p[1] - p[3]))
-        else:
-            minfrac = abs((cgi.minpos - p[3]) / (p[1] - p[3]))
-            maxfrac = abs((cgi.maxpos - p[3]) / (p[1] - p[3]))
-            axisfrac = abs((cgi.axispos - p[0]) / (p[2] - p[0]))
 
-        if minfrac > maxfrac:
-            minfrac, maxfrac = maxfrac, minfrac
+        if cgi.zoomed():
+            # zoom axis scale
+            c1, c2 = self.plotterToGraphCoords(
+                cgi.maxposn, N.array([cgi.minzoom, cgi.maxzoom]))
+            if c1 > c2:
+                c1, c2 = c2, c1
+            operations = (
+                document.OperationSettingSet(s.get('min'), float(c1)),
+                document.OperationSettingSet(s.get('max'), float(c2)),
+                document.OperationSettingSet(s.get('autoExtend'), False),
+                document.OperationSettingSet(s.get('autoExtendZero'), False),
+                )
+            self.document.applyOperation(
+                document.OperationMultiple(operations, descr='zoom axis'))
+        elif cgi.moved():
+            # move axis
+            # convert positions to fractions
+            pt1, pt2, ppt1, ppt2 = ( (3, 1, 0, 2), (0, 2, 3, 1)
+                                     ) [s.direction == 'horizontal']
+            minfrac = abs((cgi.minpos - p[pt1]) / (p[pt2] - p[pt1]))
+            maxfrac = abs((cgi.maxpos - p[pt1]) / (p[pt2] - p[pt1]))
+            axisfrac = abs((cgi.axispos - p[ppt1]) / (p[ppt2] - p[ppt1]))
 
-        operations = (
-            document.OperationSettingSet(s.get('lowerPosition'), minfrac),
-            document.OperationSettingSet(s.get('upperPosition'), maxfrac),
-            document.OperationSettingSet(s.get('otherPosition'), axisfrac),
-            )
-        self.document.applyOperation(
-            document.OperationMultiple(operations, descr='adjust axis'))
+            # swap if wrong way around
+            if minfrac > maxfrac:
+                minfrac, maxfrac = maxfrac, minfrac
+
+            # update doc
+            operations = (
+                document.OperationSettingSet(s.get('lowerPosition'), minfrac),
+                document.OperationSettingSet(s.get('upperPosition'), maxfrac),
+                document.OperationSettingSet(s.get('otherPosition'), axisfrac),
+                )
+            self.document.applyOperation(
+                document.OperationMultiple(operations, descr='adjust axis'))
 
 # allow the factory to instantiate an axis
 document.thefactory.register( Axis )
