@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: bar.py 1449 2010-11-22 09:26:58Z jeremysanders $
-
 """For plotting bar graphs."""
 
 from itertools import izip, repeat
@@ -156,11 +154,11 @@ class BarPlotter(GenericPlotter):
             positions = s.get('posn').getData(doc)
             if positions is None:
                 lengths = s.get('lengths').getData(doc)
-                if lengths is None:
+                if not lengths:
                     return (None, None)
                 p = N.arange( max([len(d.data) for d in lengths]) )+1.
             else:
-                p = positions
+                p = positions.data
             
             return (labels, p)
 
@@ -427,10 +425,10 @@ class BarPlotter(GenericPlotter):
                                      qt4.QPointF(x+width, y+height*0.8)) )
 
 
-    def draw(self, parentposn, painter, outerbounds=None):
+    def draw(self, parentposn, phelper, outerbounds=None):
         """Plot the data on a plotter."""
 
-        widgetposn = GenericPlotter.draw(self, parentposn, painter,
+        widgetposn = GenericPlotter.draw(self, parentposn, phelper,
                                          outerbounds=outerbounds)
         s = self.settings
 
@@ -477,17 +475,13 @@ class BarPlotter(GenericPlotter):
             dsvals.append(vals)
 
         # clip data within bounds of plotter
-        painter.beginPaintingWidget(self, widgetposn)
-        painter.save()
-        clip = self.clipAxesBounds(painter, axes, widgetposn)
+        clip = self.clipAxesBounds(axes, widgetposn)
+        painter = phelper.painter(self, widgetposn, clip=clip)
 
         # actually do the drawing
         fn = {'stacked': self.barDrawStacked,
               'grouped': self.barDrawGroup}[s.mode]
         fn(painter, barposns, maxwidth, dsvals, axes, widgetposn, clip)
 
-        painter.restore()
-        painter.endPaintingWidget()
-
 # allow the factory to instantiate a bar plotter
 document.thefactory.register( BarPlotter )
