@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: boxplot.py 1470 2010-12-09 09:39:12Z jeremysanders $
-
 """For making box plots."""
 
 import math
@@ -41,7 +39,8 @@ def percentile(sortedds, perc):
     # interpolate between indices
     frac, index = math.modf(index)
     index = int(index)
-    interpol = (1-frac)*sortedds[index] + frac*sortedds[index+1]
+    indexplus1 = min(index+1, sortedds.shape[0]-1)
+    interpol = (1-frac)*sortedds[index] + frac*sortedds[indexplus1]
     return interpol
 
 def swapline(painter, x1, y1, x2, y2, swap):
@@ -374,18 +373,18 @@ class BoxPlot(GenericPlotter):
                                markersize, clip=clip )
 
         # draw mean
-        meanplt = axes[not horz].dataToPlotterCoords(posn,
-                                                     N.array([stats.mean]))[0]
+        meanplt = axes[not horz].dataToPlotterCoords(
+            posn, N.array([stats.mean]))[0]
         if horz:
             x, y = meanplt, boxposn
         else:
             x, y = boxposn, meanplt
         utils.plotMarker( painter, x, y, s.meanmarker, markersize )
 
-    def draw(self, parentposn, painter, outerbounds=None):
+    def draw(self, parentposn, phelper, outerbounds=None):
         """Plot the data on a plotter."""
 
-        widgetposn = GenericPlotter.draw(self, parentposn, painter,
+        widgetposn = GenericPlotter.draw(self, parentposn, phelper,
                                          outerbounds=outerbounds)
         s = self.settings
 
@@ -418,10 +417,8 @@ class BoxPlot(GenericPlotter):
              axes[1].settings.direction != 'vertical' ):
             return
 
-        # clip data within bounds of plotter
-        painter.beginPaintingWidget(self, widgetposn)
-        painter.save()
-        clip = self.clipAxesBounds(painter, axes, widgetposn)
+        clip = self.clipAxesBounds(axes, widgetposn)
+        painter = phelper.painter(self, widgetposn, clip=clip)
 
         # get boxes visible along direction of boxes to work out width
         horz = (s.direction == 'horizontal')
@@ -468,8 +465,5 @@ class BoxPlot(GenericPlotter):
                 self.plotBox(painter, axes, vals[6][i], widgetposn,
                              width, clip, stats)
 
-        painter.restore()
-        painter.endPaintingWidget()
-
 # allow the factory to instantiate a boxplot
 document.thefactory.register( BoxPlot )
