@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: contour.py 1401 2010-09-11 09:28:00Z jeremysanders $
-
 """Contour plotting from 2d datasets.
 
 Contour plotting requires that the veusz_helpers package is installed,
@@ -238,15 +236,15 @@ class Contour(plotters.GenericPlotter):
         s = self.settings
         d = self.document
 
-        if s.data not in d.data:
-            # this dataset doesn't exist
-            minval = 0.
-            maxval = 1.
-        else:
+        minval, maxval = 0., 1.
+        if s.data in d.data:
             # scan data
-            data = d.data[s.data]
-            minval = data.data.min()
-            maxval = data.data.max()
+            data = d.data[s.data].data
+            minval, maxval = N.nanmin(data), N.nanmax(data)
+            if not N.isfinite(minval):
+                minval = 0.
+            if not N.isfinite(maxval):
+                maxval = 1.
 
         # override if not auto
         if s.min != 'Auto':
@@ -293,7 +291,7 @@ class Contour(plotters.GenericPlotter):
         """Calculate sublevels between contours."""
         s = self.settings
         num = s.SubLines.numLevels
-        if s.SubLines.hide or len(s.SubLines.lines) == 0:
+        if s.SubLines.hide or len(s.SubLines.lines) == 0 or len(levels) <= 1:
             return N.array([])
 
         # indices where contour levels should be placed
@@ -364,7 +362,8 @@ class Contour(plotters.GenericPlotter):
         if s.keyLevels:
             cl = s.get('ContourLabels')
             return utils.formatNumber( s.levelsOut[number] * cl.scale,
-                                       cl.format )
+                                       cl.format,
+                                       locale=self.document.locale )
         else:
             return ''
 
@@ -384,7 +383,7 @@ class Contour(plotters.GenericPlotter):
 
         # return if no data or if the dataset isn't two dimensional
         data = d.data.get(s.data, None)
-        if data is None or data.dimensions != 2:
+        if data is None or data.dimensions != 2 or data.data.size == 0:
             self.contsettings = self.lastdataset = None
             s.levelsOut = []
             return False
@@ -402,10 +401,10 @@ class Contour(plotters.GenericPlotter):
 
         return True
 
-    def draw(self, parentposn, painter, outerbounds = None):
+    def draw(self, parentposn, phelper, outerbounds = None):
         """Draw the contours."""
 
-        posn = plotters.GenericPlotter.draw(self, parentposn, painter,
+        posn = plotters.GenericPlotter.draw(self, parentposn, phelper,
                                             outerbounds = outerbounds)
         s = self.settings
 
@@ -427,17 +426,13 @@ class Contour(plotters.GenericPlotter):
             return
 
         # plot the precalculated contours
-        painter.beginPaintingWidget(self, posn)
-        painter.save()
-        clip = self.clipAxesBounds(painter, axes, posn)
+        clip = self.clipAxesBounds(axes, posn)
+        painter = phelper.painter(self, posn, clip=clip)
 
         self.plotContourFills(painter, posn, axes, clip)
         self.plotContours(painter, posn, axes, clip)
         self.plotSubContours(painter, posn, axes, clip)
 
-        painter.restore()
-        painter.endPaintingWidget()
-
     def updateContours(self):
         """Update calculated contours."""
 
@@ -452,6 +447,9 @@ class Contour(plotters.GenericPlotter):
         rangex, rangey = data.getDataRanges()
         yw, xw = data.data.shape
 
+        if xw == 0 or yw == 0:
+            return
+
         # arrays containing coordinates of pixels in x and y
         xpts = N.fromfunction(lambda y,x:
                               (x+0.5)*((rangex[1]-rangex[0])/xw) + rangex[0],
@@ -460,13 +458,16 @@ class Contour(plotters.GenericPlotter):
                               (y+0.5)*((rangey[1]-rangey[0])/yw) + rangey[0],
                               (yw, xw))
 
+        # only keep finite data points
+        mask = N.logical_not(N.isfinite(data.data))
+
         # iterate over the levels and trace the contours
         self._cachedcontours = None
         self._cachedpolygons = None
         self._cachedsubcontours = None
 
         if self.Cntr is not None:
-            c = self.Cntr(xpts, ypts, data.data)
+            c = self.Cntr(xpts, ypts, data.data, mask)
 
             # trace the contour levels
             if len(s.Lines.lines) != 0:
@@ -490,13 +491,17 @@ class Contour(plotters.GenericPlotter):
                     self._cachedsubcontours.append( finitePoly(linelist) )
 
     def plotContourLabel(self, painter, number, xplt, yplt, showline):
+        """Draw a label on a contour.
+        This clips when drawing the line, plotting the label on top.
+        """
         s = self.settings
         cl = s.get('ContourLabels')
 
         painter.save()
 
         # get text and font
-        text = utils.formatNumber(number * cl.scale, cl.format)
+        text = utils.formatNumber(number * cl.scale, cl.format,
+                                  locale=self.document.locale)
         font = cl.makeQFont(painter)
         descent = utils.FontMetrics(font, painter.device()).descent()
 
@@ -535,8 +540,8 @@ class Contour(plotters.GenericPlotter):
 
         painter.restore()
 
-    def _plotContours(self, painter, posn, axes, linestyles, contours,
-                      showlabels, hidelines, clip):
+    def _plotContours(self, painter, posn, axes, linestyles,
+                      contours, showlabels, hidelines, clip):
         """Plot a set of contours.
         """
 
@@ -562,8 +567,8 @@ class Contour(plotters.GenericPlotter):
                 utils.addNumpyToPolygonF(pts, xplt, yplt)
 
                 if showlabels:
-                    self.plotContourLabel(painter, s.levelsOut[num], xplt, yplt,
-                                          not hidelines)
+                    self.plotContourLabel(painter, s.levelsOut[num],
+                                          xplt, yplt, not hidelines)
                 else:
                     # actually draw the curve to the plotter
                     if not hidelines:
