@@ -16,8 +16,6 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
-# $Id: point.py 1449 2010-11-22 09:26:58Z jeremysanders $
-
 """For plotting xy points."""
 
 import veusz.qtall as qt4
@@ -28,6 +26,7 @@ import veusz.document as document
 import veusz.setting as setting
 import veusz.utils as utils
 
+import pickable
 from plotters import GenericPlotter
 
 try:
@@ -143,7 +142,7 @@ def _errorBarsFilled(style, xmin, xmax, ymin, ymax, xplotter, yplotter,
 
     hidevert = True  # keep track of what's shown
     hidehorz = True
-    if ( (style == 'fillvert' or style == 'linevert') and
+    if ( 'vert' in style and
          (ymin is not None and ymax is not None) and
          not s.ErrorBarLine.hideVert ):
         hidevert = False
@@ -151,7 +150,7 @@ def _errorBarsFilled(style, xmin, xmax, ymin, ymax, xplotter, yplotter,
         utils.addNumpyToPolygonF(ptsbelow, xplotter, ymin)
         utils.addNumpyToPolygonF(ptsabove, xplotter, ymax)
 
-    elif ( (style == 'fillhorz' or style == 'linehorz') and
+    elif ( 'horz' in style and
            (xmin is not None and xmax is not None) and
            not s.ErrorBarLine.hideHorz ):
         hidehorz = False
@@ -160,8 +159,7 @@ def _errorBarsFilled(style, xmin, xmax, ymin, ymax, xplotter, yplotter,
         utils.addNumpyToPolygonF(ptsabove, xmax, yplotter)
 
     # draw filled regions above/left and below/right
-    if ( (style == 'fillvert' or style == 'fillhorz') and
-         not (hidehorz and hidevert) ):
+    if 'fill' in style and not (hidehorz and hidevert):
         # construct points for error bar regions
         retnpts = qt4.QPolygonF()
         utils.addNumpyToPolygonF(retnpts, xplotter[::-1], yplotter[::-1])
@@ -198,16 +196,53 @@ _errorBarFunctionMap = {
     'fillvert': (_errorBarsFilled,),
     'linehorz': (_errorBarsFilled,),
     'linevert': (_errorBarsFilled,),
+    'linehorzbar': (_errorBarsBar, _errorBarsFilled),
+    'linevertbar': (_errorBarsBar, _errorBarsFilled),
     }
 
-
 class MarkerFillBrush(setting.Brush):
     def __init__(self, name, **args):
         setting.Brush.__init__(self, name, **args)
 
         self.get('color').newDefault( setting.Reference(
             '../PlotLine/color') )
-    
+
+        self.add( setting.Colormap(
+                'colorMap', 'grey',
+                descr = 'If color markers dataset is given, use this colormap '
+                'instead of the fill color',
+                usertext='Color map',
+                formatting=True) )
+        self.add( setting.Bool(
+                'colorMapInvert', False,
+                descr = 'Invert color map',
+                usertext = 'Invert map',
+                formatting=True) )
+
+class ColorSettings(setting.Settings):
+    """Settings for a coloring points using data values."""
+
+    def __init__(self, name):
+        setting.Settings.__init__(self, name, setnsmode='groupedsetting')
+        self.add( setting.DatasetOrFloatList(
+                'points', '',
+                descr = 'Use color value (0-1) in dataset to paint points',
+                usertext='Color markers'), 7 )
+        self.add( setting.Float(
+                'min', 0.,
+                descr = 'Minimum value of color dataset',
+                usertext = 'Min val' ))
+        self.add( setting.Float(
+                'max', 1.,
+                descr = 'Maximum value of color dataset',
+                usertext = 'Max val' ))
+        self.add( setting.Choice(
+                'scaling',
+                ['linear', 'sqrt', 'log', 'squared'],
+                'linear',
+                descr = 'Scaling to transform numbers to color',
+                usertext='Scaling'))
+
 class PointPlotter(GenericPlotter):
     """A class for plotting points and their errors."""
 
@@ -251,6 +286,8 @@ class PointPlotter(GenericPlotter):
                 descr = 'Scale size of plotted markers by this dataset or'
                 ' list of values', usertext='Scale markers'), 6 )
 
+        s.add( ColorSettings('Color') )
+
         s.add( setting.DatasetOrFloatList(
                 'yData', 'y',
                 descr='Dataset containing y data or list of values',
@@ -349,13 +386,26 @@ class PointPlotter(GenericPlotter):
         return ( (s.xAxis, 'sx'), (s.yAxis, 'sy') )
 
     def updateAxisRange(self, axis, depname, axrange):
+        """Compute the effect of data on the axis range."""
         dataname = {'sx': 'xData', 'sy': 'yData'}[depname]
-        data = self.settings.get(dataname).getData(self.document)
+        dsetn = self.settings.get(dataname)
+        data = dsetn.getData(self.document)
+
         if data:
             drange = data.getRange()
             if drange:
                 axrange[0] = min(axrange[0], drange[0])
                 axrange[1] = max(axrange[1], drange[1])
+        elif dsetn.isEmpty():
+            # no valid dataset.
+            # check if there a valid dataset for the other axis.
+            # if there is, treat this as a row number
+            dataname = {'sy': 'xData', 'sx': 'yData'}[depname]
+            data = self.settings.get(dataname).getData(self.document)
+            if data:
+                length = data.data.shape[0]
+                axrange[0] = min(axrange[0], 1)
+                axrange[1] = max(axrange[1], length)
 
     def _getLinePoints( self, xvals, yvals, posn, xdata, ydata ):
         """Get the points corresponding to the line connecting the points."""
@@ -436,7 +486,8 @@ class PointPlotter(GenericPlotter):
         return path
         painter.strokePath(p, painter.pen())
 
-    def _drawBezierLine( self, painter, xvals, yvals, posn, xdata, ydata):
+    def _drawBezierLine( self, painter, xvals, yvals, posn,
+                         xdata, ydata):
         """Handle bezier lines and fills."""
 
         pts = self._getLinePoints(xvals, yvals, posn, xdata, ydata)
@@ -533,8 +584,8 @@ class PointPlotter(GenericPlotter):
         # plot error bar
         painter.setPen( s.ErrorBarLine.makeQPenWHide(painter) )
         for function in _errorBarFunctionMap[style]:
-            function(style, xneg, xpos, yneg, ypos, xpts, ypts, s, painter,
-                     cliprect)
+            function(style, xneg, xpos, yneg, ypos, xpts, ypts, s,
+                     painter, cliprect)
 
         # draw line
         if not s.PlotLine.hide:
@@ -555,7 +606,8 @@ class PointPlotter(GenericPlotter):
 
         painter.restore()
 
-    def drawLabels(self, painter, xplotter, yplotter, textvals, markersize):
+    def drawLabels(self, painter, xplotter, yplotter,
+                   textvals, markersize):
         """Draw labels for the points."""
 
         s = self.settings
@@ -594,10 +646,53 @@ class PointPlotter(GenericPlotter):
         else:
             return (text, yv.data)
 
-    def draw(self, parentposn, painter, outerbounds=None):
+    def _fetchAxes(self):
+        """Returns the axes for this widget"""
+
+        s = self.settings
+        axes = self.parent.getAxes( (s.xAxis, s.yAxis) )
+
+        # fail if we don't have good axes
+        if ( None in axes or
+             axes[0].settings.direction != 'horizontal' or
+             axes[1].settings.direction != 'vertical' ):
+            return None
+
+        return axes
+
+    def _pickable(self, bounds):
+        axes = self._fetchAxes()
+
+        if axes is None:
+            map_fn = None
+        else:
+            map_fn = lambda x, y: ( axes[0].dataToPlotterCoords(bounds, x),
+                                    axes[1].dataToPlotterCoords(bounds, y) )
+
+        return pickable.DiscretePickable(self, 'xData', 'yData', map_fn)
+
+    def pickPoint(self, x0, y0, bounds, distance = 'radial'):
+        return self._pickable(bounds).pickPoint(x0, y0, bounds, distance)
+
+    def pickIndex(self, oldindex, direction, bounds):
+        return self._pickable(bounds).pickIndex(oldindex, direction, bounds)
+
+    def makeColorbarImage(self, direction='horz'):
+        """Make a QImage colorbar for the current plot."""
+
+        s = self.settings
+        c = s.Color
+        cmap = self.document.getColormap(
+            s.MarkerFill.colorMap, s.MarkerFill.colorMapInvert)
+
+        return utils.makeColorbarImage(
+            c.min, c.max, c.scaling, cmap, 0,
+            direction=direction)
+
+    def draw(self, parentposn, phelper, outerbounds=None):
         """Plot the data on a plotter."""
 
-        posn = GenericPlotter.draw(self, parentposn, painter,
+        posn = GenericPlotter.draw(self, parentposn, phelper,
                                    outerbounds=outerbounds)
         x1, y1, x2, y2 = posn
 
@@ -613,8 +708,20 @@ class PointPlotter(GenericPlotter):
         yv = s.get('yData').getData(doc)
         text = s.get('labels').getData(doc, checknull=True)
         scalepoints = s.get('scalePoints').getData(doc)
+        colorpoints = s.Color.get('points').getData(doc)
 
+        # if a missing dataset, make a fake dataset for the second one
+        # based on a row number
+        if xv and not yv and s.get('yData').isEmpty():
+            # use index for y data
+            length = xv.data.shape[0]
+            yv = document.DatasetRange(length, (1,length))
+        elif yv and not xv and s.get('xData').isEmpty():
+            # use index for x data
+            length = yv.data.shape[0]
+            xv = document.DatasetRange(length, (1,length))
         if not xv or not yv:
+            # no valid dataset, so exit
             return
 
         # if text entered, then multiply up to get same number of values
@@ -624,22 +731,19 @@ class PointPlotter(GenericPlotter):
             text = text*(length / len(text)) + text[:length % len(text)]
 
         # get axes widgets
-        axes = self.parent.getAxes( (s.xAxis, s.yAxis) )
-
-        # return if there's no proper axes
-        if ( None in axes or
-             axes[0].settings.direction != 'horizontal' or
-             axes[1].settings.direction != 'vertical' ):
+        axes = self._fetchAxes()
+        if not axes:
+            # no valid axes, so exit
             return
 
         # clip data within bounds of plotter
-        painter.beginPaintingWidget(self, posn)
-        painter.save()
-        cliprect = self.clipAxesBounds(painter, axes, posn)
+        cliprect = self.clipAxesBounds(axes, posn)
+        painter = phelper.painter(self, posn, clip=cliprect)
 
         # loop over chopped up values
-        for xvals, yvals, tvals, ptvals in document.generateValidDatasetParts(
-            xv, yv, text, scalepoints):
+        for xvals, yvals, tvals, ptvals, cvals in (
+            document.generateValidDatasetParts(
+                xv, yv, text, scalepoints, colorpoints)):
 
             #print "Calculating coordinates"
             # calc plotter coords of x and y points
@@ -689,20 +793,26 @@ class PointPlotter(GenericPlotter):
                                   yplotter[::s.thinfactor])
 
                 # whether to scale markers
-                scaling = None
+                scaling = colorvals = cmap = None
                 if ptvals:
                     scaling = ptvals.data
+                # color point individually
+                if cvals:
+                    colorvals = utils.applyScaling(
+                        cvals.data, s.Color.scaling,
+                        s.Color.min, s.Color.max)
+                    cmap = self.document.getColormap(
+                        s.MarkerFill.colorMap, s.MarkerFill.colorMapInvert)
 
                 # actually plot datapoints
                 utils.plotMarkers(painter, xplt, yplt, s.marker, markersize,
-                                  scaling=scaling, clip=cliprect)
+                                  scaling=scaling, clip=cliprect,
+                                  cmap=cmap, colorvals=colorvals)
 
             # finally plot any labels
             if tvals and not s.Label.hide:
-                self.drawLabels(painter, xplotter, yplotter, tvals, markersize)
-
-        painter.restore()
-        painter.endPaintingWidget()
+                self.drawLabels(painter, xplotter, yplotter,
+                                tvals, markersize)
 
 # allow the factory to instantiate an x,y plotter
 document.thefactory.register( PointPlotter )
