@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #    Copyright (C) 2003 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -16,13 +17,12 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
-# $Id: mainwindow.py 1455 2010-11-26 22:35:03Z jeremysanders $
-
 """Implements the main window of the application."""
 
 import os.path
 import sys
 import traceback
+import glob
 
 import veusz.qtall as qt4
 
@@ -34,6 +34,7 @@ import veusz.plugins as plugins
 import consolewindow
 import plotwindow
 import treeeditwindow
+from datanavigator import DataNavigatorWindow
 
 from veusz.dialogs.aboutdialog import AboutDialog
 from veusz.dialogs.reloaddata import ReloadData
@@ -83,6 +84,13 @@ class MainWindow(qt4.QMainWindow):
         win.treeedit.doInitialWidgetSelect()
             
         cls.windows.append(win)
+
+        # check if tutorial wanted
+        if not setting.settingdb['ask_tutorial']:
+            win.askTutorial()
+        # don't ask again
+        setting.settingdb['ask_tutorial'] = True
+
         return win
 
     def __init__(self, *args):
@@ -121,6 +129,8 @@ class MainWindow(qt4.QMainWindow):
         self.formatdock = treeeditwindow.FormatDock(self.document,
                                                     self.treeedit, self)
         self.addDockWidget(qt4.Qt.LeftDockWidgetArea, self.formatdock)
+        self.datadock = DataNavigatorWindow(self.document, self, self)
+        self.addDockWidget(qt4.Qt.RightDockWidgetArea, self.datadock)
 
         # make the console window a dock
         self.console = consolewindow.ConsoleWindow(self.document,
@@ -129,15 +139,36 @@ class MainWindow(qt4.QMainWindow):
         self.interpreter = self.console.interpreter
         self.addDockWidget(qt4.Qt.BottomDockWidgetArea, self.console)
 
-        # keep page number up to date
+        # assemble the statusbar
         statusbar = self.statusbar = qt4.QStatusBar(self)
         self.setStatusBar(statusbar)
         self.updateStatusbar('Ready')
-        self.pagelabel = qt4.QLabel(statusbar)
-        statusbar.addWidget(self.pagelabel)
+
+        # a label for the picker readout
+        self.pickerlabel = qt4.QLabel(statusbar)
+        self._setPickerFont(self.pickerlabel)
+        statusbar.addPermanentWidget(self.pickerlabel)
+        self.pickerlabel.hide()
+
+        # plot queue - how many plots are currently being drawn
+        self.plotqueuecount = 0
+        self.connect( self.plot, qt4.SIGNAL("queuechange"),
+                      self.plotQueueChanged )
+        self.plotqueuelabel = qt4.QLabel()
+        self.plotqueuelabel.setToolTip("Number of rendering jobs remaining")
+        statusbar.addWidget(self.plotqueuelabel)
+        self.plotqueuelabel.show()
+
+        # a label for the cursor position readout
         self.axisvalueslabel = qt4.QLabel(statusbar)
-        statusbar.addWidget(self.axisvalueslabel)
+        statusbar.addPermanentWidget(self.axisvalueslabel)
         self.axisvalueslabel.show()
+        self.slotUpdateAxisValues(None)
+
+        # a label for the page number readout
+        self.pagelabel = qt4.QLabel(statusbar)
+        statusbar.addPermanentWidget(self.pagelabel)
+        self.pagelabel.show()
 
         # working directory - use previous one
         self.dirname = setdb.get('dirname', qt4.QDir.homePath())
@@ -150,7 +181,11 @@ class MainWindow(qt4.QMainWindow):
                       self.slotUpdatePage )
         self.connect( self.plot, qt4.SIGNAL("sigAxisValuesFromMouse"),
                       self.slotUpdateAxisValues )
-        
+        self.connect( self.plot, qt4.SIGNAL("sigPickerEnabled"),
+                      self.slotPickerEnabled )
+        self.connect( self.plot, qt4.SIGNAL("sigPointPicked"),
+                      self.slotUpdatePickerLabel )
+
         # disable save if already saved
         self.connect( self.document, qt4.SIGNAL("sigModified"),
                       self.slotModifiedDoc )
@@ -161,8 +196,8 @@ class MainWindow(qt4.QMainWindow):
         # if a widget in the plot window is clicked by the user
         self.connect( self.plot, qt4.SIGNAL("sigWidgetClicked"),
                       self.treeedit.selectWidget )
-        self.connect( self.treeedit, qt4.SIGNAL("widgetSelected"),
-                      self.plot.selectedWidget )
+        self.connect( self.treeedit, qt4.SIGNAL("widgetsSelected"),
+                      self.plot.selectedWidgets )
 
         # enable/disable undo/redo
         self.connect(self.menus['edit'], qt4.SIGNAL('aboutToShow()'),
@@ -267,6 +302,7 @@ class MainWindow(qt4.QMainWindow):
         """Undo the previous operation"""
         if self.document.canUndo():
             self.document.undoOperation()
+        self.treeedit.checkWidgetSelected()
         
     def slotEditRedo(self):
         """Redo the previous operation"""
@@ -405,6 +441,10 @@ class MainWindow(qt4.QMainWindow):
             'view.console':
                 a(self, 'Show or hide console window', 'Console window',
                   None, checkable=True),
+            'view.datanav':
+                a(self, 'Show or hide data navigator window', 'Data navigator window',
+                  None, checkable=True),
+
             'view.maintool':
                 a(self, 'Show or hide main toolbar', 'Main toolbar',
                   None, checkable=True),
@@ -452,6 +492,9 @@ class MainWindow(qt4.QMainWindow):
             'help.bug':
                 a(self, 'Report a bug on the internet',
                   'Suggestions and bugs', self.slotHelpBug),
+            'help.tutorial':
+                a(self, 'An interactive Veusz tutorial',
+                  'Tutorial', self.slotHelpTutorial),
             'help.about':
                 a(self, 'Displays information about the program', 'About...',
                   self.slotHelpAbout, icon='veusz')
@@ -491,12 +534,14 @@ class MainWindow(qt4.QMainWindow):
         editmenu = [
             'edit.undo', 'edit.redo',
             '',
+            ['edit.select', '&Select', []],
+            '',
             'edit.prefs', 'edit.stylesheet', 'edit.custom',
             ''
             ]
         viewwindowsmenu = [
             'view.edit', 'view.props', 'view.format',
-            'view.console',
+            'view.console', 'view.datanav',
             '',
             'view.maintool', 'view.viewtool',
             'view.addtool', 'view.edittool'
@@ -521,6 +566,10 @@ class MainWindow(qt4.QMainWindow):
         helpmenu = [
             'help.home', 'help.project', 'help.bug',
             '',
+            'help.tutorial',
+            '',
+            ['help.examples', '&Example documents', []],
+            '',
             'help.about'
             ]
 
@@ -541,6 +590,28 @@ class MainWindow(qt4.QMainWindow):
         self.menus = {}
         utils.constructMenus(self.menuBar(), self.menus, menus, self.vzactions)
 
+        self.populateExamplesMenu()
+
+    def _setPickerFont(self, label):
+        f = label.font()
+        f.setBold(True)
+        f.setPointSizeF(f.pointSizeF() * 1.2)
+        label.setFont(f)
+
+    def populateExamplesMenu(self):
+        """Add examples to help menu."""
+
+        examples = glob.glob(os.path.join(utils.exampleDirectory, '*.vsz'))
+        menu = self.menus["help.examples"]
+        for ex in sorted(examples):
+            name = os.path.splitext(os.path.basename(ex))[0]
+
+            def _openexample(ex=ex):
+                MainWindow.CreateWindow(ex)
+
+            a = menu.addAction(name, _openexample)
+            a.setStatusTip("Open %s example document" % name)
+
     def defineViewWindowMenu(self):
         """Setup View -> Window menu."""
 
@@ -557,6 +628,7 @@ class MainWindow(qt4.QMainWindow):
                          (self.propdock, 'view.props'),
                          (self.formatdock, 'view.format'),
                          (self.console, 'view.console'),
+                         (self.datadock, 'view.datanav'),
                          (self.maintoolbar, 'view.maintool'),
                          (self.datatoolbar, 'view.datatool'),
                          (self.treeedit.edittoolbar, 'view.edittool'),
@@ -584,6 +656,7 @@ class MainWindow(qt4.QMainWindow):
         self.connect(dialog, qt4.SIGNAL('dialogFinished'), self.deleteDialog)
         self.dialogs.append(dialog)
         dialog.show()
+        self.emit( qt4.SIGNAL('dialogShown'), dialog )
 
     def deleteDialog(self, dialog):
         """Remove dialog from list of dialogs."""
@@ -599,10 +672,15 @@ class MainWindow(qt4.QMainWindow):
         self.showDialog(dialog)
         return dialog
 
-    def slotDataEdit(self):
-        """Edit existing datasets."""
+    def slotDataEdit(self, editdataset=None):
+        """Edit existing datasets.
+
+        If editdataset is set to a dataset name, edit this dataset
+        """
         dialog = dataeditdialog.DataEditDialog(self, self.document)
         self.showDialog(dialog)
+        if editdataset is not None:
+            dialog.selectDataset(editdataset)
         return dialog
 
     def slotDataCreate(self):
@@ -648,6 +726,32 @@ class MainWindow(qt4.QMainWindow):
         qt4.QDesktopServices.openUrl(
             qt4.QUrl('https://gna.org/bugs/?group=veusz') )
 
+    def askTutorial(self):
+        """Ask if tutorial wanted."""
+        retn = qt4.QMessageBox.question(
+            self, "Veusz Tutorial",
+            "Veusz includes a tutorial to help get you started.\n"
+            "Would you like to start the tutorial now?\n"
+            "If not, you can access it later through the Help menu.",
+            qt4.QMessageBox.Yes | qt4.QMessageBox.No
+            )
+
+        if retn == qt4.QMessageBox.Yes:
+            self.slotHelpTutorial()
+
+    def slotHelpTutorial(self):
+        """Show a Veusz tutorial."""
+        if self.document.isBlank():
+            # run the tutorial
+            from veusz.windows.tutorial import TutorialDock
+            tutdock = TutorialDock(self.document, self, self)
+            self.addDockWidget(qt4.Qt.RightDockWidgetArea, tutdock)
+            tutdock.show()
+        else:
+            # open up a blank window for tutorial
+            win = self.CreateWindow()
+            win.slotHelpTutorial()
+
     def slotHelpAbout(self):
         """Show about dialog."""
         AboutDialog(self).exec_()
@@ -688,8 +792,8 @@ class MainWindow(qt4.QMainWindow):
                 self.slotFileSave()
 
         # store working directory
-        setdb['dirname'] = unicode(self.dirname)
-        setdb['dirname_export'] = unicode(self.dirname_export)
+        setdb['dirname'] = self.dirname
+        setdb['dirname_export'] = self.dirname_export
 
         # store the current geometry in the settings database
         geometry = ( self.x(), self.y(), self.width(), self.height() )
@@ -699,6 +803,9 @@ class MainWindow(qt4.QMainWindow):
         data = str(self.saveState())
         setdb['geometry_mainwindowstate'] = data
 
+        # save current setting db
+        setdb.writeSettings()
+
         event.accept()
 
     def setupWindowGeometry(self):
@@ -756,6 +863,11 @@ class MainWindow(qt4.QMainWindow):
             self.setWindowTitle( "%s - Veusz" %
                                  os.path.basename(self.filename) )
 
+    def plotQueueChanged(self, incr):
+        self.plotqueuecount += incr
+        text = u'•' * self.plotqueuecount
+        self.plotqueuelabel.setText(text)
+
     def _fileSaveDialog(self, filetype, filedescr, dialogtitle):
         """A generic file save dialog for exporting / saving."""
         
@@ -768,7 +880,7 @@ class MainWindow(qt4.QMainWindow):
         # okay was selected (and is okay to overwrite if it exists)
         if fd.exec_() == qt4.QDialog.Accepted:
             # save directory for next time
-            self.dirname = fd.directory()
+            self.dirname = fd.directory().absolutePath()
             # update the edit box
             filename = unicode( fd.selectedFiles()[0] )
             if os.path.splitext(filename)[1] == '':
@@ -789,7 +901,7 @@ class MainWindow(qt4.QMainWindow):
         # if the user chooses a file
         if fd.exec_() == qt4.QDialog.Accepted:
             # save directory for next time
-            self.dirname = fd.directory()
+            self.dirname = fd.directory().absolutePath()
 
             filename = unicode( fd.selectedFiles()[0] )
             try:
@@ -966,6 +1078,8 @@ class MainWindow(qt4.QMainWindow):
             self.dirname = os.path.dirname( os.path.abspath(filename) )
             self.dirname_export = self.dirname
 
+        # notify cmpts which need notification that doc has finished opening
+        self.emit(qt4.SIGNAL("documentopened"))
         qt4.QApplication.restoreOverrideCursor()
 
     def addRecentFile(self, filename):
@@ -1030,41 +1144,37 @@ class MainWindow(qt4.QMainWindow):
         # File types we can export to in the form ([extensions], Name)
         fd = qt4.QFileDialog(self, 'Export page')
         fd.setDirectory( self.dirname_export )
-            
+
         fd.setFileMode( qt4.QFileDialog.AnyFile )
         fd.setAcceptMode( qt4.QFileDialog.AcceptSave )
 
         # Create a mapping between a format string and extensions
         filtertoext = {}
+        # convert extensions to filter
+        exttofilter = {}
         filters = []
         # a list of extensions which are allowed
         validextns = []
-        formats = self.document.getExportFormats()
+        formats = document.Export.formats
         for extns, name in formats:
             extensions = " ".join(["*." + item for item in extns])
             # join eveything together to make a filter string
             filterstr = '%s (%s)' % (name, extensions)
             filtertoext[filterstr] = extns
+            for e in extns:
+                exttofilter[e] = filterstr
             filters.append(filterstr)
             validextns += extns
-
-        try:
-            # Qt >= 4.4 (reqd for Fedora 12 Qt 4.6)
-            fd.setNameFilters(filters)
-        except AttributeError:
-            fd.setFilters(filters)
+        fd.setNameFilters(filters)
 
         # restore last format if possible
         try:
             filt = setdb['export_lastformat']
-            try:
-                # Qt >= 4.4 (reqd for Fedora 12 Qt 4.6)
-                fd.selectNameFilter(filt)
-            except AttributeError:
-                fd.selectFilter(filt)
+            fd.selectNameFilter(filt)
             extn = formats[filters.index(filt)][0][0]
         except (KeyError, IndexError, ValueError):
-            extn = 'eps'
+            extn = 'pdf'
+            fd.selectNameFilter( exttofilter[extn] )
 
         if self.filename:
             # try to convert current filename to export name
@@ -1074,7 +1184,7 @@ class MainWindow(qt4.QMainWindow):
         
         if fd.exec_() == qt4.QDialog.Accepted:
             # save directory for next time
-            self.dirname_export = unicode(fd.directory().absolutePath())
+            self.dirname_export = fd.directory().absolutePath()
 
             filterused = str(fd.selectedFilter())
             setdb['export_lastformat'] = filterused
@@ -1090,19 +1200,23 @@ class MainWindow(qt4.QMainWindow):
             # this is the extension without the dot
             ext = os.path.splitext(filename)[1][1:]
             if (ext not in validextns) and (ext not in chosenextns):
-                filename = filename + "." + chosenextns[0]
+                filename += "." + chosenextns[0]
 
+            e = document.Export( self.document,
+                                 filename,
+                                 self.plot.getPageNumber(),
+                                 bitmapdpi=setdb['export_DPI'],
+                                 pdfdpi=setdb['export_DPI_PDF'],
+                                 antialias=setdb['export_antialias'],
+                                 color=setdb['export_color'],
+                                 quality=setdb['export_quality'],
+                                 backcolor=setdb['export_background'] )
             try:
-                self.document.export(filename, self.plot.getPageNumber(),
-                                     dpi=setdb['export_DPI'],
-                                     antialias=setdb['export_antialias'],
-                                     color=setdb['export_color'],
-                                     quality=setdb['export_quality'],
-                                     backcolor=setdb['export_background'])
+                e.export()
             except (IOError, RuntimeError), inst:
                 qt4.QMessageBox.critical(self, "Veusz",
                                          "Error exporting file:\n%s" % inst)
-                
+
             # restore the cursor
             qt4.QApplication.restoreOverrideCursor()
 
@@ -1182,6 +1296,40 @@ class MainWindow(qt4.QMainWindow):
         else:
             self.axisvalueslabel.setText('No position')
 
+    def slotPickerEnabled(self, enabled):
+        if enabled:
+            self.pickerlabel.setText('No point selected')
+            self.pickerlabel.show()
+        else:
+            self.pickerlabel.hide()
+
+    def slotUpdatePickerLabel(self, info):
+        """Display the picked point"""
+        xv, yv = info.coords
+        xn, yn = info.labels
+        xt, yt = info.displaytype
+        ix = str(info.index)
+        if ix:
+            ix = '[' + ix + ']'
+
+        # format values for display
+        def fmt(val, dtype):
+            if dtype == 'date':
+                return utils.dateFloatToString(val)
+            elif dtype == 'numeric':
+                return '%0.5g' % val
+            elif dtype == 'text':
+                return val
+            else:
+                raise RuntimeError
+
+        xtext = fmt(xv, xt)
+        ytext = fmt(yv, yt)
+
+        t = '%s: %s%s = %s, %s%s = %s' % (
+                info.widget.name, xn, ix, xtext, yn, ix, ytext)
+        self.pickerlabel.setText(t)
+
     def slotAllowedImportsDoc(self, module, names):
         """Are allowed imports?"""
 
